@@ -37,28 +37,28 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "nsAccessible.h"
 #include "nsBaseWidgetAccessible.h"
-#include "nsCOMPtr.h"
+#include "nsAccessibleWrap.h"
 #include "nsGUIEvent.h"
-#include "nsIContent.h"
-#include "nsIDOMElement.h"
-#include "nsIDOMEventReceiver.h"
 #include "nsIFrame.h"
 #include "nsILink.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
 #include "nsISelection.h"
 #include "nsISelectionController.h"
+#include "nsIServiceManager.h"
+#include "nsIAccessibilityService.h"
 
 // ------------
 // nsBlockAccessible
 // ------------
 
-nsBlockAccessible::nsBlockAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):nsAccessible(aNode, aShell)
+nsBlockAccessible::nsBlockAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):nsAccessibleWrap(aNode, aShell)
 {
 }
 
+NS_IMPL_ISUPPORTS_INHERITED0(nsBlockAccessible, nsAccessible)
+
 /* nsIAccessible accGetAt (in long x, in long y); */
 NS_IMETHODIMP nsBlockAccessible::AccGetAt(PRInt32 tx, PRInt32 ty, nsIAccessible **_retval)
 {
@@ -118,10 +118,12 @@ NS_IMETHODIMP nsBlockAccessible::AccGetAt(PRInt32 tx, PRInt32 ty, nsIAccessible
   * nsContainerAccessible
   */
 nsContainerAccessible::nsContainerAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsAccessible(aNode, aShell)
+nsAccessibleWrap(aNode, aShell)
 {
 }
 
+NS_IMPL_ISUPPORTS_INHERITED0(nsContainerAccessible, nsAccessible)
+
 /** no actions */
 NS_IMETHODIMP nsContainerAccessible::GetAccNumActions(PRUint8 *_retval)
 {
@@ -166,10 +168,12 @@ NS_IMETHODIMP nsContainerAccessible::GetAccName(nsAString& _retval)
 //-------------
 
 nsLeafAccessible::nsLeafAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsAccessible(aNode, aShell)
+nsAccessibleWrap(aNode, aShell)
 {
 }
 
+NS_IMPL_ISUPPORTS_INHERITED0(nsLeafAccessible, nsAccessible)
+
 /* nsIAccessible getAccFirstChild (); */
 NS_IMETHODIMP nsLeafAccessible::GetAccFirstChild(nsIAccessible **_retval)
 {
@@ -197,19 +201,16 @@ NS_IMETHODIMP nsLeafAccessible::GetAccChildCount(PRInt32 *_retval)
 //----------------
 
 nsLinkableAccessible::nsLinkableAccessible(nsIDOMNode* aNode, nsIWeakReference* aShell):
-nsAccessible(aNode, aShell), mIsALinkCached(PR_FALSE), mLinkContent(nsnull), mIsLinkVisited(PR_FALSE)
+nsAccessibleWrap(aNode, aShell), mIsALinkCached(PR_FALSE), mLinkContent(nsnull), mIsLinkVisited(PR_FALSE)
 { 
 }
 
+NS_IMPL_ISUPPORTS_INHERITED0(nsLinkableAccessible, nsAccessible)
+
 NS_IMETHODIMP nsLinkableAccessible::AccTakeFocus()
 { 
   if (IsALink()) {
-    nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
-    if (!shell)
-      return NS_ERROR_FAILURE;  
-    nsCOMPtr<nsIPresContext> context;
-    shell->GetPresContext(getter_AddRefs(context));
-    mLinkContent->SetFocus(context);
+    mLinkContent->SetFocus(nsCOMPtr<nsIPresContext>(GetPresContext()));
   }
   
   return NS_OK;
@@ -227,7 +228,7 @@ NS_IMETHODIMP nsLinkableAccessible::GetAccState(PRUint32 *_retval)
   }
   
   // Get current selection and find out if current node is in it
-  nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
+  nsCOMPtr<nsIPresShell> shell(GetPresShell());
   if (!shell) {
      return NS_ERROR_FAILURE;  
   }
@@ -275,7 +276,7 @@ NS_IMETHODIMP nsLinkableAccessible::GetAccValue(nsAString& _retval)
 {
   if (IsALink()) {
     nsCOMPtr<nsIDOMNode> linkNode(do_QueryInterface(mLinkContent));
-    nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mPresShell));
+    nsCOMPtr<nsIPresShell> presShell(do_QueryReferent(mWeakShell));
     if (linkNode && presShell)
       return presShell->GetLinkLocation(linkNode, _retval);
   }
@@ -310,12 +311,7 @@ NS_IMETHODIMP nsLinkableAccessible::AccDoAction(PRUint8 index)
   // Action 0 (default action): Jump to link
   if (index == eAction_Jump) {
     if (IsALink()) {
-      nsCOMPtr<nsIPresShell> shell(do_QueryReferent(mPresShell));
-      if (!shell)
-        return NS_ERROR_FAILURE;  
-
-      nsCOMPtr<nsIPresContext> presContext;
-      shell->GetPresContext(getter_AddRefs(presContext));
+      nsCOMPtr<nsIPresContext> presContext(GetPresContext());
       if (presContext) {
         nsMouseEvent linkClickEvent;
         linkClickEvent.eventStructType = NS_EVENT;
@@ -340,6 +336,25 @@ NS_IMETHODIMP nsLinkableAccessible::AccDoAction(PRUint8 index)
   return NS_ERROR_INVALID_ARG;
 }
 
+NS_IMETHODIMP nsLinkableAccessible::GetAccKeyboardShortcut(nsAString& _retval)
+{
+  if (IsALink()) {
+    nsresult rv;
+    nsCOMPtr<nsIDOMNode> linkNode(do_QueryInterface(mLinkContent));
+    if (linkNode && mDOMNode != linkNode) {
+      nsCOMPtr<nsIAccessible> linkAccessible;
+      nsCOMPtr<nsIAccessibilityService> accService = 
+        do_GetService("@mozilla.org/accessibilityService;1");
+      rv = accService->GetAccessibleInWeakShell(linkNode, mWeakShell,
+                                                getter_AddRefs(linkAccessible));
+      if (NS_SUCCEEDED(rv) && linkAccessible)
+        return linkAccessible->GetAccKeyboardShortcut(_retval);
+      else
+        return rv;
+    }
+  }
+  return nsAccessible::GetAccKeyboardShortcut(_retval);;
+}
 
 PRBool nsLinkableAccessible::IsALink()
 {
@@ -368,72 +383,8 @@ PRBool nsLinkableAccessible::IsALink()
   return PR_FALSE;
 }
 
-// ------------
-// nsMenuListenerAccessible
-// ------------
-
-NS_IMPL_ISUPPORTS_INHERITED1(nsMenuListenerAccessible, nsAccessible, nsIDOMXULListener)
-
-nsMenuListenerAccessible::nsMenuListenerAccessible(nsIDOMNode* aDOMNode, nsIWeakReference* aShell):
-nsAccessible(aDOMNode, aShell)
+NS_IMETHODIMP nsLinkableAccessible::Shutdown()
 {
-  mRegistered = PR_FALSE;
-  mOpen = PR_FALSE;
+  mLinkContent = nsnull;
+  return nsAccessibleWrap::Shutdown();
 }
-
-nsMenuListenerAccessible::~nsMenuListenerAccessible()
-{
-  if (mRegistered) {
-     nsCOMPtr<nsIDOMEventReceiver> eventReceiver(do_QueryInterface(mDOMNode));
-     if (eventReceiver) 
-       eventReceiver->RemoveEventListener(NS_LITERAL_STRING("popupshowing"), this, PR_TRUE);   
-  }
-}
-
-NS_IMETHODIMP nsMenuListenerAccessible::PopupShowing(nsIDOMEvent* aEvent)
-{ 
-  mOpen = PR_TRUE;
-
-  /* TBD send state change event */ 
-
-  return NS_OK; 
-}
-
-NS_IMETHODIMP nsMenuListenerAccessible::PopupHiding(nsIDOMEvent* aEvent)
-{ 
-  mOpen = PR_FALSE;
-
-  /* TBD send state change event */ 
-
-  return NS_OK; 
-}
-
-NS_IMETHODIMP nsMenuListenerAccessible::Close(nsIDOMEvent* aEvent)
-{ 
-  mOpen = PR_FALSE;
-
-  /* TBD send state change event */ 
-
-  return NS_OK; 
-}
-
-void
-nsMenuListenerAccessible::SetupMenuListener()
-{
-  // if not already one, register ourselves as a popup listener
-  if (!mRegistered) {
-     nsCOMPtr<nsIDOMEventReceiver> eventReceiver(do_QueryInterface(mDOMNode));
-     if (!eventReceiver) {
-       return;
-     }
-
-     nsresult rv = eventReceiver->AddEventListener(NS_LITERAL_STRING("popupshowing"), this, PR_TRUE);   
-
-     if (NS_FAILED(rv)) {
-       return;
-     }
-
-     mRegistered = PR_TRUE;
-  }
-}
-
