@@ -40,13 +40,124 @@
                              
 icaltimetype ConvertFromPrtime( PRTime indate );
 PRTime ConvertToPrtime ( icaltimetype indate );
+extern "C" {
+extern icalarray *builtin_timezones;
+extern char *gDefaultTzidPrefix;
+}
+//icaltimetype ConvertFromPrtime( PRTime indate );
+void icaltimezone_init_mozilla_zones (void)
+{
+    gDefaultTzidPrefix="/Mozilla.org/";
+
+    char timezonecalstr[] = \
+"BEGIN:VCALENDAR\n\
+PRODID:-//Mozilla.org/NONSGML Mozilla Calendar Timezone Table V1.0//EN\n\
+VERSION:2.0\n\
+END:VCALENDAR\n\
+";
+
+    char *timezoneformatstr = "BEGIN:VTIMEZONE\n\
+TZID:/Mozilla.org/BasicTimezones/NH-GMT%c%02d:%02d\n\
+LOCATION:NH-GMT%c%02d:%02d\n\
+BEGIN:STANDARD\n\
+TZOFFSETFROM:%c%02d%02d\n\
+TZOFFSETTO:%c%02d%02d\n\
+TZNAME:NHS-GMT%c%02d:%02d\n\
+DTSTART:19991031T020000\n\
+RRULE:FREQ=YEARLY;BYMONTH=10;BYDAY=-1SU\n\
+END:STANDARD\n\
+BEGIN:DAYLIGHT\n\
+TZOFFSETFROM:%c%02d%02d\n\
+TZOFFSETTO:%c%02d%02d\n\
+TZNAME:NHD-GMT%c%02d:%02d\n\
+DTSTART:20000402T020000\n\
+RRULE:FREQ=YEARLY;BYMONTH=4;BYMONTHDAY=1,2,3,4,5,6,7;BYDAY=SU\n\
+END:DAYLIGHT\n\
+END:VTIMEZONE\n";
+
+    char *zulutimezonestr = "BEGIN:VTIMEZONE\n\
+TZID:/Mozilla.org/BasicTimezones/GMT\n\
+LOCATION:GMT\n\
+END:VTIMEZONE\n";
+
+    char timezonestr[1024];
+
+    if( builtin_timezones )
+        return;
+
+    builtin_timezones = icalarray_new ( 44, 32); //HARDCODED sizeof(icaltimezone)=44
+
+    icalcomponent *vcalendar = icalparser_parse_string(timezonecalstr);
+    icalcomponent *vtimezone;
+
+    //first add the UTC timezone
+    vtimezone = icalcomponent_new_from_string( zulutimezonestr );
+    icalcomponent_add_component( vcalendar, vtimezone );
+
+    //Components should be sorted by location
+    //+00:00 to +12:00
+    int i;
+    for( i=0; i<25; i++ ) {
+        char sign = '+';
+        int hour = i/2;
+        int minute = i%2==0 ? 00 : 30;
+        int hour2 = (i+2)/2;
+        sprintf( timezonestr, timezoneformatstr
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign, hour2, minute
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign, hour2, minute
+                 , sign, hour, minute );
+        vtimezone = icalcomponent_new_from_string( timezonestr );
+        icalcomponent_add_component( vcalendar, vtimezone );
+    }
+    //-00:30 to -12:00
+    for( i=-1; i>-25; i-- ) {
+        char sign = '-';
+        int hour = -1*i/2;
+        int minute = i%2==0 ? 00 : 30;
+        char sign2 = (i+2)<0 ? '-' : '+';
+        int hour2 = (i+2)<0 ? -1*(i+2)/2 : (i+2)/2;
+        sprintf( timezonestr, timezoneformatstr
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign2, hour2, minute
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign, hour, minute
+                 , sign2, hour2, minute
+                 , sign, hour, minute );
+        vtimezone = icalcomponent_new_from_string( timezonestr );
+        icalcomponent_add_component( vcalendar, vtimezone );
+    }
+
+    for( vtimezone = icalcomponent_get_first_component( vcalendar, ICAL_VTIMEZONE_COMPONENT );
+         vtimezone != NULL;
+         vtimezone = icalcomponent_get_next_component( vcalendar, ICAL_VTIMEZONE_COMPONENT ) ) {
+        icaltimezone *zone=icaltimezone_new();
+        icaltimezone_get_vtimezone_properties  ( zone, vtimezone );
+        icalarray_append (builtin_timezones, zone);
+    }
+
+    PRExplodedTime ext;
+    PR_ExplodeTime( PR_Now(), PR_LocalTimeParameters, &ext);
+    int gmtoffsethour = ext.tm_params.tp_gmt_offset < 0 ? -1*ext.tm_params.tp_gmt_offset / 3600 : ext.tm_params.tp_gmt_offset / 3600;
+    int gmtoffsetminute = ext.tm_params.tp_gmt_offset%3600 ? 30 : 00;
+    char zone_location[20];
+    sprintf( zone_location, "NH-GMT%c%02d:%02d", ext.tm_params.tp_gmt_offset < 0 ? '-' : '+'
+                                                , gmtoffsethour, gmtoffsetminute );
+
+    currenttimezone = icaltimezone_get_builtin_timezone ( zone_location );
+}
 
 oeICalContainerImpl::oeICalContainerImpl()
 {
 #ifdef ICAL_DEBUG
     printf( "oeICalContainerImpl::oeICalContainerImpl()\n" );
 #endif
-        NS_INIT_ISUPPORTS();
 
         m_batchMode = false;
 
@@ -74,6 +185,8 @@ oeICalContainerImpl::oeICalContainerImpl()
         m_filter = new oeICalContainerFilter();
         NS_ADDREF( m_filter );
         m_filter->m_calendarArray = m_calendarArray;
+
+        icaltimezone_init_mozilla_zones ();
 }
 
 oeICalContainerImpl::~oeICalContainerImpl()
@@ -82,6 +195,7 @@ oeICalContainerImpl::~oeICalContainerImpl()
     printf( "oeICalContainerImpl::~oeICalContainerImpl()\n" );
 #endif
 
+    m_calendarArray->Clear();
     m_calendarArray = nsnull;
     m_observerArray = nsnull;
     m_todoobserverArray = nsnull;
@@ -149,12 +263,13 @@ oeICalContainerImpl::GetCalendar( const char *server, oeIICal **calendar ) {
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* tmpcalendar;
+        nsCOMPtr<oeIICal> tmpcalendar;
         char *tmpserver;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&tmpcalendar );
+        m_calendarArray->GetElementAt( i, getter_AddRefs( tmpcalendar ) );
         tmpcalendar->GetServer( &tmpserver );
         if( strcmp( tmpserver, server ) == 0 ) {
             *calendar = tmpcalendar;
+            NS_ADDREF( *calendar );
             nsMemory::Free( tmpserver );
             return NS_OK;
         }
@@ -175,9 +290,9 @@ oeICalContainerImpl::RemoveCalendar( const char *server ) {
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* tmpcalendar;
+        nsCOMPtr<oeIICal> tmpcalendar;
         char *tmpserver;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&tmpcalendar );
+        m_calendarArray->GetElementAt( i, getter_AddRefs( tmpcalendar ) );
         tmpcalendar->GetServer( &tmpserver );
         if( strcmp( tmpserver, server ) == 0 ) {
             nsMemory::Free( tmpserver );
@@ -223,8 +338,8 @@ NS_IMETHODIMP oeICalContainerImpl::SetBatchMode(PRBool aBatchMode)
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeIICal* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
             calendar->SetBatchMode( aBatchMode );
         }
 
@@ -288,11 +403,11 @@ NS_IMETHODIMP oeICalContainerImpl::AddEvent( oeIICalEvent *icalevent, const char
     printf( "oeICalContainerImpl::AddEvent()\n" );
 #endif
     nsresult rv;
-    oeIICal *calendar;
-    GetCalendar( server , &calendar );
+    nsCOMPtr<oeIICal> calendar;
+    GetCalendar( server , getter_AddRefs(calendar) );
     if( !calendar ) {
         AddCalendar( server );
-        GetCalendar( server , &calendar );
+        GetCalendar( server , getter_AddRefs(calendar) );
         if( !calendar ) {
             #ifdef ICAL_DEBUG
             printf( "oeICalContainerImpl::AddEvent()-Error cannot find or create calendar\n" );
@@ -322,6 +437,7 @@ NS_IMETHODIMP oeICalContainerImpl::ModifyEvent( oeIICalEvent *icalevent, char **
         return NS_ERROR_FAILURE;
     }
     nsresult rv = calendar->ModifyEvent( icalevent, retid );
+    NS_RELEASE( calendar );
     return rv;
 }
 
@@ -346,8 +462,8 @@ NS_IMETHODIMP oeICalContainerImpl::FetchEvent( const char *id, oeIICalEvent **ev
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->FetchEvent( id, ev );
         if( *ev )
             break;
@@ -386,6 +502,7 @@ oeICalContainerImpl::DeleteEvent( const char *id )
         return NS_ERROR_FAILURE;
     }
     nsresult rv = calendar->DeleteEvent( id );
+    NS_RELEASE( calendar );
 
 	return rv;
 }
@@ -409,8 +526,8 @@ oeICalContainerImpl::GetAllEvents(nsISimpleEnumerator **resultList )
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetAllEvents( (nsISimpleEnumerator **)&eventEnum );
     }
 
@@ -435,8 +552,8 @@ oeICalContainerImpl::GetEventsForMonth( PRTime datems, nsISimpleEnumerator **eve
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetEventsForMonth( datems, (nsISimpleEnumerator **)&eventEnum );
     }
 
@@ -461,8 +578,8 @@ oeICalContainerImpl::GetEventsForWeek( PRTime datems, nsISimpleEnumerator **even
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetEventsForWeek( datems, (nsISimpleEnumerator **)&eventEnum );
     }
     return NS_OK;
@@ -487,8 +604,8 @@ oeICalContainerImpl::GetEventsForDay( PRTime datems, nsISimpleEnumerator **event
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetEventsForDay( datems, (nsISimpleEnumerator **)&eventEnum );
     }
     return NS_OK;
@@ -513,8 +630,8 @@ oeICalContainerImpl::GetEventsForRange( PRTime checkdateinms, PRTime checkenddat
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetEventsForRange( checkdateinms, checkenddateinms, (nsISimpleEnumerator **)&eventEnum );
     }
 
@@ -540,8 +657,8 @@ oeICalContainerImpl::GetFirstEventsForRange( PRTime checkdateinms, PRTime checke
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->GetFirstEventsForRange( checkdateinms, checkenddateinms, (nsISimpleEnumerator **)&eventEnum );
     }
 
@@ -560,8 +677,9 @@ icaltimetype oeICalContainerImpl::GetNextEvent( icaltimetype starting ) {
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeICalImpl* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> tmpcalendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(tmpcalendar) );
+        oeICalImpl *calendar = (oeICalImpl *)(&tmpcalendar);
         icaltimetype next = calendar->GetNextEvent( starting );
         if( !icaltime_is_null_time( next ) && ( icaltime_is_null_time( soonest ) || (icaltime_compare( soonest, next ) > 0) ) ) {
             soonest = next;
@@ -598,8 +716,9 @@ oeICalContainerImpl::GetNextNEvents( PRTime datems, PRInt32 maxcount, nsISimpleE
             m_calendarArray->Count( &num );
             for( i=0; i<num && count<maxcount; i++ ) 
             {
-                oeICalImpl* calendar;
-                m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+                nsCOMPtr<oeIICal> tmpcalendar;
+                m_calendarArray->GetElementAt( i, getter_AddRefs(tmpcalendar) );
+                oeICalImpl *calendar = (oeICalImpl *)(&tmpcalendar);
                 EventList *tmplistptr = calendar->GetEventList();
                 while( tmplistptr && count<maxcount ) {
                     if( tmplistptr->event ) {
@@ -638,8 +757,8 @@ oeICalContainerImpl::AddObserver(oeIICalObserver *observer)
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeICalImpl* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs( calendar ) );
             calendar->AddObserver( observer );
         }
         observer->OnLoad();
@@ -659,8 +778,8 @@ oeICalContainerImpl::RemoveObserver(oeIICalObserver *observer)
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeICalImpl* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs( calendar ) );
             calendar->RemoveObserver( observer );
         }
         m_observerArray->RemoveElement( observer );
@@ -684,8 +803,8 @@ oeICalContainerImpl::AddTodoObserver(oeIICalTodoObserver *observer)
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeICalImpl* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs( calendar ) );
             calendar->AddTodoObserver( observer );
         }
         observer->OnLoad();
@@ -705,8 +824,8 @@ oeICalContainerImpl::RemoveTodoObserver(oeIICalTodoObserver *observer)
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeICalImpl* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs( calendar ) );
             calendar->RemoveTodoObserver( observer );
         }
         m_todoobserverArray->RemoveElement( observer );
@@ -721,11 +840,11 @@ NS_IMETHODIMP oeICalContainerImpl::AddTodo(oeIICalTodo *icaltodo, const char *se
     printf( "oeICalContainerImpl::AddTodo()\n" );
 #endif
     nsresult rv;
-    oeIICal *calendar;
-    GetCalendar(server , &calendar );
+    nsCOMPtr<oeIICal> calendar;
+    GetCalendar( server , getter_AddRefs(calendar) );
     if( !calendar ) {
         AddCalendar( server );
-        GetCalendar( server , &calendar );
+        GetCalendar( server , getter_AddRefs(calendar) );
         if( !calendar ) {
             #ifdef ICAL_DEBUG
             printf( "oeICalContainerImpl::AddTodo()-Error cannot find or create calendar\n" );
@@ -771,6 +890,7 @@ oeICalContainerImpl::DeleteTodo( const char *id )
         return NS_ERROR_FAILURE;
     }
     nsresult rv = calendar->DeleteTodo( id );
+    NS_RELEASE( calendar );
 
 	return rv;
 }
@@ -796,8 +916,8 @@ NS_IMETHODIMP oeICalContainerImpl::FetchTodo( const char *id, oeIICalTodo **ev)
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->FetchTodo( id, ev );
         if( *ev )
             break;
@@ -820,6 +940,7 @@ NS_IMETHODIMP oeICalContainerImpl::ModifyTodo(oeIICalTodo *icalevent, char **ret
         return NS_ERROR_FAILURE;
     }
     nsresult rv = calendar->ModifyTodo( icalevent, retid );
+    NS_RELEASE( calendar );
     return rv;
 }
 
@@ -842,8 +963,8 @@ oeICalContainerImpl::GetAllTodos(nsISimpleEnumerator **resultList )
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs( calendar ) );
         calendar->GetAllTodos( (nsISimpleEnumerator **)&eventEnum );
     }
     return NS_OK;
@@ -871,8 +992,8 @@ NS_IMETHODIMP oeICalContainerImpl::ResetFilter()
     m_calendarArray->Count( &num );
     for( i=0; i<num; i++ ) 
     {
-        oeIICal* calendar;
-        m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+        nsCOMPtr<oeIICal> calendar;
+        m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
         calendar->ResetFilter();
     }
     oeIDateTime *completed;
@@ -888,10 +1009,12 @@ oeICalContainerFilter::oeICalContainerFilter()
     m_completed = new oeFilterDateTime();
     m_completed->SetFieldType( ICAL_COMPLETED_PROPERTY );
     m_completed->m_parent = this;
+    NS_ADDREF( m_completed );
 }
 
 oeICalContainerFilter::~oeICalContainerFilter()
 {
+    NS_RELEASE( m_completed );
 }
 
 NS_IMETHODIMP oeICalContainerFilter::GetId(char **aRetVal)
@@ -1303,8 +1426,8 @@ void oeICalContainerFilter::UpdateAllFilters( PRInt32 icaltype )
         m_calendarArray->Count( &num );
         for( i=0; i<num; i++ ) 
         {
-            oeIICal* calendar;
-            m_calendarArray->GetElementAt( i, (nsISupports **)&calendar );
+            nsCOMPtr<oeIICal> calendar;
+            m_calendarArray->GetElementAt( i, getter_AddRefs(calendar) );
             oeIICalTodo *filter;
             calendar->GetFilter( &filter );
             oeIDateTime *completed;
@@ -1326,7 +1449,6 @@ NS_IMPL_ISUPPORTS1(oeFilterDateTime, oeIDateTime)
 
 oeFilterDateTime::oeFilterDateTime()
 {
-  NS_INIT_ISUPPORTS();
   m_datetime = icaltime_null_time();
 }
 
@@ -1447,3 +1569,14 @@ void oeFilterDateTime::SetFieldType( PRInt32 icaltype )
 {
     m_icaltype = icaltype;
 }
+
+NS_IMETHODIMP oeFilterDateTime::GetTzID(char **aRetVal)
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP oeFilterDateTime::SetTimeInTimezone( PRTime ms, const char *tzid )
+{
+    return NS_ERROR_NOT_IMPLEMENTED;
+}
+
