@@ -263,9 +263,6 @@ class nsScriptSecurityManager : public nsIScriptSecurityManager,
                                 public nsIObserver
 {
 public:
-    nsScriptSecurityManager();
-    virtual ~nsScriptSecurityManager();
-
     static void Shutdown();
     
     NS_DEFINE_STATIC_CID_ACCESSOR(NS_SCRIPTSECURITYMANAGER_CID)
@@ -285,12 +282,21 @@ public:
 
     JSContext* GetSafeJSContext();
 
+    static nsresult
+    SecurityCompareURIs(nsIURI* aSourceURI,
+                        nsIURI* aTargetURI,
+                        PRBool* result);
+
 private:
 
+    // GetScriptSecurityManager is the only call that can make one
+    nsScriptSecurityManager();
+    virtual ~nsScriptSecurityManager();
+
     static JSBool JS_DLL_CALLBACK
-    CheckJSFunctionCallerAccess(JSContext *cx, JSObject *obj,
-                                jsval id, JSAccessMode mode,
-                                jsval *vp);
+    CheckObjectAccess(JSContext *cx, JSObject *obj,
+                      jsval id, JSAccessMode mode,
+                      jsval *vp);
 
     static nsresult
     doGetObjectPrincipal(JSContext *cx, JSObject *obj, nsIPrincipal **result);
@@ -299,7 +305,8 @@ private:
     GetBaseURIScheme(nsIURI* aURI, char** aScheme);
 
     static nsresult 
-    ReportErrorToConsole(nsIURI* aTarget);
+    ReportError(JSContext* cx, const nsAString& messageTag,
+                nsIURI* aSource, nsIURI* aTarget);
 
     nsresult
     GetRootDocShell(JSContext* cx, nsIDocShell **result);
@@ -314,8 +321,9 @@ private:
                             void** aCachedClassPolicy);
 
     nsresult
-    CheckSameOrigin(JSContext* aCx, nsIPrincipal* aSubject, 
-                    nsIPrincipal* aObject, PRUint32 aAction);
+    CheckSameOriginDOMProp(nsIPrincipal* aSubject, 
+                           nsIPrincipal* aObject,
+                           PRUint32 aAction);
     
     PRInt32 
     GetSecurityLevel(nsIPrincipal *principal,
@@ -362,6 +370,9 @@ private:
     CheckXPCPermissions(nsISupports* aObj,
                         const char* aObjectSecurityLevel);
 
+    nsresult
+    Init();
+    
     nsresult
     InitPrefs();
 
@@ -379,13 +390,18 @@ private:
     InitPrincipals(PRUint32 prefCount, const char** prefNames,
                    nsISecurityPref* securityPref);
 
+#ifdef XPC_IDISPATCH_SUPPORT
+    // While this header is included outside of caps, this class isn't 
+    // referenced so this should be fine.
+    nsresult
+    CheckComponentPermissions(JSContext *cx, const nsCID &aCID);
+#endif
 #ifdef DEBUG_mstoltz
     void
     PrintPolicyDB();
 #endif
 
     // JS strings we need to clean up on shutdown
-    static jsval sCallerID;
     static jsval sEnabledID;
 
     inline void
@@ -409,6 +425,10 @@ private:
     nsCOMPtr<nsIThreadJSContextStack> mJSContextStack;
     PRBool mNameSetRegistered;
     PRBool mPolicyPrefsChanged;
+#ifdef XPC_IDISPATCH_SUPPORT    
+    PRBool mXPCDefaultGrantAll;
+    static const char* sXPCDefaultGrantAllName;
+#endif
 };
 
 #endif /*_NS_SCRIPT_SECURITY_MANAGER_H_*/
