@@ -71,9 +71,8 @@ NS_NewTreeWalker(nsIDOMNode *aRoot,
                                             aFilter,
                                             aEntityReferenceExpansion);
     NS_ENSURE_TRUE(walker, NS_ERROR_OUT_OF_MEMORY);
-    
-    return walker->QueryInterface(NS_GET_IID(nsIDOMTreeWalker),
-                                  (void**) aInstancePtrResult);
+
+    return CallQueryInterface(walker, aInstancePtrResult);
 }
 
 nsTreeWalker::nsTreeWalker(nsIDOMNode *aRoot,
@@ -87,7 +86,6 @@ nsTreeWalker::nsTreeWalker(nsIDOMNode *aRoot,
     mCurrentNode(aRoot),
     mPossibleIndexesPos(-1)
 {
-    NS_INIT_ISUPPORTS();
 
     NS_ASSERTION(aRoot, "invalid root in call to nsTreeWalker constructor");
 }
@@ -159,6 +157,11 @@ NS_IMETHODIMP nsTreeWalker::GetCurrentNode(nsIDOMNode * *aCurrentNode)
 NS_IMETHODIMP nsTreeWalker::SetCurrentNode(nsIDOMNode * aCurrentNode)
 {
     NS_ENSURE_TRUE(aCurrentNode, NS_ERROR_DOM_NOT_SUPPORTED_ERR);
+
+    nsresult rv = nsContentUtils::CheckSameOrigin(mRoot, aCurrentNode);
+    if(NS_FAILED(rv))
+        return rv;
+
     mCurrentNode = aCurrentNode;
     return NS_OK;
 }
@@ -638,7 +641,7 @@ nsresult nsTreeWalker::IndexOf(nsIDOMNode* aParent,
 
         if (possibleIndex >= 0) {
             nsCOMPtr<nsIContent> tmp;
-            contParent->ChildAt(possibleIndex, *getter_AddRefs(tmp));
+            docParent->ChildAt(possibleIndex, *getter_AddRefs(tmp));
             if (tmp == child) {
                 *_childNum = possibleIndex;
                 return NS_OK;
