@@ -42,7 +42,7 @@
 #include "nsIHTMLContent.h"
 #include "nsGenericHTMLElement.h"
 #include "nsHTMLAtoms.h"
-#include "nsIStyleContext.h"
+#include "nsIPresShell.h"
 #include "nsStyleConsts.h"
 #include "nsIPresContext.h"
 #include "nsIFormControl.h"
@@ -55,7 +55,6 @@
 #include "nsIEventStateManager.h"
 #include "nsIDOMEvent.h"
 #include "nsIDOMNSEvent.h"
-#include "nsISizeOfHandler.h"
 #include "nsIDocument.h"
 #include "nsGUIEvent.h"
 #include "nsUnicharUtils.h"
@@ -88,7 +87,7 @@ public:
   NS_DECL_NSIDOMNSHTMLBUTTONELEMENT
 
   // overrided nsIFormControl method
-  NS_IMETHOD GetType(PRInt32* aType);
+  NS_IMETHOD_(PRInt32) GetType() { return mType; }
   NS_IMETHOD Reset();
   NS_IMETHOD SubmitNamesValues(nsIFormSubmission* aFormSubmission,
                                nsIContent* aSubmitElement);
@@ -111,9 +110,6 @@ public:
                             nsIDOMEvent** aDOMEvent,
                             PRUint32 aFlags,
                             nsEventStatus* aEventStatus);
-#ifdef DEBUG
-  NS_IMETHOD SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const;
-#endif
 
 protected:
   PRInt8 mType;
@@ -377,7 +373,7 @@ nsHTMLButtonElement::RemoveFocus(nsIPresContext* aPresContext)
   return rv;
 }
 
-static nsGenericHTMLElement::EnumTable kButtonTypeTable[] = {
+static nsHTMLValue::EnumTable kButtonTypeTable[] = {
   { "button", NS_FORM_BUTTON_BUTTON },
   { "reset", NS_FORM_BUTTON_RESET },
   { "submit", NS_FORM_BUTTON_SUBMIT },
@@ -390,12 +386,12 @@ nsHTMLButtonElement::StringToAttribute(nsIAtom* aAttribute,
                                        nsHTMLValue& aResult)
 {
   if (aAttribute == nsHTMLAtoms::tabindex) {
-    if (ParseValue(aValue, 0, 32767, aResult, eHTMLUnit_Integer)) {
+    if (aResult.ParseIntWithBounds(aValue, eHTMLUnit_Integer, 0, 32767)) {
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
   else if (aAttribute == nsHTMLAtoms::type) {
-    nsGenericHTMLElement::EnumTable *table = kButtonTypeTable;
+    nsHTMLValue::EnumTable *table = kButtonTypeTable;
     nsAutoString val(aValue);
     while (nsnull != table->tag) { 
       if (val.EqualsIgnoreCase(table->tag)) {
@@ -421,7 +417,7 @@ nsHTMLButtonElement::AttributeToString(nsIAtom* aAttribute,
 {
   if (aAttribute == nsHTMLAtoms::type) {
     if (eHTMLUnit_Enumerated == aValue.GetUnit()) {
-      EnumValueToString(aValue, kButtonTypeTable, aResult);
+      aValue.EnumValueToString(kButtonTypeTable, aResult);
       return NS_CONTENT_ATTR_HAS_VALUE;
     }
   }
@@ -454,9 +450,7 @@ nsHTMLButtonElement::HandleDOMEvent(nsIPresContext* aPresContext,
     CallQueryInterface(formControlFrame, &formFrame);
 
     if (formFrame) {
-      const nsStyleUserInterface* uiStyle;
-      formFrame->GetStyleData(eStyleStruct_UserInterface,
-                              (const nsStyleStruct *&)uiStyle);
+      const nsStyleUserInterface* uiStyle = formFrame->GetStyleUserInterface();
 
       if (uiStyle->mUserInput == NS_STYLE_USER_INPUT_NONE ||
           uiStyle->mUserInput == NS_STYLE_USER_INPUT_DISABLED)
@@ -524,15 +518,6 @@ nsHTMLButtonElement::HandleDOMEvent(nsIPresContext* aPresContext,
       case NS_MOUSE_LEFT_CLICK:
         {
           if (mForm) {
-            if (mType == NS_FORM_BUTTON_SUBMIT) {
-              // tell the form to forget a possible pending submission.
-              // the reason is that the script returned true (the event was
-              // ignored) so if there is a stored submission, it will miss
-              // the name/value of the submitting element, thus we need
-              // to forget it and the form element will build a new one
-              mForm->ForgetPendingSubmission();
-            }
-
             if (mType == NS_FORM_BUTTON_SUBMIT || mType == NS_FORM_BUTTON_RESET) {
               nsFormEvent event;
               event.eventStructType = NS_FORM_EVENT;
@@ -635,24 +620,6 @@ nsHTMLButtonElement::HandleDOMEvent(nsIPresContext* aPresContext,
   return ret;
 }
 
-NS_IMETHODIMP
-nsHTMLButtonElement::GetType(PRInt32* aType)
-{
-  NS_ASSERTION(aType, "Null pointer bad!");
-  *aType = mType;
-  return NS_OK;
-}
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsHTMLButtonElement::SizeOf(nsISizeOfHandler* aSizer, PRUint32* aResult) const
-{
-  *aResult = sizeof(*this) + BaseSizeOf(aSizer);
-
-  return NS_OK;
-}
-#endif
-
 nsresult
 nsHTMLButtonElement::GetDefaultValue(nsAString& aDefaultValue)
 {
