@@ -52,6 +52,9 @@
 
  ******/
 
+// OUTPUT_CLASS=nsHTMLAtoms
+// MACRO_NAME=HTML_ATOM
+
 HTML_ATOM(_baseHref, NS_HTML_BASE_HREF)
 HTML_ATOM(_baseTarget, NS_HTML_BASE_TARGET)
 HTML_ATOM(a, "a")
@@ -72,9 +75,7 @@ HTML_ATOM(axis, "axis")
 HTML_ATOM(background, "background")
 HTML_ATOM(base, "base")
 HTML_ATOM(below, "below")
-#ifdef IBMBIDI
 HTML_ATOM(bdo, "bdo")
-#endif
 HTML_ATOM(bgcolor, "bgcolor")
 HTML_ATOM(big, "big")
 HTML_ATOM(blockquote, "blockquote")
