@@ -55,6 +55,8 @@
 
  ******/
 
+// OUTPUT_CLASS=nsXULAtoms
+// MACRO_NAME=XUL_ATOM
 
 XUL_ATOM(button, "button")
 XUL_ATOM(spinner, "spinner")
@@ -105,18 +107,6 @@ XUL_ATOM(progressNormal, "progressNormal")
 XUL_ATOM(progressUndetermined, "progressUndetermined")
 XUL_ATOM(progressNone, "progressNone")
 
-XUL_ATOM(moztreerow, ":-moz-tree-row")
-XUL_ATOM(moztreecell, ":-moz-tree-cell")
-XUL_ATOM(moztreecolumn, ":-moz-tree-column")
-XUL_ATOM(moztreecelltext, ":-moz-tree-cell-text")
-XUL_ATOM(moztreetwisty, ":-moz-tree-twisty")
-XUL_ATOM(moztreeindentation, ":-moz-tree-indentation")
-XUL_ATOM(moztreeline, ":-moz-tree-line")
-XUL_ATOM(moztreeimage, ":-moz-tree-image")
-XUL_ATOM(moztreeseparator, ":-moz-tree-separator")
-XUL_ATOM(moztreedropfeedback, ":-moz-tree-drop-feedback")
-XUL_ATOM(moztreeprogressmeter, ":-moz-tree-progressmeter")
-
 XUL_ATOM(menubar, "menubar") // An XP menu bar.
 XUL_ATOM(menu, "menu") // Represents an XP menu
 XUL_ATOM(menuitem, "menuitem") // Represents an XP menu item
@@ -179,11 +169,13 @@ XUL_ATOM(tabpanel, "tabpanel")
 XUL_ATOM(index, "index")
 XUL_ATOM(maxpos, "maxpos")
 XUL_ATOM(curpos, "curpos")
+XUL_ATOM(smooth, "smooth")
 XUL_ATOM(scrollbarbutton, "scrollbarbutton")
 XUL_ATOM(increment, "increment")
 XUL_ATOM(pageincrement, "pageincrement")
 XUL_ATOM(thumb, "thumb")
 XUL_ATOM(toggled, "toggled")
+XUL_ATOM(lazy, "lazy")
 XUL_ATOM(grippy, "grippy")
 XUL_ATOM(splitter, "splitter")
 XUL_ATOM(collapse, "collapse")
@@ -202,18 +194,8 @@ XUL_ATOM(columns, "columns")
 XUL_ATOM(row, "row")
 XUL_ATOM(column, "column")
 
-// toolbar & toolbar d&d atoms
-XUL_ATOM(ddDropLocation, "dd-droplocation")
-XUL_ATOM(ddDropLocationCoord, "dd-droplocationcoord")
-XUL_ATOM(ddDropOn, "dd-dropon")
-XUL_ATOM(ddTriggerRepaintSorted, "dd-triggerrepaintsorted")
-XUL_ATOM(ddTriggerRepaintRestore, "dd-triggerrepaintrestore")
-XUL_ATOM(ddTriggerRepaint, "dd-triggerrepaint")
-XUL_ATOM(ddNoDropBetweenRows, "dd-nodropbetweenrows")
 XUL_ATOM(container, "container")
 XUL_ATOM(leaf,"leaf")
-XUL_ATOM(ddDragDropArea, "dragdroparea")
-XUL_ATOM(ddDropMarker, ":-moz-drop-marker")
 
 XUL_ATOM(widget, "widget")
 XUL_ATOM(window, "window")
@@ -307,14 +289,21 @@ XUL_ATOM(titlebar, "titlebar")
 XUL_ATOM(resizer, "resizer")
 XUL_ATOM(dir, "dir")
 XUL_ATOM(properties, "properties")
+XUL_ATOM(resource, "resource")
 XUL_ATOM(sort, "sort")
 XUL_ATOM(sortDirection, "sortDirection")
 XUL_ATOM(sortActive, "sortActive")
+XUL_ATOM(sortResource, "sortResource")
+XUL_ATOM(sortResource2, "sortResource2")
+XUL_ATOM(sortSeparators, "sortSeparators")
+XUL_ATOM(sortStaticsLast, "sortStaticsLast")
 XUL_ATOM(selectedIndex, "selectedIndex")
+XUL_ATOM(staticHint, "staticHint")
 XUL_ATOM(_star, "*")
 XUL_ATOM(defaultz, "default")
 XUL_ATOM(screenX, "screenX")
 XUL_ATOM(screenY, "screenY")
+XUL_ATOM(type, "type")
 XUL_ATOM(hidechrome, "hidechrome")
 XUL_ATOM(popupset, "popupset")
 XUL_ATOM(parsetype, "parsetype")
