@@ -67,8 +67,7 @@
 
 #include "nsIXBLBinding.h"
 #include "nsIXBLDocumentInfo.h"
-#include "nsIXBLBindingAttachedHandler.h"
-#include "nsIXBLInsertionPoint.h"
+#include "nsXBLInsertionPoint.h"
 
 #include "nsIStyleSheet.h"
 #include "nsIHTMLStyleSheet.h"
@@ -77,7 +76,6 @@
 
 #include "nsIStyleRuleProcessor.h"
 #include "nsIStyleSet.h"
-#include "nsIXBLPrototypeBinding.h"
 #include "nsIWeakReference.h"
 
 #include "jsapi.h"
@@ -97,57 +95,45 @@
 // = nsAnonymousContentList 
 // ==================================================================
 
-// {CC949466-626E-4e3b-88EE-967DC5CEF7BF}
-#define NS_IANONYMOUSCONTENTLIST_IID \
-{ 0xcc949466, 0x626e, 0x4e3b, { 0x88, 0xee, 0x96, 0x7d, 0xc5, 0xce, 0xf7, 0xbf } }
-
-class nsIAnonymousContentList : public nsISupports
+class nsAnonymousContentList : public nsGenericDOMNodeList
 {
 public:
-  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IANONYMOUSCONTENTLIST_IID)
-
-   // nsIAnonymousContentList
-  NS_IMETHOD GetInsertionPointCount(PRUint32* aCount)=0;
-  NS_IMETHOD GetInsertionPointAt(PRUint32 i, nsIXBLInsertionPoint** aResult)=0;
-};
-
-class nsAnonymousContentList : public nsGenericDOMNodeList, public nsIAnonymousContentList
-{
-public:
-  nsAnonymousContentList(nsISupportsArray* aElements);
+  nsAnonymousContentList(nsVoidArray* aElements);
   virtual ~nsAnonymousContentList();
 
-  NS_DECL_ISUPPORTS_INHERITED
-
   // nsIDOMNodeList interface
   NS_DECL_NSIDOMNODELIST
 
-  // nsIAnonymousContentList
-  NS_IMETHOD GetInsertionPointCount(PRUint32* aCount);
-  NS_IMETHOD GetInsertionPointAt(PRUint32 i, nsIXBLInsertionPoint** aResult);
-  
+  PRInt32 GetInsertionPointCount() { return mElements->Count(); }
+
+  nsXBLInsertionPoint* GetInsertionPointAt(PRInt32 i) { return NS_STATIC_CAST(nsXBLInsertionPoint*, mElements->ElementAt(i)); }
+
 private:
-  nsISupportsArray* mElements;
+  nsVoidArray* mElements;
 };
 
-MOZ_DECL_CTOR_COUNTER(nsAnonymousContentList);
+MOZ_DECL_CTOR_COUNTER(nsAnonymousContentList)
 
-NS_IMPL_ISUPPORTS_INHERITED1(nsAnonymousContentList, nsGenericDOMNodeList, nsIAnonymousContentList)
-
-nsAnonymousContentList::nsAnonymousContentList(nsISupportsArray* aElements)
+nsAnonymousContentList::nsAnonymousContentList(nsVoidArray* aElements)
+  : mElements(aElements)
 {
   MOZ_COUNT_CTOR(nsAnonymousContentList);
 
   // We don't reference count our Anonymous reference (to avoid circular
   // references). We'll be told when the Anonymous goes away.
-  mElements = aElements;
-  NS_IF_ADDREF(mElements);
 }
- 
+
+static PRBool PR_CALLBACK DeleteInsertionPoint(void* aElement, void* aData)
+{
+  delete NS_STATIC_CAST(nsXBLInsertionPoint*, aElement);
+  return PR_TRUE;
+}
+
 nsAnonymousContentList::~nsAnonymousContentList()
 {
   MOZ_COUNT_DTOR(nsAnonymousContentList);
-  NS_IF_RELEASE(mElements);
+  mElements->EnumerateForwards(DeleteInsertionPoint, nsnull);
+  delete mElements;
 }
 
 NS_IMETHODIMP
@@ -157,17 +143,11 @@ nsAnonymousContentList::GetLength(PRUint32* aLength)
   if (! aLength)
       return NS_ERROR_NULL_POINTER;
 
-  PRUint32 cnt;
-  mElements->Count(&cnt);
+  PRInt32 cnt = mElements->Count();
 
   *aLength = 0;
-  nsCOMPtr<nsIXBLInsertionPoint> point;
-  PRUint32 l;
-  for (PRUint32 i = 0; i < cnt; i++) {
-    point = getter_AddRefs((nsIXBLInsertionPoint*)(mElements->ElementAt(i)));
-    point->ChildCount(&l);
-    *aLength += l;
-  }
+  for (PRInt32 i = 0; i < cnt; i++)
+    *aLength += NS_STATIC_CAST(nsXBLInsertionPoint*, mElements->ElementAt(i))->ChildCount();
 
   return NS_OK;
 }
@@ -175,48 +155,26 @@ nsAnonymousContentList::GetLength(PRUint32* aLength)
 NS_IMETHODIMP    
 nsAnonymousContentList::Item(PRUint32 aIndex, nsIDOMNode** aReturn)
 {
-  PRUint32 cnt;
-  nsresult rv = mElements->Count(&cnt);
-  if (NS_FAILED(rv)) return rv;
-
+  PRInt32 cnt = mElements->Count();
   PRUint32 pointCount = 0;
-  nsCOMPtr<nsIXBLInsertionPoint> point;
-  for (PRUint32 i = 0; i < cnt; i++) {
+
+  for (PRInt32 i = 0; i < cnt; i++) {
     aIndex -= pointCount;
     
-    point = getter_AddRefs((nsIXBLInsertionPoint*)(mElements->ElementAt(i)));
-    point->ChildCount(&pointCount);
+    nsXBLInsertionPoint* point = NS_STATIC_CAST(nsXBLInsertionPoint*, mElements->ElementAt(i));
+    pointCount = point->ChildCount();
 
     if (aIndex < pointCount) {
-      nsCOMPtr<nsIContent> result;
-      rv = point->ChildAt(aIndex, getter_AddRefs(result));
-      if (result && NS_SUCCEEDED(rv))
-        return result->QueryInterface(NS_GET_IID(nsIDOMNode), (void**)aReturn);
-      else return rv;
+      nsCOMPtr<nsIContent> result = point->ChildAt(aIndex);
+      if (result)
+        return CallQueryInterface(result, aReturn);
+      return NS_ERROR_FAILURE;
     }
   }
 
   return NS_ERROR_FAILURE;
 }
 
-NS_IMETHODIMP
-nsAnonymousContentList::GetInsertionPointCount(PRUint32* aCount)
-{
-  *aCount = 0;
-  if (mElements)
-    mElements->Count(aCount);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsAnonymousContentList::GetInsertionPointAt(PRUint32 i, nsIXBLInsertionPoint** aResult)
-{
-  *aResult = nsnull;
-  if (mElements)
-    *aResult = (nsIXBLInsertionPoint*)mElements->ElementAt(i);
-  return NS_OK;
-}
-
 // nsDoubleHashtable stuff to create a generic string -> nsISupports
 // table
 
@@ -379,11 +337,11 @@ public:
   NS_IMETHOD ResolveTag(nsIContent* aContent, PRInt32* aNameSpaceID, nsIAtom** aResult);
 
   NS_IMETHOD GetContentListFor(nsIContent* aContent, nsIDOMNodeList** aResult);
-  NS_IMETHOD SetContentListFor(nsIContent* aContent, nsISupportsArray* aList);
+  NS_IMETHOD SetContentListFor(nsIContent* aContent, nsVoidArray* aList);
   NS_IMETHOD HasContentListFor(nsIContent* aContent, PRBool* aResult);
 
   NS_IMETHOD GetAnonymousNodesFor(nsIContent* aContent, nsIDOMNodeList** aResult);
-  NS_IMETHOD SetAnonymousNodesFor(nsIContent* aContent, nsISupportsArray* aList);
+  NS_IMETHOD SetAnonymousNodesFor(nsIContent* aContent, nsVoidArray* aList);
 
   NS_IMETHOD GetXBLChildNodesFor(nsIContent* aContent, nsIDOMNodeList** aResult);
 
@@ -397,7 +355,6 @@ public:
                                  nsIDocument** aResult);
 
   NS_IMETHOD AddToAttachedQueue(nsIXBLBinding* aBinding);
-  NS_IMETHOD AddHandlerToAttachedQueue(nsIXBLBindingAttachedHandler* aHandler);
   NS_IMETHOD ClearAttachedQueue();
   NS_IMETHOD ProcessAttachedQueue();
 
@@ -423,23 +380,24 @@ public:
   NS_IMETHOD WalkRules(nsIStyleSet* aStyleSet, 
                        nsISupportsArrayEnumFunc aFunc,
                        RuleProcessorData* aData);
-  NS_IMETHOD AttributeAffectsStyle(nsISupportsArrayEnumFunc aFunc,
-                                   void* aData, nsIContent* aContent,
-                                   PRBool* aAffects);
 
   // nsIDocumentObserver
   NS_DECL_NSIDOCUMENTOBSERVER
 
 protected:
+  nsresult GetXBLChildNodesInternal(nsIContent* aContent,
+                                    nsIDOMNodeList** aResult,
+                                    PRBool* aIsAnonymousContentList);
+  nsresult GetAnonymousNodesInternal(nsIContent* aContent,
+                                     nsIDOMNodeList** aResult,
+                                     PRBool* aIsAnonymousContentList);
+
   void GetEnclosingScope(nsIContent* aContent, nsIContent** aParent);
   void GetOutermostStyleScope(nsIContent* aContent, nsIContent** aParent);
 
   void WalkRules(nsISupportsArrayEnumFunc aFunc, RuleProcessorData* aData,
                  nsIContent* aParent, nsIContent* aCurrContent);
 
-  void AttributeAffectsStyle(nsISupportsArrayEnumFunc aFunc, void* aData,
-                             nsIContent* aParent, nsIContent* aCurrContent, PRBool* aAffects);
-
   nsresult GetNestedInsertionPoint(nsIContent* aParent, nsIContent* aChild, nsIContent** aResult);
 
 // MEMBER VARIABLES
@@ -502,7 +460,6 @@ NS_IMPL_ISUPPORTS3(nsBindingManager, nsIBindingManager, nsIStyleRuleSupplier, ns
 // Constructors/Destructors
 nsBindingManager::nsBindingManager(void)
 {
-  NS_INIT_ISUPPORTS();
 
   mBindingTable.ops = nsnull;
   mContentListTable.ops = nsnull;
@@ -696,7 +653,7 @@ nsBindingManager::GetContentListFor(nsIContent* aContent, nsIDOMNodeList** aResu
 }
 
 NS_IMETHODIMP
-nsBindingManager::SetContentListFor(nsIContent* aContent, nsISupportsArray* aList)
+nsBindingManager::SetContentListFor(nsIContent* aContent, nsVoidArray* aList)
 {
   nsIDOMNodeList* contentList = nsnull;
   if (aList) {
@@ -719,8 +676,10 @@ nsBindingManager::HasContentListFor(nsIContent* aContent, PRBool* aResult)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsBindingManager::GetAnonymousNodesFor(nsIContent* aContent, nsIDOMNodeList** aResult)
+nsresult
+nsBindingManager::GetAnonymousNodesInternal(nsIContent* aContent,
+                                            nsIDOMNodeList** aResult,
+                                            PRBool* aIsAnonymousContentList)
 { 
   // Locate the primary binding and get its node list of anonymous children.
   *aResult = nsnull;
@@ -731,17 +690,27 @@ nsBindingManager::GetAnonymousNodesFor(nsIContent* aContent, nsIDOMNodeList** aR
   }
 
   if (!*aResult) {
+    *aIsAnonymousContentList = PR_FALSE;
     nsCOMPtr<nsIXBLBinding> binding;
     GetBinding(aContent, getter_AddRefs(binding));
     if (binding)
       return binding->GetAnonymousNodes(aResult);
-  }
+  } else
+    *aIsAnonymousContentList = PR_TRUE;
 
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBindingManager::SetAnonymousNodesFor(nsIContent* aContent, nsISupportsArray* aList)
+nsBindingManager::GetAnonymousNodesFor(nsIContent* aContent,
+                                       nsIDOMNodeList** aResult)
+{
+  PRBool dummy;
+  return GetAnonymousNodesInternal(aContent, aResult, &dummy);
+}
+
+NS_IMETHODIMP
+nsBindingManager::SetAnonymousNodesFor(nsIContent* aContent, nsVoidArray* aList)
 {
   nsIDOMNodeList* contentList = nsnull;
   if (aList) {
@@ -752,15 +721,17 @@ nsBindingManager::SetAnonymousNodesFor(nsIContent* aContent, nsISupportsArray* a
   return SetOrRemoveObject(mAnonymousNodesTable, aContent, contentList);
 }
 
-NS_IMETHODIMP
-nsBindingManager::GetXBLChildNodesFor(nsIContent* aContent, nsIDOMNodeList** aResult)
+nsresult
+nsBindingManager::GetXBLChildNodesInternal(nsIContent* aContent,
+                                           nsIDOMNodeList** aResult,
+                                           PRBool* aIsAnonymousContentList)
 {
   *aResult = nsnull;
 
   PRUint32 length;
 
   // Retrieve the anonymous content that we should build.
-  GetAnonymousNodesFor(aContent, aResult);
+  GetAnonymousNodesInternal(aContent, aResult, aIsAnonymousContentList);
   if (*aResult) {
     (*aResult)->GetLength(&length);
     if (length == 0)
@@ -775,11 +746,20 @@ nsBindingManager::GetXBLChildNodesFor(nsIContent* aContent, nsIDOMNodeList** aRe
       *aResult = NS_STATIC_CAST(nsIDOMNodeList*,
                                 LookupObject(mContentListTable, aContent));
       NS_IF_ADDREF(*aResult);
+      *aIsAnonymousContentList = PR_TRUE;
     }
   }
+
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsBindingManager::GetXBLChildNodesFor(nsIContent* aContent, nsIDOMNodeList** aResult)
+{
+  PRBool dummy;
+  return GetXBLChildNodesInternal(aContent, aResult, &dummy);
+}
+
 NS_IMETHODIMP
 nsBindingManager::GetInsertionPoint(nsIContent* aParent, nsIContent* aChild, nsIContent** aResult, PRUint32* aIndex)
 {
@@ -948,17 +928,6 @@ nsBindingManager::AddToAttachedQueue(nsIXBLBinding* aBinding)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsBindingManager::AddHandlerToAttachedQueue(nsIXBLBindingAttachedHandler* aBinding)
-{
-  if (!mAttachedQueue)
-    NS_NewISupportsArray(getter_AddRefs(mAttachedQueue)); // This call addrefs the array.
-
-  mAttachedQueue->AppendElement(aBinding);
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsBindingManager::ClearAttachedQueue()
 {
@@ -983,11 +952,6 @@ nsBindingManager::ProcessAttachedQueue()
     nsCOMPtr<nsIXBLBinding> binding(do_QueryInterface(supp));
     if (binding)
       binding->ExecuteAttachedHandler();
-    else {
-      nsCOMPtr<nsIXBLBindingAttachedHandler> handler(do_QueryInterface(supp));
-      if (handler)
-        handler->OnBindingAttached();
-    }
   }
 
   ClearAttachedQueue();
@@ -1388,47 +1352,6 @@ nsBindingManager::WalkRules(nsIStyleSet* aStyleSet,
   return NS_OK;
 }
 
-void
-nsBindingManager::AttributeAffectsStyle(nsISupportsArrayEnumFunc aFunc, void* aData,
-                                        nsIContent* aParent, nsIContent* aCurrContent, PRBool* aAffects)
-{    
-  nsCOMPtr<nsIXBLBinding> binding;
-  GetBinding(aCurrContent, getter_AddRefs(binding));
-  if (binding) {
-    binding->AttributeAffectsStyle(aFunc, aData, aAffects);
-  }
-
-  if (*aAffects)
-    return;
-
-  if (aParent != aCurrContent) {
-    nsCOMPtr<nsIContent> par;
-    GetEnclosingScope(aCurrContent, getter_AddRefs(par));
-    if (par)
-      AttributeAffectsStyle(aFunc, aData, aParent, par, aAffects);
-  }
-}
-
-
-NS_IMETHODIMP
-nsBindingManager::AttributeAffectsStyle(nsISupportsArrayEnumFunc aFunc, void* aData, 
-                                        nsIContent* aContent, PRBool* aAffects)
-{
-  *aAffects = PR_FALSE;
-  if (!aContent)
-    return NS_OK;
-
-  nsCOMPtr<nsIContent> parent;
-  GetOutermostStyleScope(aContent, getter_AddRefs(parent));
-
-  AttributeAffectsStyle(aFunc, aData, parent, aContent, aAffects);
-
-  if (*aAffects)
-    return NS_OK;
-
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsBindingManager::ShouldBuildChildFrames(nsIContent* aContent, PRBool* aResult)
 {
@@ -1516,30 +1439,28 @@ nsBindingManager::ContentAppended(nsIDocument* aDocument,
 
   if (ins) {
     nsCOMPtr<nsIDOMNodeList> nodeList;
-    GetXBLChildNodesFor(ins, getter_AddRefs(nodeList));
+    PRBool isAnonymousContentList;
+    GetXBLChildNodesInternal(ins, getter_AddRefs(nodeList),
+                             &isAnonymousContentList);
 
-    if (nodeList) {
-      nsCOMPtr<nsIAnonymousContentList> contentList(do_QueryInterface(nodeList));
-      if (contentList) {
-        // Find a non-pseudo-insertion point and just jam ourselves in.
-        // This is not 100% correct.  Hack city, baby.
-        PRUint32 count;
-        contentList->GetInsertionPointCount(&count);
-        for (PRUint32 i =0; i < count; i++) {
-          nsCOMPtr<nsIXBLInsertionPoint> point;
-          contentList->GetInsertionPointAt(i, getter_AddRefs(point));
-          PRInt32 index;
-          point->GetInsertionIndex(&index);
-          if (index != -1) {
-            // We're real. Jam all the kids in.
-            // XXX Check the filters to find the correct points.
-            for (PRInt32 j = aNewIndexInContainer; j < childCount; j++) {
-              aContainer->ChildAt(j, *getter_AddRefs(child));
-              point->AddChild(child);
-              SetInsertionParent(child, ins);
-            }
-            break;
+    if (nodeList && isAnonymousContentList) {
+      // Find a non-pseudo-insertion point and just jam ourselves in.
+      // This is not 100% correct.  Hack city, baby.
+      nsAnonymousContentList* contentList = NS_STATIC_CAST(nsAnonymousContentList*, NS_STATIC_CAST(nsIDOMNodeList*, nodeList.get()));
+
+      PRInt32 count = contentList->GetInsertionPointCount();
+      for (PRInt32 i = 0; i < count; i++) {
+        nsXBLInsertionPoint* point = contentList->GetInsertionPointAt(i);
+        PRInt32 index = point->GetInsertionIndex();
+        if (index != -1) {
+          // We're real. Jam all the kids in.
+          // XXX Check the filters to find the correct points.
+          for (PRInt32 j = aNewIndexInContainer; j < childCount; j++) {
+            aContainer->ChildAt(j, *getter_AddRefs(child));
+            point->AddChild(child);
+            SetInsertionParent(child, ins);
           }
+          break;
         }
       }
     }
@@ -1550,9 +1471,9 @@ nsBindingManager::ContentAppended(nsIDocument* aDocument,
 
 NS_IMETHODIMP
 nsBindingManager::ContentInserted(nsIDocument* aDocument,
-			                     nsIContent* aContainer,
-                           nsIContent* aChild,
-                           PRInt32 aIndexInContainer)
+                                  nsIContent* aContainer,
+                                  nsIContent* aChild,
+                                  PRInt32 aIndexInContainer)
 {
 // XXX This is hacked just to make menus work again.
   if (aIndexInContainer == -1 || !mContentListTable.ops)
@@ -1564,27 +1485,24 @@ nsBindingManager::ContentInserted(nsIDocument* aDocument,
 
   if (ins) {
     nsCOMPtr<nsIDOMNodeList> nodeList;
-    GetXBLChildNodesFor(ins, getter_AddRefs(nodeList));
+    PRBool isAnonymousContentList;
+    GetXBLChildNodesInternal(ins, getter_AddRefs(nodeList),
+                             &isAnonymousContentList);
 
-    if (nodeList) {
-      nsCOMPtr<nsIAnonymousContentList> contentList(do_QueryInterface(nodeList));
-      if (contentList) {
-        // Find a non-pseudo-insertion point and just jam ourselves in.
-        // This is not 100% correct.  Hack city, baby.
-        PRUint32 count;
-        contentList->GetInsertionPointCount(&count);
-        for (PRUint32 i =0; i < count; i++) {
-          nsCOMPtr<nsIXBLInsertionPoint> point;
-          contentList->GetInsertionPointAt(i, getter_AddRefs(point));
-          PRInt32 index;
-          point->GetInsertionIndex(&index);
-          if (index != -1) {
-            // We're real. Jam the kid in.
-            // XXX Check the filters to find the correct points.
-            point->InsertChildAt(aIndexInContainer, aChild);
-            SetInsertionParent(aChild, ins);
-            break;
-          }
+    if (nodeList && isAnonymousContentList) {
+      // Find a non-pseudo-insertion point and just jam ourselves in.
+      // This is not 100% correct.  Hack city, baby.
+      nsAnonymousContentList* contentList = NS_STATIC_CAST(nsAnonymousContentList*, NS_STATIC_CAST(nsIDOMNodeList*, nodeList.get()));
+
+      PRInt32 count = contentList->GetInsertionPointCount();
+      for (PRInt32 i = 0; i < count; i++) {
+        nsXBLInsertionPoint* point = contentList->GetInsertionPointAt(i);
+        if (point->GetInsertionIndex() != -1) {
+          // We're real. Jam the kid in.
+          // XXX Check the filters to find the correct points.
+          point->InsertChildAt(aIndexInContainer, aChild);
+          SetInsertionParent(aChild, ins);
+          break;
         }
       }
     }
@@ -1619,22 +1537,18 @@ nsBindingManager::ContentRemoved(nsIDocument* aDocument,
 
   if (point) {
     nsCOMPtr<nsIDOMNodeList> nodeList;
-    GetXBLChildNodesFor(point, getter_AddRefs(nodeList));
+    PRBool isAnonymousContentList;
+    GetXBLChildNodesInternal(point, getter_AddRefs(nodeList),
+                             &isAnonymousContentList);
 
-    if (nodeList) {
-      nsCOMPtr<nsIAnonymousContentList> contentList(do_QueryInterface(nodeList));
-      if (contentList) {
-        // Find a non-pseudo-insertion point and remove ourselves.
-        PRUint32 count;
-        contentList->GetInsertionPointCount(&count);
-        for (PRUint32 i =0; i < count; i++) {
-          nsCOMPtr<nsIXBLInsertionPoint> point;
-          contentList->GetInsertionPointAt(i, getter_AddRefs(point));
-          PRInt32 index;
-          point->GetInsertionIndex(&index);
-          if (index != -1) {
-            point->RemoveChild(aChild);
-          }
+    if (nodeList && isAnonymousContentList) {
+      // Find a non-pseudo-insertion point and remove ourselves.
+      nsAnonymousContentList* contentList = NS_STATIC_CAST(nsAnonymousContentList*, NS_STATIC_CAST(nsIDOMNodeList*, nodeList));
+      PRInt32 count = contentList->GetInsertionPointCount();
+      for (PRInt32 i =0; i < count; i++) {
+        nsXBLInsertionPoint* point = contentList->GetInsertionPointAt(i);
+        if (point->GetInsertionIndex() != -1) {
+          point->RemoveChild(aChild);
         }
       }
     }
