@@ -95,16 +95,15 @@ nsresult
 nsXULContentUtils::Init()
 {
     if (gRefCnt++ == 0) {
-        nsresult rv;
-        rv = nsServiceManager::GetService(kRDFServiceCID,
-                                          NS_GET_IID(nsIRDFService),
-                                          (nsISupports**) &gRDF);
-        if (NS_FAILED(rv)) return rv;
+        nsresult rv = CallGetService(kRDFServiceCID, &gRDF);
+        if (NS_FAILED(rv)) {
+            return rv;
+        }
 
-#define XUL_RESOURCE(ident, uri)            \
-  PR_BEGIN_MACRO                            \
-   rv = gRDF->GetResource((uri), &(ident)); \
-   if (NS_FAILED(rv)) return rv;            \
+#define XUL_RESOURCE(ident, uri)                              \
+  PR_BEGIN_MACRO                                              \
+   rv = gRDF->GetResource(NS_LITERAL_CSTRING(uri), &(ident)); \
+   if (NS_FAILED(rv)) return rv;                              \
   PR_END_MACRO
 
 #define XUL_LITERAL(ident, val)                                   \
@@ -117,15 +116,13 @@ nsXULContentUtils::Init()
 #undef XUL_RESOURCE
 #undef XUL_LITERAL
 
-        rv = nsComponentManager::CreateInstance(kDateTimeFormatCID,
-                                                nsnull,
-                                                NS_GET_IID(nsIDateTimeFormat),
-                                                (void**) &gFormat);
-
-        if (NS_FAILED(rv)) return rv;
+        rv = CallCreateInstance(kDateTimeFormatCID, &gFormat);
+        if (NS_FAILED(rv)) {
+            return rv;
+        }
     }
 
-    return gRefCnt;
+    return NS_OK;
 }
 
 
@@ -133,10 +130,7 @@ nsresult
 nsXULContentUtils::Finish()
 {
     if (--gRefCnt == 0) {
-        if (gRDF) {
-            nsServiceManager::ReleaseService(kRDFServiceCID, gRDF);
-            gRDF = nsnull;
-        }
+        NS_IF_RELEASE(gRDF);
 
 #define XUL_RESOURCE(ident, uri) NS_IF_RELEASE(ident)
 #define XUL_LITERAL(ident, val) NS_IF_RELEASE(ident)
@@ -147,7 +141,7 @@ nsXULContentUtils::Finish()
         NS_IF_RELEASE(gFormat);
     }
 
-    return gRefCnt;
+    return NS_OK;
 }
 
 
@@ -259,7 +253,7 @@ nsXULContentUtils::GetElementRefResource(nsIContent* aElement, nsIRDFResource**
         // the protocol), uriStr will be untouched.
         NS_MakeAbsoluteURI(uri, uri, url);
 
-        rv = gRDF->GetUnicodeResource(uri.get(), aResult);
+        rv = gRDF->GetUnicodeResource(uri, aResult);
     }
     else {
         rv = GetElementResource(aElement, aResult);
@@ -397,7 +391,7 @@ nsXULContentUtils::MakeElementResource(nsIDocument* aDocument, const nsAString&
     rv = MakeElementURI(aDocument, aID, uri);
     if (NS_FAILED(rv)) return rv;
 
-    rv = gRDF->GetResource(uri.get(), aResult);
+    rv = gRDF->GetResource(uri, aResult);
     NS_ASSERTION(NS_SUCCEEDED(rv), "unable to create resource");
     if (NS_FAILED(rv)) return rv;
 
@@ -481,7 +475,7 @@ nsXULContentUtils::GetResource(PRInt32 aNameSpaceID, const nsAString& aAttribute
 
     uri.Append(aAttribute);
 
-    rv = gRDF->GetUnicodeResource(uri.get(), aResult);
+    rv = gRDF->GetUnicodeResource(uri, aResult);
     NS_ASSERTION(NS_SUCCEEDED(rv), "unable to get resource");
     if (NS_FAILED(rv)) return rv;
 
