@@ -47,6 +47,7 @@
 #include "nsITreeBoxObject.h"
 #include "nsITreeSelection.h"
 #include "nsITreeView.h"
+#include "nsTreeUtils.h"
 #include "nsIServiceManager.h"
 #include "nsReadableUtils.h"
 
@@ -152,13 +153,6 @@ protected:
     nsresult
     GetTemplateActionCellFor(PRInt32 aRow, const PRUnichar* aColID, nsIContent** aResult);
 
-    /**
-     * Parse a whitespace separated list of properties into an array
-     * of atoms.
-     */
-    nsresult
-    TokenizeProperties(const nsAString& aString, nsISupportsArray* aProprerties);
-
     /**
      * Return the resource corresponding to a row in the tree. The
      * result is *not* addref'd
@@ -179,6 +173,7 @@ protected:
      */
     nsresult
     OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
+                  PRInt32 aIndex,
                   nsIRDFResource* aContainer,
                   PRInt32* aDelta);
 
@@ -330,8 +325,9 @@ nsXULTreeBuilder::Init()
     if (NS_FAILED(rv)) return rv;
 
     if (gRefCnt++ == 0) {
-        gRDFService->GetResource(RDF_NAMESPACE_URI "type", &kRDF_type);
-        gRDFService->GetResource(NC_NAMESPACE_URI "BookmarkSeparator", &kNC_BookmarkSeparator);
+        gRDFService->GetResource(NS_LITERAL_CSTRING(RDF_NAMESPACE_URI "type"), &kRDF_type);
+        gRDFService->GetResource(NS_LITERAL_CSTRING(NC_NAMESPACE_URI "BookmarkSeparator"),
+                                 &kNC_BookmarkSeparator);
     }
 
     // Try to acquire a collation object for sorting
@@ -442,6 +438,7 @@ nsXULTreeBuilder::Sort(nsIDOMElement* aElement)
             mBoxObject->Invalidate();
 
         header->SetAttr(kNameSpaceID_None, nsXULAtoms::sortDirection, dir, PR_TRUE);
+        header->SetAttr(kNameSpaceID_None, nsXULAtoms::sortActive, NS_LITERAL_STRING("true"), PR_TRUE);
 
         // Unset sort attribute(s) on the other columns
         nsCOMPtr<nsIContent> parentContent;
@@ -461,6 +458,8 @@ nsXULTreeBuilder::Sort(nsIDOMElement* aElement)
                         if (childTag == nsXULAtoms::treecol && childContent != header) {
                             childContent->UnsetAttr(kNameSpaceID_None,
                                                     nsXULAtoms::sortDirection, PR_TRUE);
+                            childContent->UnsetAttr(kNameSpaceID_None,
+                                                    nsXULAtoms::sortActive, PR_TRUE);
                         }
                     }
                 }
@@ -514,7 +513,7 @@ nsXULTreeBuilder::GetRowProperties(PRInt32 aIndex, nsISupportsArray* aProperties
             nsAutoString cooked;
             SubstituteText(*(mRows[aIndex]->mMatch), raw, cooked);
 
-            TokenizeProperties(cooked, aProperties);
+            nsTreeUtils::TokenizeProperties(cooked, aProperties);
         }
     }
 
@@ -538,7 +537,7 @@ nsXULTreeBuilder::GetCellProperties(PRInt32 aRow, const PRUnichar* aColID, nsISu
             nsAutoString cooked;
             SubstituteText(*(mRows[aRow]->mMatch), raw, cooked);
 
-            TokenizeProperties(cooked, aProperties);
+            nsTreeUtils::TokenizeProperties(cooked, aProperties);
         }
     }
 
@@ -583,7 +582,19 @@ nsXULTreeBuilder::IsContainerOpen(PRInt32 aIndex, PRBool* aResult)
     if (aIndex < 0 || aIndex >= mRows.Count())
         return NS_ERROR_INVALID_ARG;
 
-    return IsContainerOpen(GetResourceFor(aIndex), aResult);
+    nsTreeRows::iterator iter = mRows[aIndex];
+
+    if (iter->mContainerState == nsTreeRows::eContainerState_Unknown) {
+        PRBool isOpen;
+        IsContainerOpen(GetResourceFor(aIndex), &isOpen);
+
+        iter->mContainerState = isOpen
+            ? nsTreeRows::eContainerState_Open
+            : nsTreeRows::eContainerState_Closed;
+    }
+
+    *aResult = (iter->mContainerState == nsTreeRows::eContainerState_Open);
+    return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -597,16 +608,16 @@ nsXULTreeBuilder::IsContainerEmpty(PRInt32 aIndex, PRBool* aResult)
     NS_ASSERTION(iter->mContainerType == nsTreeRows::eContainerType_Container,
                  "asking for empty state on non-container");
 
-    if (iter->mContainerState == nsTreeRows::eContainerState_Unknown) {
+    if (iter->mContainerFill == nsTreeRows::eContainerFill_Unknown) {
         PRBool isEmpty;
         CheckContainer(GetResourceFor(aIndex), nsnull, &isEmpty);
 
-        iter->mContainerState = isEmpty
-            ? nsTreeRows::eContainerState_Empty
-            : nsTreeRows::eContainerState_Nonempty;
+        iter->mContainerFill = isEmpty
+            ? nsTreeRows::eContainerFill_Empty
+            : nsTreeRows::eContainerFill_Nonempty;
     }
 
-    *aResult = (iter->mContainerState == nsTreeRows::eContainerState_Empty);
+    *aResult = (iter->mContainerFill == nsTreeRows::eContainerFill_Empty);
     return NS_OK;
 }
 
@@ -687,21 +698,6 @@ nsXULTreeBuilder::GetImageSrc(PRInt32 aRow, const PRUnichar* aColID, nsAString&
     if (aRow < 0 || aRow >= mRows.Count())
         return NS_ERROR_INVALID_ARG;
 
-    // See if any XUL builder observers have an image source to supply.
-    if (mObservers) {
-        PRUint32 count;
-        mObservers->Count(&count);
-        for (PRUint32 i = 0; i < count; ++i) {
-            nsCOMPtr<nsIXULTreeBuilderObserver> observer;
-            mObservers->QueryElementAt(i, NS_GET_IID(nsIXULTreeBuilderObserver), getter_AddRefs(observer));
-            if (observer) {
-                observer->GetImageSource(aRow, aColID, aResult);
-                if (!aResult.IsEmpty())
-                  return NS_OK;
-            }
-        }
-    }
-
     // Find the <cell> that corresponds to the column we want.
     nsCOMPtr<nsIContent> cell;
     GetTemplateActionCellFor(aRow, aColID, getter_AddRefs(cell));
@@ -869,25 +865,32 @@ nsXULTreeBuilder::ToggleOpenState(PRInt32 aIndex)
     }
     
     if (mPersistStateStore) {
+        PRBool isOpen;
+        IsContainerOpen(aIndex, &isOpen);
+
         nsIRDFResource* container = GetResourceFor(aIndex);
         if (! container)
             return NS_ERROR_FAILURE;
 
-        PRBool open;
-        IsContainerOpen(container, &open);
+        PRBool hasProperty;
+        IsContainerOpen(container, &hasProperty);
 
-        if (open) {
-            mPersistStateStore->Unassert(container,
-                                         nsXULContentUtils::NC_open,
-                                         nsXULContentUtils::true_);
+        if (isOpen) {
+            if (hasProperty) {
+                mPersistStateStore->Unassert(container,
+                                             nsXULContentUtils::NC_open,
+                                             nsXULContentUtils::true_);
+            }
 
             CloseContainer(aIndex, container);
         }
         else {
-            mPersistStateStore->Assert(VALUE_TO_IRDFRESOURCE(container),
-                                       nsXULContentUtils::NC_open,
-                                       nsXULContentUtils::true_,
-                                       PR_TRUE);
+            if (! hasProperty) {
+                mPersistStateStore->Assert(VALUE_TO_IRDFRESOURCE(container),
+                                           nsXULContentUtils::NC_open,
+                                           nsXULContentUtils::true_,
+                                           PR_TRUE);
+            }
 
             OpenContainer(aIndex, container);
         }
@@ -1175,13 +1178,7 @@ nsXULTreeBuilder::ReplaceMatch(nsIRDFResource* aMember,
             // Use the persist store to remember if the container
             // is open or closed.
             PRBool open = PR_FALSE;
-
-            if (mPersistStateStore)
-                mPersistStateStore->HasAssertion(container,
-                                                 nsXULContentUtils::NC_open,
-                                                 nsXULContentUtils::true_,
-                                                 PR_TRUE,
-                                                 &open);
+            IsContainerOpen(row, &open);
 
             // If it's open, make sure that we've got a subtree structure ready.
             if (open)
@@ -1191,9 +1188,9 @@ nsXULTreeBuilder::ReplaceMatch(nsIRDFResource* aMember,
             // container, so whether its open or closed, make sure
             // that we've got our tree row's state correct.
             if ((iter->mContainerType != nsTreeRows::eContainerType_Container) ||
-                (iter->mContainerState != nsTreeRows::eContainerState_Nonempty)) {
+                (iter->mContainerFill != nsTreeRows::eContainerFill_Nonempty)) {
                 iter->mContainerType  = nsTreeRows::eContainerType_Container;
-                iter->mContainerState = nsTreeRows::eContainerState_Nonempty;
+                iter->mContainerFill = nsTreeRows::eContainerFill_Nonempty;
                 mBoxObject->InvalidateRow(iter.GetRowIndex());
             }
         }
@@ -1508,48 +1505,6 @@ nsXULTreeBuilder::GetTemplateActionCellFor(PRInt32 aRow,
     return NS_OK;
 }
 
-nsresult
-nsXULTreeBuilder::TokenizeProperties(const nsAString& aString,
-                                         nsISupportsArray* aProperties)
-{
-    NS_PRECONDITION(aProperties != nsnull, "null ptr");
-    if (! aProperties)
-        return NS_ERROR_NULL_POINTER;
-
-    nsAString::const_iterator end;
-    aString.EndReading(end);
-
-    nsAString::const_iterator iter;
-    aString.BeginReading(iter);
-
-    do {
-        // Skip whitespace
-        while (iter != end && nsCRT::IsAsciiSpace(*iter))
-            ++iter;
-
-        // If only whitespace, we're done
-        if (iter == end)
-            break;
-
-        // Note the first non-whitespace character
-        nsAString::const_iterator first = iter;
-
-        // Advance to the next whitespace character
-        while (iter != end && ! nsCRT::IsAsciiSpace(*iter))
-            ++iter;
-
-        // XXX this would be nonsensical
-        NS_ASSERTION(iter != first, "eh? something's wrong here");
-        if (iter == first)
-            break;
-
-        nsCOMPtr<nsIAtom> atom = dont_AddRef(NS_NewAtom(Substring(first, iter)));
-        aProperties->AppendElement(atom);
-    } while (iter != end);
-
-    return NS_OK;
-}
-
 nsIRDFResource*
 nsXULTreeBuilder::GetResourceFor(PRInt32 aRow)
 {
@@ -1575,6 +1530,8 @@ nsXULTreeBuilder::OpenContainer(PRInt32 aIndex, nsIRDFResource* aContainer)
         nsTreeRows::iterator iter = mRows[aIndex];
         container = mRows.EnsureSubtreeFor(iter.GetParent(),
                                            iter.GetChildIndex());
+
+        iter->mContainerState = nsTreeRows::eContainerState_Open;
     }
     else
         container = mRows.GetRoot();
@@ -1583,7 +1540,7 @@ nsXULTreeBuilder::OpenContainer(PRInt32 aIndex, nsIRDFResource* aContainer)
         return NS_ERROR_OUT_OF_MEMORY;
 
     PRInt32 count;
-    OpenSubtreeOf(container, aContainer, &count);
+    OpenSubtreeOf(container, aIndex, aContainer, &count);
 
     // Notify the box object
     if (mBoxObject) {
@@ -1599,6 +1556,7 @@ nsXULTreeBuilder::OpenContainer(PRInt32 aIndex, nsIRDFResource* aContainer)
 
 nsresult
 nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
+                                    PRInt32 aIndex,
                                     nsIRDFResource* aContainer,
                                     PRInt32* aDelta)
 {
@@ -1656,6 +1614,35 @@ nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
                ("xultemplate[%p] %smatch=%p", this, space.get(), match));
 #endif
 
+        Value val;
+        match->GetAssignmentFor(mConflictSet,
+                                match->mRule->GetMemberVariable(),
+                                &val);
+
+        // Don't allow cyclic graphs to get our knickers in a knot.
+        PRBool cyclic = PR_FALSE;
+
+        if (aIndex >= 0) {
+            for (nsTreeRows::iterator iter = mRows[aIndex]; iter.GetDepth() > 0; iter.Pop()) {
+                nsTemplateMatch* parentMatch = iter->mMatch;
+
+                Value parentVal;
+                parentMatch->GetAssignmentFor(mConflictSet,
+                                              parentMatch->mRule->GetMemberVariable(),
+                                              &parentVal);
+
+                if (val == parentVal) {
+                    cyclic = PR_TRUE;
+                    break;
+                }
+            }
+        }
+
+        if (cyclic) {
+            NS_WARNING("outliner cannot handle cyclic graphs");
+            continue;
+        }
+
         // Remember that this match applied to this row
         mRows.InsertRowAt(match, aSubtree, count);
 
@@ -1664,11 +1651,6 @@ nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
 
         // If this is open, then remember it so we can recursively add
         // *its* rows to the tree.
-        Value val;
-        match->GetAssignmentFor(mConflictSet,
-                                match->mRule->GetMemberVariable(),
-                                &val);
-
         PRBool isOpen = PR_FALSE;
         IsContainerOpen(VALUE_TO_IRDFRESOURCE(val), &isOpen);
         if (isOpen)
@@ -1678,8 +1660,8 @@ nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
     }
 
     // Now recursively deal with any open sub-containers that just got
-    // inserted
-    for (PRInt32 i = 0; i < open.Count(); ++i) {
+    // inserted. We need to do this back-to-front to avoid skewing offsets.
+    for (PRInt32 i = open.Count() - 1; i >= 0; --i) {
         PRInt32 index = NS_PTR_TO_INT32(open[i]);
 
         nsTreeRows::Subtree* child =
@@ -1689,9 +1671,9 @@ nsXULTreeBuilder::OpenSubtreeOf(nsTreeRows::Subtree* aSubtree,
 
         Value val;
         match->GetAssignmentFor(mConflictSet, match->mRule->GetMemberVariable(), &val);
-        
+
         PRInt32 delta;
-        OpenSubtreeOf(child, VALUE_TO_IRDFRESOURCE(val), &delta);
+        OpenSubtreeOf(child, aIndex + index, VALUE_TO_IRDFRESOURCE(val), &delta);
         count += delta;
     }
 
@@ -1756,6 +1738,8 @@ nsXULTreeBuilder::CloseContainer(PRInt32 aIndex, nsIRDFResource* aContainer)
         PRInt32 count = mRows.GetSubtreeSizeFor(iter);
         mRows.RemoveSubtreeFor(iter);
 
+        iter->mContainerState = nsTreeRows::eContainerState_Closed;
+
         if (mBoxObject) {
             mBoxObject->InvalidateRow(aIndex);
 
@@ -1933,11 +1917,10 @@ nsXULTreeBuilder::CompareMatches(nsTemplateMatch* aLeft, nsTemplateMatch* aRight
                 r->GetValueConst(&rstr);
 
                 if (mCollation) {
-                    mCollation->CompareRawSortKey(NS_REINTERPRET_CAST(const PRUint8*, lstr),
-                                                  nsCRT::strlen(lstr) * sizeof(PRUnichar),
-                                                  NS_REINTERPRET_CAST(const PRUint8*, rstr),
-                                                  nsCRT::strlen(rstr) * sizeof(PRUnichar),
-                                                  &result);
+                    mCollation->CompareString(kCollationCaseInSensitive,
+                                              nsDependentString(lstr),
+                                              nsDependentString(rstr),
+                                              &result);
                 }
                 else
                     result = ::Compare(nsDependentString(lstr),
