@@ -63,6 +63,7 @@ struct nsDOMClassInfoData
     nsDOMClassInfoConstructorFnc mConstructorFptr;
     nsDOMClassInfoExternalConstructorFnc mExternalConstructorFptr;
   } u;
+
   nsIClassInfo *mCachedClassInfo; // low bit is set to 1 if external,
                                   // so be sure to mask if necessary!
   const nsIID *mProtoChainInterface;
@@ -150,6 +151,7 @@ protected:
   static inline PRBool IsReadonlyReplaceable(jsval id)
   {
     return (id == sTop_id          ||
+            id == sParent_id       ||
             id == sScrollbars_id   ||
             id == sContent_id      ||
             id == sSidebar_id      ||
@@ -162,7 +164,11 @@ protected:
             id == sControllers_id  ||
             id == sScrollX_id      ||
             id == sScrollY_id      ||
-            id == sLength_id);
+            id == sScrollMaxX_id   ||
+            id == sScrollMaxY_id   ||
+            id == sLength_id       ||
+            id == sFrames_id       ||
+            id == sSelf_id);
   }
 
   static inline PRBool IsWritableReplaceable(jsval id)
@@ -193,6 +199,7 @@ protected:
   static PRBool sIsInitialized;
 
   static jsval sTop_id;
+  static jsval sParent_id;
   static jsval sScrollbars_id;
   static jsval sLocation_id;
   static jsval sComponents_id;
@@ -242,12 +249,16 @@ protected:
   static jsval sScrollIntoView_id;
   static jsval sScrollX_id;
   static jsval sScrollY_id;
+  static jsval sScrollMaxX_id;
+  static jsval sScrollMaxY_id;
   static jsval sOpen_id;
   static jsval sItem_id;
   static jsval sEnumerate_id;
   static jsval sNavigator_id;
   static jsval sDocument_id;
   static jsval sWindow_id;
+  static jsval sFrames_id;
+  static jsval sSelf_id;
 
   static const JSClass *sObjectClass;
 
@@ -804,7 +815,8 @@ public:
 class nsHTMLPluginObjElementSH : public nsHTMLAppletElementSH
 {
 protected:
-  nsHTMLPluginObjElementSH(nsDOMClassInfoData* aData) : nsHTMLAppletElementSH(aData)
+  nsHTMLPluginObjElementSH(nsDOMClassInfoData* aData)
+    : nsHTMLAppletElementSH(aData)
   {
   }
 
@@ -829,16 +841,17 @@ public:
 };
 
 
-// HTMLOptionCollection helper
+// HTMLOptionsCollection helper
 
-class nsHTMLOptionCollectionSH : public nsHTMLCollectionSH
+class nsHTMLOptionsCollectionSH : public nsHTMLCollectionSH
 {
 protected:
-  nsHTMLOptionCollectionSH(nsDOMClassInfoData* aData) : nsHTMLCollectionSH(aData)
+  nsHTMLOptionsCollectionSH(nsDOMClassInfoData* aData)
+    : nsHTMLCollectionSH(aData)
   {
   }
 
-  virtual ~nsHTMLOptionCollectionSH()
+  virtual ~nsHTMLOptionsCollectionSH()
   {
   }
 
@@ -848,7 +861,7 @@ public:
 
   static nsIClassInfo *doCreate(nsDOMClassInfoData* aData)
   {
-    return new nsHTMLOptionCollectionSH(aData);
+    return new nsHTMLOptionsCollectionSH(aData);
   }
 };
 
