@@ -23,6 +23,7 @@
  *   Simon Fraser   <sfraser@netscape.com>
  *   Michael Judge  <mjudge@netscape.com>
  *   Charles Manske <cmanske@netscape.com>
+ *   Kathleen Brade <brade@netscape.com>
  *
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -53,6 +54,7 @@
 #include "nsIChannel.h"
 #include "nsIWebProgress.h"
 #include "nsIWebNavigation.h"
+#include "nsIRefreshURI.h"
 
 #include "nsIControllers.h"
 #include "nsIController.h"
@@ -74,6 +76,9 @@
 #include "nsIPlaintextEditor.h"
 
 #include "nsIDOMNSDocument.h"
+#include "nsIScriptContext.h"
+#include "imgIContainer.h"
+#include "nsIPresContext.h"
 
 #include "nsEditorParserObserver.h"
 
@@ -96,7 +101,6 @@ nsEditingSession::nsEditingSession()
 , mDocStateControllerId(0)
 , mHTMLCommandControllerId(0)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 /*---------------------------------------------------------------------------
@@ -106,6 +110,9 @@ nsEditingSession::nsEditingSession()
 ----------------------------------------------------------------------------*/
 nsEditingSession::~nsEditingSession()
 {
+  // Must cancel previous timer?
+  if (mLoadBlankDocTimer)
+    mLoadBlankDocTimer->Cancel();
 }
 
 NS_IMPL_ISUPPORTS3(nsEditingSession, nsIEditingSession, nsIWebProgressListener, 
@@ -148,6 +155,28 @@ nsEditingSession::MakeWindowEditable(nsIDOMWindow *aWindow,
 {
   mEditorType.Truncate();
   mEditorFlags = 0;
+  mWindowToBeEdited = getter_AddRefs(NS_GetWeakReference(aWindow));
+
+  // disable plugins
+  nsCOMPtr<nsIDocShell> docShell;
+  nsresult rv = GetDocShellFromWindow(aWindow, getter_AddRefs(docShell));
+  if (NS_FAILED(rv)) return rv;
+
+  rv = docShell->SetAllowPlugins(PR_FALSE);
+  if (NS_FAILED(rv)) return rv;
+
+  // Disable JavaScript in this document:
+  nsCOMPtr<nsIScriptGlobalObject> sgo (do_QueryInterface(aWindow));
+  if (sgo)
+  {
+    nsCOMPtr<nsIScriptContext> scriptContext;
+    sgo->GetContext(getter_AddRefs(scriptContext));
+    if (scriptContext)
+    {
+      rv = scriptContext->SetScriptsEnabled(PR_FALSE, PR_TRUE);
+      if (NS_FAILED(rv)) return rv;
+    }
+  }
 
   // Always remove existing editor
   TearDownEditorOnWindow(aWindow);
@@ -160,7 +189,9 @@ nsEditingSession::MakeWindowEditable(nsIDOMWindow *aWindow,
     aEditorType = DEFAULT_EDITOR_TYPE;
   mEditorType = aEditorType;
 
-  nsresult rv = PrepareForEditing();
+  // if all this does is setup listeners and I don't need listeners, 
+  // can't this step be ignored?? (based on aDoAfterURILoad)
+  rv = PrepareForEditing();
   if (NS_FAILED(rv)) return rv;  
   
   nsCOMPtr<nsIEditorDocShell> editorDocShell;
@@ -372,6 +403,21 @@ nsEditingSession::SetupEditorOnWindow(nsIDOMWindow *aWindow)
   rv = GetDocShellFromWindow(aWindow, getter_AddRefs(docShell));
   if (NS_FAILED(rv)) return rv;  
 
+  nsCOMPtr<nsIPresShell> presShell;
+  rv = docShell->GetPresShell(getter_AddRefs(presShell));
+  if (NS_FAILED(rv)) return rv;
+  if (!presShell) return NS_ERROR_FAILURE;
+
+  // Disable animation of images in this document:
+  nsCOMPtr<nsIPresContext> presContext;
+  rv = presShell->GetPresContext(getter_AddRefs(presContext));
+  if (NS_FAILED(rv)) return rv;
+  if (!presContext) return NS_ERROR_FAILURE;
+
+  rv = presContext->SetImageAnimationMode(imgIContainer::kDontAnimMode);
+  if (NS_FAILED(rv)) return rv;
+
+  // create and set editor
   nsCOMPtr<nsIEditorDocShell> editorDocShell = do_QueryInterface(docShell, &rv);
   if (NS_FAILED(rv)) return rv;
 
@@ -384,7 +430,7 @@ nsEditingSession::SetupEditorOnWindow(nsIDOMWindow *aWindow)
   // setup the HTML editor command controller
   if (needHTMLController)
   {
-    // The third controller takes an nsIEditor as the refCon
+    // The third controller takes an nsIEditor as the context
     rv = SetupEditorCommandController("@mozilla.org/editor/htmleditorcontroller;1",
                                       aWindow, editor,
                                       &mHTMLCommandControllerId);
@@ -395,11 +441,6 @@ nsEditingSession::SetupEditorOnWindow(nsIDOMWindow *aWindow)
   rv = editor->SetContentsMIMEType(mimeCType.get());
   if (NS_FAILED(rv)) return rv;
 
-  nsCOMPtr<nsIPresShell> presShell;
-  rv = docShell->GetPresShell(getter_AddRefs(presShell));
-  if (NS_FAILED(rv)) return rv;
-  if (!presShell) return NS_ERROR_FAILURE;
-
   nsCOMPtr<nsIContentViewer> contentViewer;
   rv = docShell->GetContentViewer(getter_AddRefs(contentViewer));
   if (NS_FAILED(rv)) return rv;
@@ -436,7 +477,7 @@ nsEditingSession::SetupEditorOnWindow(nsIDOMWindow *aWindow)
     txnMgr->AddListener(NS_STATIC_CAST(nsITransactionListener*,
                         stateMaintainer));
 
-  // Set refCon on all controllers to be the editor
+  // Set context on all controllers to be the editor
   rv = SetEditorOnControllers(aWindow, editor);
   if (NS_FAILED(rv)) return rv;
 
@@ -458,11 +499,29 @@ NS_IMETHODIMP
 nsEditingSession::TearDownEditorOnWindow(nsIDOMWindow *aWindow)
 {
   nsresult rv;
+  
+   // Kill any existing reload timer
+   if (mLoadBlankDocTimer)
+   {
+     mLoadBlankDocTimer->Cancel();
+     mLoadBlankDocTimer = nsnull;
+   }
+
+  nsCOMPtr<nsIEditorDocShell> editorDocShell;
+  rv = GetEditorDocShellFromWindow(aWindow, getter_AddRefs(editorDocShell));
+  if (NS_FAILED(rv)) return rv;
+  
+  nsCOMPtr<nsIEditor> editor;
+  rv = editorDocShell->GetEditor(getter_AddRefs(editor));
+  if (NS_FAILED(rv)) return rv;
+
+  // null out the editor on the docShell to trigger PreDestroy which
+  // needs to happen before document state listeners are removed below
+  rv = editorDocShell->SetEditor(nsnull);
+  if (NS_FAILED(rv)) return rv;
 
   if (mStateMaintainer)
   {
-    nsCOMPtr<nsIEditor> editor;
-    rv = GetEditorForWindow(aWindow, getter_AddRefs(editor));
     if (editor)
     {
       // If we had an editor -- we are loading a new URL into existing window
@@ -471,12 +530,13 @@ nsEditingSession::TearDownEditorOnWindow(nsIDOMWindow *aWindow)
       nsCOMPtr<nsISelection>	selection;
       editor->GetSelection(getter_AddRefs(selection));	
       nsCOMPtr<nsISelectionPrivate> selPriv = do_QueryInterface(selection);
-      if (!selPriv) return NS_ERROR_FAILURE;
-
-      nsCOMPtr<nsISelectionListener> listener = 
+      if (selPriv)
+      {
+        nsCOMPtr<nsISelectionListener> listener = 
                                         do_QueryInterface(mStateMaintainer);
-      rv = selPriv->RemoveSelectionListener(listener);
-      if (NS_FAILED(rv)) return rv;
+        rv = selPriv->RemoveSelectionListener(listener);
+        if (NS_FAILED(rv)) return rv;
+      }
 
       nsCOMPtr<nsIDocumentStateListener> docListener =
                                             do_QueryInterface(mStateMaintainer);
@@ -506,12 +566,8 @@ nsEditingSession::TearDownEditorOnWindow(nsIDOMWindow *aWindow)
       mHTMLCommandControllerId = 0;
     }
   }
-  nsCOMPtr<nsIEditorDocShell> editorDocShell;
-  rv = GetEditorDocShellFromWindow(aWindow, getter_AddRefs(editorDocShell));
-  if (NS_FAILED(rv)) return rv;  
-  
-  // null out the editor on the docShell
-  return editorDocShell->SetEditor(nsnull);
+
+  return rv;
 }
 
 /*---------------------------------------------------------------------------
@@ -554,7 +610,7 @@ nsEditingSession::OnStateChange(nsIWebProgress *aWebProgress,
     nsCAutoString contentType;
     channel->GetContentType(contentType);
     if (!contentType.IsEmpty())
-      printf(" ++++++ MIMETYPE = %s", contentType.get());
+      printf(" ++++++ MIMETYPE = %s\n", contentType.get());
   }
 #endif
 
@@ -574,8 +630,8 @@ nsEditingSession::OnStateChange(nsIWebProgress *aWebProgress,
       {
         nsXPIDLCString spec;
         uri->GetSpec(spec);
-        printf(" **** STATE_START: CHANNEL URI=%s, 
-               flags=%x\n",spec.get(), aStateFlags);
+        printf(" **** STATE_START: CHANNEL URI=%s, flags=%x\n",
+               spec.get(), aStateFlags);
       }
     }
     else
@@ -596,8 +652,8 @@ nsEditingSession::OnStateChange(nsIWebProgress *aWebProgress,
     if (aStateFlags & nsIWebProgressListener::STATE_IS_DOCUMENT)
     {
       mCanCreateEditor = PR_TRUE;
-      if (NotifyingCurrentDocument(aWebProgress))
-        (void)StartDocumentLoad(aWebProgress);
+      (void)StartDocumentLoad(aWebProgress,
+                              IsProgressForTargetDocument(aWebProgress));
 #ifdef NOISY_DOC_LOADING
       printf("STATE_START & STATE_IS_DOCUMENT flags=%x\n", aStateFlags);
 #endif
@@ -640,30 +696,24 @@ nsEditingSession::OnStateChange(nsIWebProgress *aWebProgress,
       {
         nsXPIDLCString spec;
         uri->GetSpec(spec);
-        printf(" **** STATE_STOP: CHANNEL URI=%s,
-                flags=%x\n",spec.get(), aStateFlags);
+        printf(" **** STATE_STOP: CHANNEL URI=%s, flags=%x\n",
+               spec.get(), aStateFlags);
       }
     }
     else
       printf("     STATE_STOP: NO CHANNEL  flags=%x\n", aStateFlags);
   }
 #endif
+
     // Document level notification...
     if (aStateFlags & nsIWebProgressListener::STATE_IS_DOCUMENT)
     {
-      if (NotifyingCurrentDocument(aWebProgress))
-      {
-        // Ignore STOP messages we get before we received a START
-        if (mCanCreateEditor)
-        {
-          mCanCreateEditor = PR_FALSE;
-          nsCOMPtr<nsIChannel> channel = do_QueryInterface(aRequest);
-          nsresult rv = EndDocumentLoad(aWebProgress, channel, aStatus);
+      nsCOMPtr<nsIChannel> channel = do_QueryInterface(aRequest);
+      nsresult rv = EndDocumentLoad(aWebProgress, channel, aStatus,
+                                    IsProgressForTargetDocument(aWebProgress));
 #ifdef NOISY_DOC_LOADING
-          printf("STATE_STOP & STATE_IS_DOCUMENT flags=%x\n", aStateFlags);
+      printf("STATE_STOP & STATE_IS_DOCUMENT flags=%x\n", aStateFlags);
 #endif
-        }
-      }      
     }
 
     // Page level notification...
@@ -769,15 +819,20 @@ nsEditingSession::OnSecurityChange(nsIWebProgress *aWebProgress,
 
 /*---------------------------------------------------------------------------
 
-  NotifyingCurrentDocument
+  IsProgressForTargetDocument
 
-  Check that this notification is for our document. Necessary?
+  Check that this notification is for our document.
 ----------------------------------------------------------------------------*/
 
 PRBool
-nsEditingSession::NotifyingCurrentDocument(nsIWebProgress *aWebProgress)
+nsEditingSession::IsProgressForTargetDocument(nsIWebProgress *aWebProgress)
 {
-  return PR_TRUE;
+  nsCOMPtr<nsIDOMWindow> domWindow;
+  if (aWebProgress)
+    aWebProgress->GetDOMWindow(getter_AddRefs(domWindow));
+  nsCOMPtr<nsIDOMWindow> editedDOMWindow = do_QueryReferent(mWindowToBeEdited);
+
+  return (domWindow && (domWindow == editedDOMWindow));
 }
 
 
@@ -804,10 +859,11 @@ nsEditingSession::GetEditorStatus(PRUint32 *aStatus)
   Called on start of load in a single frame
 ----------------------------------------------------------------------------*/
 nsresult
-nsEditingSession::StartDocumentLoad(nsIWebProgress *aWebProgress)
+nsEditingSession::StartDocumentLoad(nsIWebProgress *aWebProgress, 
+                                    PRBool aIsToBeMadeEditable)
 {
 #ifdef NOISY_DOC_LOADING
-  printf("Editing session StartDocumentLoad\n");
+  printf("======= StartDocumentLoad ========\n");
 #endif
 
   NS_ENSURE_ARG_POINTER(aWebProgress);
@@ -816,13 +872,14 @@ nsEditingSession::StartDocumentLoad(nsIWebProgress *aWebProgress)
   // We need to blow it away and make a new one at the end of the load.
   nsCOMPtr<nsIDOMWindow> domWindow;
   aWebProgress->GetDOMWindow(getter_AddRefs(domWindow));
-
   if (domWindow)
   {
     nsresult rv = TearDownEditorOnWindow(domWindow);
   }
     
-  mEditorStatus = eEditorCreationInProgress;
+  if (aIsToBeMadeEditable)
+    mEditorStatus = eEditorCreationInProgress;
+
   return NS_OK;
 }
 
@@ -834,13 +891,22 @@ nsEditingSession::StartDocumentLoad(nsIWebProgress *aWebProgress)
 ----------------------------------------------------------------------------*/
 nsresult
 nsEditingSession::EndDocumentLoad(nsIWebProgress *aWebProgress,
-                                  nsIChannel* aChannel, nsresult aStatus)
+                                  nsIChannel* aChannel, nsresult aStatus,
+                                  PRBool aIsToBeMadeEditable)
 {
   NS_ENSURE_ARG_POINTER(aWebProgress);
   
 #ifdef NOISY_DOC_LOADING
-  printf("Editing shell EndDocumentLoad\n");
+  printf("======= EndDocumentLoad ========\n");
+  printf("with status %d, ", aStatus);
+  nsCOMPtr<nsIURI> uri;
+  nsXPIDLCString spec;
+  if (NS_SUCCEEDED(aChannel->GetURI(getter_AddRefs(uri)))) {
+    uri->GetSpec(spec);
+    printf(" uri %s\n", spec.get());
+  }
 #endif
+
   // We want to call the base class EndDocumentLoad,
   // but avoid some of the stuff
   // that nsWebShell does (need to refactor).
@@ -851,23 +917,33 @@ nsEditingSession::EndDocumentLoad(nsIWebProgress *aWebProgress,
   
   // Set the error state -- we will create an editor 
   // anyway and load empty doc later
-  if (aStatus == NS_ERROR_FILE_NOT_FOUND)
-    mEditorStatus = eEditorErrorFileNotFound;
+  if (aIsToBeMadeEditable) {
+    if (aStatus == NS_ERROR_FILE_NOT_FOUND)
+      mEditorStatus = eEditorErrorFileNotFound;
+  }
 
   nsCOMPtr<nsIDocShell> docShell;
   nsresult rv = GetDocShellFromWindow(domWindow, getter_AddRefs(docShell));
   if (NS_FAILED(rv)) return rv;       // better error handling?
-  
+
+  // cancel refresh from meta tags
+  // we need to make sure that all pages in editor (whether editable or not)
+  // can't refresh contents being edited
+  nsCOMPtr<nsIRefreshURI> refreshURI = do_QueryInterface(docShell);
+  if (refreshURI)
+    refreshURI->CancelRefreshURITimers();
+
   nsCOMPtr<nsIEditorDocShell> editorDocShell = do_QueryInterface(docShell);
 
   // did someone set the flag to make this shell editable?
-  if (editorDocShell)
+  if (aIsToBeMadeEditable && mCanCreateEditor && editorDocShell)
   {
     PRBool  makeEditable;
     editorDocShell->GetEditable(&makeEditable);
   
     if (makeEditable)
     {
+      mCanCreateEditor = PR_FALSE;
       rv = SetupEditorOnWindow(domWindow);
       if (NS_FAILED(rv))
       {
@@ -932,12 +1008,48 @@ nsresult
 nsEditingSession::EndPageLoad(nsIWebProgress *aWebProgress,
                               nsIChannel* aChannel, nsresult aStatus)
 {
+#ifdef NOISY_DOC_LOADING
+  printf("======= EndPageLoad ========\n");
+  printf("  with status %d, ", aStatus);
+  nsCOMPtr<nsIURI> uri;
+  nsXPIDLCString spec;
+  if (NS_SUCCEEDED(aChannel->GetURI(getter_AddRefs(uri)))) {
+    uri->GetSpec(spec);
+    printf("uri %s\n", spec.get());
+  }
+ 
+  nsCAutoString contentType;
+  aChannel->GetContentType(contentType);
+  if (!contentType.IsEmpty())
+    printf("   flags = %d, status = %d, MIMETYPE = %s\n", 
+               mEditorFlags, mEditorStatus, contentType.get());
+#endif
+
   // Set the error state -- we will create an editor anyway 
   // and load empty doc later
   if (aStatus == NS_ERROR_FILE_NOT_FOUND)
     mEditorStatus = eEditorErrorFileNotFound;
 
+  nsCOMPtr<nsIDOMWindow> domWindow;
+  nsresult rv = aWebProgress->GetDOMWindow(getter_AddRefs(domWindow));
+  
+  nsCOMPtr<nsIDocShell> docShell;
+  rv = GetDocShellFromWindow(domWindow, getter_AddRefs(docShell));
+  if (NS_FAILED(rv)) return rv;
+
+  // cancel refresh from meta tags
+  // we need to make sure that all pages in editor (whether editable or not)
+  // can't refresh contents being edited
+  nsCOMPtr<nsIRefreshURI> refreshURI = do_QueryInterface(docShell);
+  if (refreshURI)
+    refreshURI->CancelRefreshURITimers();
+
+#if 0
+  // Shouldn't we do this when we want to edit sub-frames?
+  return MakeWindowEditable(domWindow, "html", PR_FALSE);
+#else
   return NS_OK;
+#endif
 }
 
 
@@ -1019,18 +1131,18 @@ nsEditingSession::PrepareForEditing()
   SetupEditorCommandController
 
   Create a command controller, append to controllers,
-  get and return the controller ID, and set the refCon
+  get and return the controller ID, and set the context
 ----------------------------------------------------------------------------*/
 nsresult
 nsEditingSession::SetupEditorCommandController(
                                   const char *aControllerClassName,
                                   nsIDOMWindow *aWindow,
-                                  nsISupports *aRefCon,
+                                  nsISupports *aContext,
                                   PRUint32 *aControllerId)
 {
   NS_ENSURE_ARG_POINTER(aControllerClassName);
   NS_ENSURE_ARG_POINTER(aWindow);
-  NS_ENSURE_ARG_POINTER(aRefCon);
+  NS_ENSURE_ARG_POINTER(aContext);
   NS_ENSURE_ARG_POINTER(aControllerId);
 
   nsresult rv;
@@ -1062,8 +1174,8 @@ nsEditingSession::SetupEditorCommandController(
     if (NS_FAILED(rv)) return rv;  
   }  
 
-  // Set the refCon
-  return SetRefConOnControllerById(controllers, aRefCon, *aControllerId);
+  // Set the context
+  return SetContextOnControllerById(controllers, aContext, *aControllerId);
 }
 
 /*---------------------------------------------------------------------------
@@ -1090,33 +1202,33 @@ nsEditingSession::SetEditorOnControllers(nsIDOMWindow *aWindow,
   nsCOMPtr<nsISupports> editorAsISupports = do_QueryInterface(aEditor);
   if (mBaseCommandControllerId)
   {
-    rv = SetRefConOnControllerById(controllers, editorAsISupports,
+    rv = SetContextOnControllerById(controllers, editorAsISupports,
                                    mBaseCommandControllerId);
     if (NS_FAILED(rv)) return rv;
   }
 
   if (mDocStateControllerId)
   {
-    rv = SetRefConOnControllerById(controllers, editorAsISupports,
+    rv = SetContextOnControllerById(controllers, editorAsISupports,
                                    mDocStateControllerId);
     if (NS_FAILED(rv)) return rv;
   }
 
   if (mHTMLCommandControllerId)
-    rv = SetRefConOnControllerById(controllers, editorAsISupports,
+    rv = SetContextOnControllerById(controllers, editorAsISupports,
                                    mHTMLCommandControllerId);
 
   return rv;
 }
 
 nsresult
-nsEditingSession::SetRefConOnControllerById(nsIControllers* aControllers,
-                                            nsISupports* aRefCon,
+nsEditingSession::SetContextOnControllerById(nsIControllers* aControllers,
+                                            nsISupports* aContext,
                                             PRUint32 aID)
 {
   NS_ENSURE_ARG_POINTER(aControllers);
-  // aRefCon can be null (when destroying editor)
 
+  // aContext can be null (when destroying editor)
   nsCOMPtr<nsIController> controller;    
   aControllers->GetControllerById(aID, getter_AddRefs(controller));
   
@@ -1125,5 +1237,5 @@ nsEditingSession::SetRefConOnControllerById(nsIControllers* aControllers,
                                        do_QueryInterface(controller);
   if (!editorController) return NS_ERROR_FAILURE;
 
-  return editorController->SetCommandRefCon(aRefCon);
+  return editorController->SetCommandContext(aContext);
 }
