@@ -104,7 +104,6 @@ nsTextEditRules::nsTextEditRules()
 , mLockRulesSniffing(PR_FALSE)
 , mTheAction(0)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsTextEditRules::~nsTextEditRules()
@@ -237,7 +236,6 @@ nsTextEditRules::AfterEdit(PRInt32 action, nsIEditor::EDirection aDirection)
     res = CreateTrailingBRIfNeeded();
     if (NS_FAILED(res)) return res;
     
-#ifdef IBMBIDI
     /* After inserting text the cursor Bidi level must be set to the level of the inserted text.
      * This is difficult, because we cannot know what the level is until after the Bidi algorithm
      * is applied to the whole paragraph.
@@ -251,7 +249,6 @@ nsTextEditRules::AfterEdit(PRInt32 action, nsIEditor::EDirection aDirection)
         shell->UndefineCaretBidiLevel();
       }
     }
-#endif
   }
   return res;
 }
@@ -633,7 +630,7 @@ nsTextEditRules::WillInsertText(PRInt32          aAction,
     nsCOMPtr<nsIDOMNode> unused;
     PRInt32 pos = 0;
         
-    // for efficiency, break out the pre case seperately.  This is because
+    // for efficiency, break out the pre case separately.  This is because
     // its a lot cheaper to search the input string for only newlines than
     // it is to search for both tabs and newlines.
     if (isPRE)
@@ -863,12 +860,10 @@ nsTextEditRules::WillDeleteSelection(nsISelection *aSelection,
     
     if (bCollapsed)
     {
-#ifdef IBMBIDI
       // Test for distance between caret and text that will be deleted
       res = CheckBidiLevelForDeletion(startNode, startOffset, aCollapsedAction, aCancel);
       if (NS_FAILED(res)) return res;
       if (*aCancel) return NS_OK;
-#endif // IBMBIDI
 
       nsCOMPtr<nsIDOMText> textNode;
       PRUint32 strLength;
@@ -937,7 +932,7 @@ nsTextEditRules::WillDeleteSelection(nsISelection *aSelection,
       }
       // fix for bugzilla #125161: if we are about to forward delete a <BR>,
       // make sure it is not last node in editfield.  If it is, cancel deletion.
-      if ((aCollapsedAction == nsIEditor::eNext) && nsTextEditUtils::IsBreak(nextNode))
+      if (nextNode && (aCollapsedAction == nsIEditor::eNext) && nsTextEditUtils::IsBreak(nextNode))
       {
         nsCOMPtr<nsIDOMNode> lastChild;
         if (!mBody) return NS_ERROR_NULL_POINTER;
@@ -1194,7 +1189,7 @@ nsTextEditRules::ReplaceNewlines(nsIDOMRange *aRange)
       res = mEditor->CreateTxnForDeleteText(textNode, offset, 1, (DeleteTextTxn**)&txn);
       if (NS_FAILED(res))  return res; 
       if (!txn)  return NS_ERROR_OUT_OF_MEMORY;
-      res = mEditor->Do(txn); 
+      res = mEditor->DoTransaction(txn); 
       if (NS_FAILED(res))  return res; 
       // The transaction system (if any) has taken ownwership of txn
       NS_IF_RELEASE(txn);
@@ -1304,7 +1299,8 @@ nsTextEditRules::TruncateInsertionIfNeeded(nsISelection *aSelection,
   nsresult res = NS_OK;
   *aOutString = *aInString;
   
-  if ((-1 != aMaxLength) && (mFlags & nsIPlaintextEditor::eEditorPlaintextMask))
+  if ((-1 != aMaxLength) && (mFlags & nsIPlaintextEditor::eEditorPlaintextMask)
+      && !mEditor->IsIMEComposing() )
   {
     // Get the current text length.
     // Get the length of inString.
@@ -1313,6 +1309,8 @@ nsTextEditRules::TruncateInsertionIfNeeded(nsISelection *aSelection,
     //   Subtract the length of the selection from the len(doc) 
     //   since we'll delete the selection on insert.
     //   This is resultingDocLength.
+    // Get old length of IME composing string
+    //   which will be replaced by new one.
     // If (resultingDocLength) is at or over max, cancel the insert
     // If (resultingDocLength) + (length of input) > max, 
     //    set aOutString to subset of inString so length = max
@@ -1324,7 +1322,10 @@ nsTextEditRules::TruncateInsertionIfNeeded(nsISelection *aSelection,
     if (NS_FAILED(res)) { return res; }
     PRInt32 selectionLength = end-start;
     if (selectionLength<0) { selectionLength *= (-1); }
-    PRInt32 resultingDocLength = docLength - selectionLength;
+    PRInt32 oldCompStrLength;
+    res = mEditor->GetIMEBufferLength(&oldCompStrLength);
+    if (NS_FAILED(res)) { return res; }
+    PRInt32 resultingDocLength = docLength - selectionLength - oldCompStrLength;
     if (resultingDocLength >= aMaxLength) 
     {
       aOutString->SetLength(0);
