@@ -70,7 +70,9 @@
 #include "nsFrameTraversal.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMXULDocument.h"
+#include "nsIImageDocument.h"
 #include "nsIDOMHTMLDocument.h"
+#include "nsIDOMNSHTMLDocument.h"
 #include "nsIDOMHTMLElement.h"
 #include "nsIEventStateManager.h"
 #include "nsIViewManager.h"
@@ -106,6 +108,8 @@
 #include "nsIPrivateCompositionEvent.h"
 #include "nsGUIEvent.h"
 #include "nsIDOMEventReceiver.h"
+#include "nsIDOM3EventTarget.h"
+#include "nsIDOMEventGroup.h"
 
 // Header for this class
 #include "nsTypeAheadFind.h"
@@ -473,17 +477,16 @@ nsTypeAheadFind::Observe(nsISupports *aSubject, const char *aTopic,
 
 
 nsresult
-nsTypeAheadFind::UseInWindow(nsIDOMWindow *aDOMWin, nsIDOMWindow **aStartWindow)
+nsTypeAheadFind::UseInWindow(nsIDOMWindow *aDOMWin)
 {
   NS_ENSURE_ARG_POINTER(aDOMWin);
-  NS_ENSURE_ARG_POINTER(aStartWindow);
 
   // Set member variables and listeners up for new window and doc
 
   mFindNextBuffer.Truncate();
   CancelFind();
 
-  GetStartWindow(aDOMWin, aStartWindow);
+  GetStartWindow(aDOMWin, getter_AddRefs(mFocusedWindow));
 
   nsCOMPtr<nsIDOMDocument> domDoc;
   aDOMWin->GetDocument(getter_AddRefs(domDoc));
@@ -513,7 +516,6 @@ nsTypeAheadFind::UseInWindow(nsIDOMWindow *aDOMWin, nsIDOMWindow **aStartWindow)
   RemoveDocListeners();
 
   mIsFindAllowedInWindow = PR_TRUE;
-  mFocusedWindow = aDOMWin;
   mFocusedWeakShell = do_GetWeakReference(presShell);
 
   // Add scroll position and selection listeners, so we can cancel
@@ -910,7 +912,8 @@ nsTypeAheadFind::HandleChar(PRUnichar aChar)
     }
 
 #ifndef NO_LINK_CYCLE_ON_SAME_CHAR
-    if (NS_FAILED(rv) && !mLiteralTextSearchOnly && mAllTheSameChar) {
+    if (NS_FAILED(rv) && !mLiteralTextSearchOnly && mAllTheSameChar && 
+        mTypeAheadBuffer.Length() > 1) {
       mRepeatingMode = eRepeatingChar;
       mDontTryExactMatch = PR_TRUE;  // Repeated character find mode
       rv = FindItNow(nsnull, PR_TRUE, PR_TRUE, mIsFirstVisiblePreferred);
@@ -1330,6 +1333,7 @@ nsTypeAheadFind::FindItNow(nsIPresShell *aPresShell,
     // ======= end-inner-while (go through a single document) ==========
 
     // ---------- Nothing found yet, try next document  -------------
+    PRBool hasTriedFirstDoc = PR_FALSE;
     do {
       // ==== Second inner loop - get another while  ====
       if (NS_SUCCEEDED(docShellEnumerator->HasMoreElements(&hasMoreDocShells))
@@ -1342,11 +1346,15 @@ nsTypeAheadFind::FindItNow(nsIPresShell *aPresShell,
           break;
         }
       }
+      else if (hasTriedFirstDoc) {  // Avoid potential infinite loop
+        return NS_ERROR_FAILURE;  // No content doc shells
+      }
 
       // Reached last doc shell, loop around back to first doc shell
       rootContentDocShell->GetDocShellEnumerator(nsIDocShellTreeItem::typeContent,
                                                  nsIDocShell::ENUMERATE_FORWARDS,
                                                  getter_AddRefs(docShellEnumerator));
+      hasTriedFirstDoc = PR_TRUE;
     } while (docShellEnumerator);  // ==== end second inner while  ===
 
     PRBool continueLoop = PR_FALSE;
@@ -1801,17 +1809,16 @@ nsTypeAheadFind::StartNewFind(nsIDOMWindow *aWindow, PRBool aLinksOnly)
 
   // This routine will set up the doc listeners
   // Do it first, it does a CancelFind()
-  nsCOMPtr<nsIDOMWindow> startWindow;
-  UseInWindow(aWindow, getter_AddRefs(startWindow));
+  UseInWindow(aWindow);
 
   mLinksOnly = aLinksOnly;
   mLinksOnlyManuallySet = PR_TRUE;
   mRepeatingMode = eRepeatingNone;
 
   PRBool isAutoStartWin;
-  GetAutoStart(startWindow, &isAutoStartWin);
+  GetAutoStart(mFocusedWindow, &isAutoStartWin);
   if (!isAutoStartWin) {
-    AttachWindowListeners(startWindow);
+    AttachWindowListeners(mFocusedWindow);
   }
 
   if (mFocusedDocSelection) {
@@ -1924,7 +1931,8 @@ nsTypeAheadFind::GetAutoStart(nsIDOMWindow *aDOMWin, PRBool *aIsAutoStartOn)
   NS_ENSURE_TRUE(doc, NS_OK);
 
   nsCOMPtr<nsIDOMXULDocument> xulDoc(do_QueryInterface(doc));
-  if (xulDoc) {
+  nsCOMPtr<nsIImageDocument> imageDoc(do_QueryInterface(doc));
+  if (xulDoc || imageDoc) {
     return NS_OK; // Avoid any xul docs, whether in chrome or content
   }
 
@@ -2243,9 +2251,14 @@ nsTypeAheadFind::RemoveWindowListeners(nsIDOMWindow *aDOMWin)
   }
 
   // Use capturing, otherwise the normal find next will get activated when ours should
-  chromeEventHandler->RemoveEventListener(NS_LITERAL_STRING("keypress"),
-                                          NS_STATIC_CAST(nsIDOMKeyListener*, this),
-                                          PR_FALSE);
+  nsCOMPtr<nsIDOMEventReceiver> receiver(do_QueryInterface(chromeEventHandler));
+  nsCOMPtr<nsIDOMEventGroup> systemGroup;
+  receiver->GetSystemEventGroup(getter_AddRefs(systemGroup));
+  nsCOMPtr<nsIDOM3EventTarget> target(do_QueryInterface(receiver));
+
+  target->RemoveGroupedEventListener(NS_LITERAL_STRING("keypress"),
+                                     NS_STATIC_CAST(nsIDOMKeyListener*, this),
+                                     PR_FALSE, systemGroup);
 
   if (aDOMWin == mFocusedWindow) {
     mFocusedWindow = nsnull;
@@ -2291,9 +2304,14 @@ nsTypeAheadFind::AttachWindowListeners(nsIDOMWindow *aDOMWin)
   }
 
   // Use capturing, otherwise the normal find next will get activated when ours should
-  chromeEventHandler->AddEventListener(NS_LITERAL_STRING("keypress"),
-                                       NS_STATIC_CAST(nsIDOMKeyListener*, this),
-                                       PR_FALSE);
+  nsCOMPtr<nsIDOMEventReceiver> receiver(do_QueryInterface(chromeEventHandler));
+  nsCOMPtr<nsIDOMEventGroup> systemGroup;
+  receiver->GetSystemEventGroup(getter_AddRefs(systemGroup));
+  nsCOMPtr<nsIDOM3EventTarget> target(do_QueryInterface(receiver));
+
+  target->AddGroupedEventListener(NS_LITERAL_STRING("keypress"),
+                                  NS_STATIC_CAST(nsIDOMKeyListener*, this),
+                                  PR_FALSE, systemGroup);
 
   // Attach menu listeners, this will help us ignore keystrokes meant for menus
   nsIDOMEventListener *genericEventListener = 
@@ -2341,6 +2359,45 @@ nsTypeAheadFind::GetChromeEventHandler(nsIDOMWindow *aDOMWin,
   NS_IF_ADDREF(*aChromeTarget);
 }
 
+PRBool
+nsTypeAheadFind::IsTargetContentOkay(nsIContent *aContent)
+{
+  if (!aContent) {
+    return PR_FALSE;
+  }
+
+  if (aContent->IsContentOfType(nsIContent::eHTML_FORM_CONTROL)) {
+    nsCOMPtr<nsIFormControl> formControl(do_QueryInterface(aContent));
+    PRInt32 controlType = formControl->GetType();
+    if (controlType == NS_FORM_SELECT || 
+        controlType == NS_FORM_TEXTAREA ||
+        controlType == NS_FORM_INPUT_TEXT ||
+        controlType == NS_FORM_INPUT_PASSWORD ||
+        controlType == NS_FORM_INPUT_FILE) {
+      // Don't steal keys from these form controls 
+      // - selects have their own incremental find for options
+      // - text fields need to allow typing
+      return PR_FALSE;
+    }
+  }
+  else if (aContent->IsContentOfType(nsIContent::eHTML)) {
+    // Test for isindex, a deprecated kind of text field. We're using a string 
+    // compare because <isindex> is not considered a form control, so it does 
+    // not support nsIFormControl or eHTML_FORM_CONTROL, and it's not worth 
+    // having a table of atoms just for it. Instead, we're paying for 1 extra 
+    // string compare per keystroke, which isn't too bad.
+    nsCOMPtr<nsIAtom> targetTagAtom;
+    aContent->GetTag(*getter_AddRefs(targetTagAtom));
+    nsAutoString targetTagString;
+    targetTagAtom->ToString(targetTagString);
+    if (targetTagString.Equals(NS_LITERAL_STRING("isindex"))) {
+      return PR_FALSE;
+    }
+  }
+
+  return PR_TRUE;
+}
+
 
 nsresult
 nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent, 
@@ -2366,45 +2423,13 @@ nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent,
 
   // ---- Exit early if in form controls that can be typed in ---------
 
-  if (!targetContent) {
+  if (!IsTargetContentOkay(targetContent)) {
+    if (!mTypeAheadBuffer.IsEmpty()) {
+      CancelFind();
+    }
     return NS_OK;
   }
 
-  if (targetContent->IsContentOfType(nsIContent::eHTML_FORM_CONTROL)) {
-    nsCOMPtr<nsIFormControl> formControl(do_QueryInterface(targetContent));
-    PRInt32 controlType = formControl->GetType();
-    if (controlType == NS_FORM_SELECT || 
-        controlType == NS_FORM_TEXTAREA ||
-        controlType == NS_FORM_INPUT_TEXT ||
-        controlType == NS_FORM_INPUT_PASSWORD ||
-        controlType == NS_FORM_INPUT_FILE) {
-      // Don't steal keys from these form controls 
-      // - selects have their own incremental find for options
-      // - text fields need to allow typing
-      if (!mTypeAheadBuffer.IsEmpty()) {
-        CancelFind();
-      }
-      return NS_OK;
-    }
-  }
-  else if (targetContent->IsContentOfType(nsIContent::eHTML)) {
-    // Test for isindex, a deprecated kind of text field. We're using a string 
-    // compare because <isindex> is not considered a form control, so it does 
-    // not support nsIFormControl or eHTML_FORM_CONTROL, and it's not worth 
-    // having a table of atoms just for it. Instead, we're paying for 1 extra 
-    // string compare per keystroke, which isn't too bad.
-    nsCOMPtr<nsIAtom> targetTagAtom;
-    targetContent->GetTag(*getter_AddRefs(targetTagAtom));
-    nsAutoString targetTagString;
-    targetTagAtom->ToString(targetTagString);
-    if (targetTagString.Equals(NS_LITERAL_STRING("isindex"))) {
-      if (!mTypeAheadBuffer.IsEmpty())  {
-        CancelFind();
-      }
-      return NS_OK;
-    }
-  }
-
   NS_ADDREF(*aTargetContent = targetContent);
 
   // ---------- Is the keystroke in a new window? -------------------
@@ -2420,18 +2445,6 @@ nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent,
   nsCOMPtr<nsIDOMWindow> topContentWin;
   GetStartWindow(domWin, getter_AddRefs(topContentWin));
 
-  if (topContentWin != mFocusedWindow) {
-    mFocusedWindow = topContentWin;
-    GetAutoStart(topContentWin, &mIsFindAllowedInWindow);
-    if (mIsFindAllowedInWindow) {
-      UseInWindow(topContentWin, getter_AddRefs(mFocusedWindow));
-    }
-  }
-  if (!mIsFindAllowedInWindow) {
-    return NS_OK;
-  }
-
-
   // ---------- Get presshell -----------
 
   nsCOMPtr<nsIPresShell> presShell;
@@ -2442,6 +2455,20 @@ nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent,
 
   nsCOMPtr<nsIPresShell> lastShell(do_QueryReferent(mFocusedWeakShell));
 
+  if (lastShell != presShell || topContentWin != mFocusedWindow) {
+    GetAutoStart(topContentWin, &mIsFindAllowedInWindow);
+    if (mIsFindAllowedInWindow) {
+      UseInWindow(topContentWin);
+    }
+    else {
+      CancelFind();
+      mFocusedWindow = nsnull;
+    }
+  }
+  if (!mIsFindAllowedInWindow) {
+    return NS_OK;
+  }
+
   nsCOMPtr<nsIPresContext> presContext;
   presShell->GetPresContext(getter_AddRefs(presContext));
   nsCOMPtr<nsIPrintPreviewContext> printPreview(do_QueryInterface(presContext));
@@ -2456,11 +2483,6 @@ nsTypeAheadFind::GetTargetIfTypeAheadOkay(nsIDOMEvent *aEvent,
     return NS_OK;
   }
   
-  if (lastShell != presShell) {
-    // Same window, but a new document, so start fresh
-    UseInWindow(domWin, getter_AddRefs(mFocusedWindow));
-  }
-
   NS_ADDREF(*aTargetPresShell = presShell);  
   return NS_OK;
 }
@@ -2801,7 +2823,47 @@ nsTypeAheadController::nsTypeAheadController(nsIFocusController *aFocusControlle
 NS_IMETHODIMP
 nsTypeAheadController::IsCommandEnabled(const char *aCommand, PRBool *aResult)
 {
-  return SupportsCommand(aCommand, aResult);
+  NS_ENSURE_ARG_POINTER(aResult);
+
+  *aResult = PR_FALSE;
+
+  NS_ENSURE_TRUE(mFocusController, NS_ERROR_FAILURE);
+
+  nsCOMPtr<nsIDOMElement> focusedElement;
+  mFocusController->GetFocusedElement(getter_AddRefs(focusedElement));
+
+  nsCOMPtr<nsIContent> focusedContent(do_QueryInterface(focusedElement));
+
+  // Make sure we're not focused on a text field, listbox
+  // or other form control that needs typeahead keystrokes
+  if (focusedContent) {
+    *aResult = nsTypeAheadFind::IsTargetContentOkay(focusedContent);
+    return NS_OK;
+  }
+
+  // We're focused on a document
+  nsCOMPtr<nsIDOMWindowInternal> winInternal;
+  mFocusController->GetFocusedWindow(getter_AddRefs(winInternal));
+  nsCOMPtr<nsIDOMWindow> domWin(do_QueryInterface(winInternal));
+  if (!domWin) {
+    return NS_OK;
+  }
+
+  *aResult = PR_TRUE;
+
+  // Make sure we're not in Midas editing mode
+  nsCOMPtr<nsIDOMDocument> domDoc;
+  domWin->GetDocument(getter_AddRefs(domDoc));
+  nsCOMPtr<nsIDOMNSHTMLDocument> htmlDoc(do_QueryInterface(domDoc));
+  if (htmlDoc) {
+    nsAutoString designMode;
+    htmlDoc->GetDesignMode(designMode);
+    if (designMode.Equals(NS_LITERAL_STRING("on"))) {
+      *aResult = PR_FALSE;
+    }
+  }
+  
+  return NS_OK;
 }
 
 NS_IMETHODIMP
