@@ -20,6 +20,8 @@
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
+ *   Tomi Leppikangas <tomi.leppikangas@oulu.fi>
+ *   Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>
  *
  *
  * Alternatively, the contents of this file may be used under the terms of
@@ -36,19 +38,25 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-#include "nsFontMetricsGTK.h"
 #include "nsRenderingContextGTK.h"
 #include "nsRegionGTK.h"
 #include "nsImageGTK.h"
 #include "nsGraphicsStateGTK.h"
 #include "nsCompressedCharMap.h"
-#include "nsXFont.h"
 #include <math.h>
 #include "nsGCCache.h"
 #include <gtk/gtk.h>
+#include <gdk/gdkx.h>
 #include "prmem.h"
+#include "prenv.h"
 
-#define NS_TO_GDK_RGB(ns) (ns & 0xff) << 16 | (ns & 0xff00) | ((ns >> 16) & 0xff)
+#include "nsIFontMetricsGTK.h"
+#include "nsDeviceContextGTK.h"
+#include "nsFontMetricsUtils.h"
+
+#ifdef MOZ_WIDGET_GTK2
+#include <gdk/gdkwindow.h>
+#endif
 
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsRenderingContextGTK, nsIRenderingContext)
 
@@ -66,15 +74,12 @@ static nsGCCache *gcCache = nsnull;
 
 nsRenderingContextGTK::nsRenderingContextGTK()
 {
-  NS_INIT_REFCNT();
-
   mFontMetrics = nsnull;
   mContext = nsnull;
   mSurface = nsnull;
   mOffscreenSurface = nsnull;
   mCurrentColor = NS_RGB(255, 255, 255);  // set it to white
   mCurrentLineStyle = nsLineStyle_kSolid;
-  mCurrentFont = nsnull;
   mTranMatrix = nsnull;
   mP2T = 1.0f;
   mClipRegion = nsnull;
@@ -205,6 +210,9 @@ NS_IMETHODIMP nsRenderingContextGTK::GetHints(PRUint32& aResult)
   // XXX see if we are rendering to the local display or to a remote
   // dispaly and set the NS_RENDERING_HINT_REMOTE_RENDERING accordingly
 
+  // see if the font metrics has anything more to offer
+  result |= NS_FontMetricsGetHints();
+
   aResult = result;
   return NS_OK;
 }
@@ -217,7 +225,7 @@ NS_IMETHODIMP nsRenderingContextGTK::LockDrawingSurface(PRInt32 aX, PRInt32 aY,
   PushState();
 
   return mSurface->Lock(aX, aY, aWidth, aHeight,
-  			aBits, aStride, aWidthBytes, aFlags);
+                        aBits, aStride, aWidthBytes, aFlags);
 }
 
 NS_IMETHODIMP nsRenderingContextGTK::UnlockDrawingSurface(void)
@@ -407,22 +415,13 @@ NS_IMETHODIMP nsRenderingContextGTK::GetClipRect(nsRect &aRect, PRBool &aClipVal
 }
 
 #ifdef DEBUG
-#undef TRACE_SET_CLIP
+// #define TRACE_SET_CLIP
 #endif
 
 #ifdef TRACE_SET_CLIP
 static char *
 nsClipCombine_to_string(nsClipCombine aCombine)
 {
-#ifdef TRACE_SET_CLIP
-  printf("nsRenderingContextGTK::SetClipRect(x=%d,y=%d,w=%d,h=%d,%s)\n",
-         trect.x,
-         trect.y,
-         trect.width,
-         trect.height,
-         nsClipCombine_to_string(aCombine));
-#endif // TRACE_SET_CLIP
-
   switch(aCombine)
     {
       case nsClipCombine_kIntersect:
@@ -450,9 +449,17 @@ NS_IMETHODIMP nsRenderingContextGTK::SetClipRect(const nsRect& aRect,
                                                  nsClipCombine aCombine,
                                                  PRBool &aClipEmpty)
 {
+  nsRect trect = aRect;
+  mTranMatrix->TransformCoord(&trect.x, &trect.y,
+                              &trect.width, &trect.height);
+  SetClipRectInPixels(trect, aCombine, aClipEmpty);
+  return NS_OK;
+}
 
-
-
+void nsRenderingContextGTK::SetClipRectInPixels(const nsRect& aRect,
+                                                nsClipCombine aCombine,
+                                                PRBool &aClipEmpty)
+{
   PRUint32 cnt = mStateCache.Count();
   nsGraphicsState *state = nsnull;
 
@@ -472,29 +479,24 @@ NS_IMETHODIMP nsRenderingContextGTK::SetClipRect(const nsRect& aRect,
 
   CreateClipRegion();
 
-  nsRect trect = aRect;
-
 #ifdef TRACE_SET_CLIP
   printf("nsRenderingContextGTK::SetClipRect(%s)\n",
          nsClipCombine_to_string(aCombine));
 #endif // TRACE_SET_CLIP
 
-  mTranMatrix->TransformCoord(&trect.x, &trect.y,
-                           &trect.width, &trect.height);
-
   switch(aCombine)
   {
     case nsClipCombine_kIntersect:
-      mClipRegion->Intersect(trect.x,trect.y,trect.width,trect.height);
+      mClipRegion->Intersect(aRect.x,aRect.y,aRect.width,aRect.height);
       break;
     case nsClipCombine_kUnion:
-      mClipRegion->Union(trect.x,trect.y,trect.width,trect.height);
+      mClipRegion->Union(aRect.x,aRect.y,aRect.width,aRect.height);
       break;
     case nsClipCombine_kSubtract:
-      mClipRegion->Subtract(trect.x,trect.y,trect.width,trect.height);
+      mClipRegion->Subtract(aRect.x,aRect.y,aRect.width,aRect.height);
       break;
     case nsClipCombine_kReplace:
-      mClipRegion->SetTo(trect.x,trect.y,trect.width,trect.height);
+      mClipRegion->SetTo(aRect.x,aRect.y,aRect.width,aRect.height);
       break;
   }
 #if 0
@@ -504,8 +506,6 @@ NS_IMETHODIMP nsRenderingContextGTK::SetClipRect(const nsRect& aRect,
   SetColor(color);
 #endif
   aClipEmpty = mClipRegion->IsEmpty();
-
-  return NS_OK;
 }
 
 void nsRenderingContextGTK::UpdateGC()
@@ -518,13 +518,19 @@ void nsRenderingContextGTK::UpdateGC()
 
   memset(&values, 0, sizeof(GdkGCValues));
 
-  values.foreground.pixel = gdk_rgb_xpixel_from_rgb(NS_TO_GDK_RGB(mCurrentColor));
+  values.foreground.pixel =
+    gdk_rgb_xpixel_from_rgb(NS_TO_GDK_RGB(mCurrentColor));
   valuesMask = GDK_GC_FOREGROUND;
 
-  if ((mCurrentFont) && (mCurrentFont->GetGDKFont())) {
-    valuesMask = GdkGCValuesMask(valuesMask | GDK_GC_FONT);
-    values.font = mCurrentFont->GetGDKFont();
+#ifdef MOZ_ENABLE_COREXFONTS
+  if (mFontMetrics) {
+    GdkFont *font = mFontMetrics->GetCurrentGDKFont();
+    if (font) {
+      valuesMask = GdkGCValuesMask(valuesMask | GDK_GC_FONT);
+      values.font = font;
+    }
   }
+#endif
 
   valuesMask = GdkGCValuesMask(valuesMask | GDK_GC_LINE_STYLE);
   values.line_style = mLineStyle;
@@ -654,10 +660,10 @@ NS_IMETHODIMP nsRenderingContextGTK::GetColor(nscolor &aColor) const
   return NS_OK;
 }
 
-NS_IMETHODIMP nsRenderingContextGTK::SetFont(const nsFont& aFont)
+NS_IMETHODIMP nsRenderingContextGTK::SetFont(const nsFont& aFont, nsIAtom* aLangGroup)
 {
   nsCOMPtr<nsIFontMetrics> newMetrics;
-  nsresult rv = mContext->GetMetricsFor(aFont, *getter_AddRefs(newMetrics));
+  nsresult rv = mContext->GetMetricsFor(aFont, aLangGroup, *getter_AddRefs(newMetrics));
   if (NS_SUCCEEDED(rv)) {
     rv = SetFont(newMetrics);
   }
@@ -667,16 +673,9 @@ NS_IMETHODIMP nsRenderingContextGTK::SetFont(const nsFont& aFont)
 NS_IMETHODIMP nsRenderingContextGTK::SetFont(nsIFontMetrics *aFontMetrics)
 {
   NS_IF_RELEASE(mFontMetrics);
-  mFontMetrics = aFontMetrics;
+  mFontMetrics = NS_REINTERPRET_CAST(nsIFontMetricsGTK *, aFontMetrics);
   NS_IF_ADDREF(mFontMetrics);
 
-  if (mFontMetrics)
-  {
-    nsFontHandle  fontHandle;
-    mFontMetrics->GetFontHandle(fontHandle);
-    mCurrentFont = (nsFontGTK*) fontHandle;
-  }
-
   return NS_OK;
 }
 
@@ -768,7 +767,7 @@ NS_IMETHODIMP nsRenderingContextGTK::GetCurrentTransform(nsTransform2D *&aTransf
   return NS_OK;
 }
 
-NS_IMETHODIMP nsRenderingContextGTK::CreateDrawingSurface(nsRect *aBounds,
+NS_IMETHODIMP nsRenderingContextGTK::CreateDrawingSurface(const nsRect &aBounds,
                                                           PRUint32 aSurfFlags,
                                                           nsDrawingSurface &aSurface)
 {
@@ -777,8 +776,7 @@ NS_IMETHODIMP nsRenderingContextGTK::CreateDrawingSurface(nsRect *aBounds,
     return NS_ERROR_FAILURE;
   }
 
-  g_return_val_if_fail (aBounds != NULL, NS_ERROR_FAILURE);
-  g_return_val_if_fail ((aBounds->width > 0) && (aBounds->height > 0), NS_ERROR_FAILURE);
+  g_return_val_if_fail ((aBounds.width > 0) && (aBounds.height > 0), NS_ERROR_FAILURE);
  
   nsresult rv = NS_OK;
   nsDrawingSurfaceGTK *surf = new nsDrawingSurfaceGTK();
@@ -786,8 +784,12 @@ NS_IMETHODIMP nsRenderingContextGTK::CreateDrawingSurface(nsRect *aBounds,
   if (surf)
   {
     NS_ADDREF(surf);
+    PushState();
+    mClipRegion = nsnull;
     UpdateGC();
-    rv = surf->Init(mGC, aBounds->width, aBounds->height, aSurfFlags);    
+    rv = surf->Init(mGC, aBounds.width, aBounds.height, aSurfFlags);
+    PRBool empty;
+    PopState(empty);
   } else {
     rv = NS_ERROR_FAILURE;
   }
@@ -870,7 +872,7 @@ NS_IMETHODIMP nsRenderingContextGTK::DrawPolyline(const nsPoint aPoints[], PRInt
   g_return_val_if_fail(mSurface != NULL, NS_ERROR_FAILURE);
 
   GdkPoint *pts = new GdkPoint[aNumPoints];
-	for (i = 0; i < aNumPoints; i++)
+  for (i = 0; i < aNumPoints; i++)
   {
     nsPoint p = aPoints[i];
     mTranMatrix->TransformCoord(&p.x,&p.y);
@@ -1016,13 +1018,13 @@ NS_IMETHODIMP nsRenderingContextGTK::DrawPolygon(const nsPoint aPoints[], PRInt3
   g_return_val_if_fail(mSurface != NULL, NS_ERROR_FAILURE);
 
   GdkPoint *pts = new GdkPoint[aNumPoints];
-	for (PRInt32 i = 0; i < aNumPoints; i++)
+  for (PRInt32 i = 0; i < aNumPoints; i++)
   {
     nsPoint p = aPoints[i];
-		mTranMatrix->TransformCoord(&p.x,&p.y);
-		pts[i].x = p.x;
+    mTranMatrix->TransformCoord(&p.x,&p.y);
+    pts[i].x = p.x;
     pts[i].y = p.y;
-	}
+  }
 
   UpdateGC();
 
@@ -1039,13 +1041,13 @@ NS_IMETHODIMP nsRenderingContextGTK::FillPolygon(const nsPoint aPoints[], PRInt3
   g_return_val_if_fail(mSurface != NULL, NS_ERROR_FAILURE);
 
   GdkPoint *pts = new GdkPoint[aNumPoints];
-	for (PRInt32 i = 0; i < aNumPoints; i++)
+  for (PRInt32 i = 0; i < aNumPoints; i++)
   {
     nsPoint p = aPoints[i];
-		mTranMatrix->TransformCoord(&p.x,&p.y);
-		pts[i].x = p.x;
+    mTranMatrix->TransformCoord(&p.x,&p.y);
+    pts[i].x = p.x;
     pts[i].y = p.y;
-	}
+  }
 
   UpdateGC();
 
@@ -1105,6 +1107,15 @@ NS_IMETHODIMP nsRenderingContextGTK::FillEllipse(nscoord aX, nscoord aY, nscoord
 
   UpdateGC();
 
+  if (w < 16 || h < 16) {
+    /* Fix for bug 91816 ("bullets are not displayed correctly on certain text zooms")
+     * De-uglify bullets on some X servers:
+     * 1st: Draw... */
+    ::gdk_draw_arc(mSurface->GetDrawable(), mGC, FALSE,
+                   x, y, w, h,
+                   0, 360 * 64);
+    /*  ...then fill. */
+  }
   ::gdk_draw_arc(mSurface->GetDrawable(), mGC, TRUE,
                  x, y, w, h,
                  0, 360 * 64);
@@ -1177,77 +1188,6 @@ NS_IMETHODIMP nsRenderingContextGTK::FillArc(nscoord aX, nscoord aY,
   return NS_OK;
 }
 
-// do the 8 to 16 bit conversion on the stack
-// if the data is less than this size
-#define WIDEN_8_TO_16_BUF_SIZE 1024
-
-// handle 8 bit data with a 16 bit font
-gint
-Widen8To16AndMove(const gchar *char_p, 
-                  gint char_len, 
-                  XChar2b *xchar2b_p)
-{
-  int i;
-  for (i=0; i<char_len; i++) {
-    (xchar2b_p)->byte1 = 0;
-    (xchar2b_p++)->byte2 = *char_p++;
-  }
-  return(char_len*2);
-}
-
-// handle 8 bit data with a 16 bit font
-gint
-Widen8To16AndGetWidth (nsXFont        *xFont,
-                       const gchar    *text,
-                       gint            text_length)
-{
-  NS_ASSERTION(!xFont->IsSingleByte(),"wrong string/font size");
-  XChar2b buf[WIDEN_8_TO_16_BUF_SIZE];
-  XChar2b *p = buf;
-  int uchar_size;
-  gint rawWidth;
-
-  if (text_length > WIDEN_8_TO_16_BUF_SIZE) {
-    p = (XChar2b*)PR_Malloc(text_length*sizeof(XChar2b));
-    if (!p) return(0); // handle malloc failure
-  }
-
-  uchar_size = Widen8To16AndMove(text, text_length, p);
-  rawWidth = xFont->TextWidth16(p, uchar_size/2);
-
-  if (text_length > WIDEN_8_TO_16_BUF_SIZE) {
-    PR_Free((char*)p);
-  }
-  return(rawWidth);
-}
-
-/* static */ void
-nsRenderingContextGTK::Widen8To16AndDraw (GdkDrawable *drawable,
-                                          nsXFont     *xFont,
-                                          GdkGC       *gc,
-                                          gint         x,
-                                          gint         y,
-                                          const gchar *text,
-                                          gint         text_length)
-{
-  NS_ASSERTION(!xFont->IsSingleByte(),"wrong string/font size");
-  XChar2b buf[WIDEN_8_TO_16_BUF_SIZE];
-  XChar2b *p = buf;
-  int uchar_size;
-
-  if (text_length > WIDEN_8_TO_16_BUF_SIZE) {
-    p = (XChar2b*)PR_Malloc(text_length*sizeof(XChar2b));
-    if (!p) return; // handle malloc failure
-  }
-
-  uchar_size = Widen8To16AndMove(text, text_length, p);
-  xFont->DrawText16(drawable, gc, x, y, p, uchar_size/2);
-
-  if (text_length > WIDEN_8_TO_16_BUF_SIZE) {
-    PR_Free((char*)p);
-  }
-}
-
 NS_IMETHODIMP
 nsRenderingContextGTK::GetWidth(char aC, nscoord &aWidth)
 {
@@ -1285,25 +1225,12 @@ nsRenderingContextGTK::GetWidth(const char* aString, PRUint32 aLength,
 {
   if (0 == aLength) {
     aWidth = 0;
+    return NS_OK;
   }
-  else {
-    g_return_val_if_fail(aString != NULL, NS_ERROR_FAILURE);
-    g_return_val_if_fail(mCurrentFont != NULL, NS_ERROR_FAILURE);
-    gint rawWidth;
-    nsXFont *xFont = mCurrentFont->GetXFont();
-    if (!mCurrentFont->GetXFontIs10646()) {
-      NS_ASSERTION(xFont->IsSingleByte(),"wrong string/font size");
-      // 8 bit data with an 8 bit font
-      rawWidth = xFont->TextWidth8(aString, aLength);
-    }
-    else {
-      NS_ASSERTION(!xFont->IsSingleByte(),"wrong string/font size");
-      // we have 8 bit data but a 16 bit font
-      rawWidth = Widen8To16AndGetWidth (mCurrentFont->GetXFont(), aString, aLength);
-    }
-    aWidth = NSToCoordRound(rawWidth * mP2T);
-  }
-  return NS_OK;
+
+  g_return_val_if_fail(aString != NULL, NS_ERROR_FAILURE);
+
+  return mFontMetrics->GetWidth(aString, aLength, aWidth, this);
 }
 
 NS_IMETHODIMP
@@ -1312,56 +1239,12 @@ nsRenderingContextGTK::GetWidth(const PRUnichar* aString, PRUint32 aLength,
 {
   if (0 == aLength) {
     aWidth = 0;
+    return NS_OK;
   }
-  else {
-    g_return_val_if_fail(aString != NULL, NS_ERROR_FAILURE);
 
-    nsFontMetricsGTK* metrics = (nsFontMetricsGTK*) mFontMetrics;
+  g_return_val_if_fail(aString != NULL, NS_ERROR_FAILURE);
 
-    g_return_val_if_fail(metrics != NULL, NS_ERROR_FAILURE);
-
-    nsFontGTK* prevFont = nsnull;
-    gint rawWidth = 0;
-    PRUint32 start = 0;
-    PRUint32 i;
-    for (i = 0; i < aLength; i++) {
-      PRUnichar c = aString[i];
-      nsFontGTK* currFont = nsnull;
-      nsFontGTK** font = metrics->mLoadedFonts;
-      nsFontGTK** end = &metrics->mLoadedFonts[metrics->mLoadedFontsCount];
-      while (font < end) {
-        if (CCMAP_HAS_CHAR((*font)->mCCMap, c)) {
-          currFont = *font;
-          goto FoundFont; // for speed -- avoid "if" statement
-        }
-        font++;
-      }
-      currFont = metrics->FindFont(c);
-FoundFont:
-      // XXX avoid this test by duplicating code -- erik
-      if (prevFont) {
-        if (currFont != prevFont) {
-          rawWidth += prevFont->GetWidth(&aString[start], i - start);
-          prevFont = currFont;
-          start = i;
-        }
-      }
-      else {
-        prevFont = currFont;
-        start = i;
-      }
-    }
-
-    if (prevFont) {
-      rawWidth += prevFont->GetWidth(&aString[start], i - start);
-    }
-
-    aWidth = NSToCoordRound(rawWidth * mP2T);
-  }
-  if (nsnull != aFontID)
-    *aFontID = 0;
-
-  return NS_OK;
+  return mFontMetrics->GetWidth(aString, aLength, aWidth, aFontID, this);
 }
 
 NS_IMETHODIMP
@@ -1374,69 +1257,48 @@ nsRenderingContextGTK::GetTextDimensions(const char* aString, PRUint32 aLength,
 }
 
 NS_IMETHODIMP
-nsRenderingContextGTK::GetTextDimensions(const PRUnichar* aString, PRUint32 aLength,
-                                         nsTextDimensions& aDimensions, PRInt32* aFontID)
+nsRenderingContextGTK::GetTextDimensions(const PRUnichar* aString,
+                                         PRUint32 aLength,
+                                         nsTextDimensions& aDimensions, 
+                                         PRInt32* aFontID)
 {
-  aDimensions.Clear();
-  if (0 < aLength) {
-    g_return_val_if_fail(aString != NULL, NS_ERROR_FAILURE);
+  return mFontMetrics->GetTextDimensions(aString, aLength, aDimensions,
+                                         aFontID, this);
+}
 
-    nsFontMetricsGTK* metrics = (nsFontMetricsGTK*) mFontMetrics;
-
-    g_return_val_if_fail(metrics != NULL, NS_ERROR_FAILURE);
-
-    nsFontGTK* prevFont = nsnull;
-    gint rawWidth = 0, rawAscent = 0, rawDescent = 0;
-    PRUint32 start = 0;
-    PRUint32 i;
-    for (i = 0; i < aLength; i++) {
-      PRUnichar c = aString[i];
-      nsFontGTK* currFont = nsnull;
-      nsFontGTK** font = metrics->mLoadedFonts;
-      nsFontGTK** end = &metrics->mLoadedFonts[metrics->mLoadedFontsCount];
-      while (font < end) {
-        if (CCMAP_HAS_CHAR((*font)->mCCMap, c)) {
-          currFont = *font;
-          goto FoundFont; // for speed -- avoid "if" statement
-        }
-        font++;
-      }
-      currFont = metrics->FindFont(c);
-FoundFont:
-      // XXX avoid this test by duplicating code -- erik
-      if (prevFont) {
-        if (currFont != prevFont) {
-          rawWidth += prevFont->GetWidth(&aString[start], i - start);
-          if (rawAscent < prevFont->mMaxAscent)
-            rawAscent = prevFont->mMaxAscent;
-          if (rawDescent < prevFont->mMaxDescent)
-            rawDescent = prevFont->mMaxDescent;
-          prevFont = currFont;
-          start = i;
-        }
-      }
-      else {
-        prevFont = currFont;
-        start = i;
-      }
-    }
-
-    if (prevFont) {
-      rawWidth += prevFont->GetWidth(&aString[start], i - start);
-      if (rawAscent < prevFont->mMaxAscent)
-        rawAscent = prevFont->mMaxAscent;
-      if (rawDescent < prevFont->mMaxDescent)
-        rawDescent = prevFont->mMaxDescent;
-    }
-
-    aDimensions.width = NSToCoordRound(rawWidth * mP2T);
-    aDimensions.ascent = NSToCoordRound(rawAscent * mP2T);
-    aDimensions.descent = NSToCoordRound(rawDescent * mP2T);
-  }
-  if (nsnull != aFontID)
-    *aFontID = 0;
-
-  return NS_OK;
+NS_IMETHODIMP
+nsRenderingContextGTK::GetTextDimensions(const char*       aString,
+                                         PRInt32           aLength,
+                                         PRInt32           aAvailWidth,
+                                         PRInt32*          aBreaks,
+                                         PRInt32           aNumBreaks,
+                                         nsTextDimensions& aDimensions,
+                                         PRInt32&          aNumCharsFit,
+                                         nsTextDimensions& aLastWordDimensions,
+                                         PRInt32*          aFontID)
+{
+  return mFontMetrics->GetTextDimensions(aString, aLength, aAvailWidth,
+                                         aBreaks, aNumBreaks, aDimensions,
+                                         aNumCharsFit,
+                                         aLastWordDimensions, aFontID,
+                                         this);
+}
+NS_IMETHODIMP
+nsRenderingContextGTK::GetTextDimensions(const PRUnichar*  aString,
+                                         PRInt32           aLength,
+                                         PRInt32           aAvailWidth,
+                                         PRInt32*          aBreaks,
+                                         PRInt32           aNumBreaks,
+                                         nsTextDimensions& aDimensions,
+                                         PRInt32&          aNumCharsFit,
+                                         nsTextDimensions& aLastWordDimensions,
+                                         PRInt32*          aFontID)
+{
+  return mFontMetrics->GetTextDimensions(aString, aLength, aAvailWidth,
+                                         aBreaks, aNumBreaks, aDimensions,
+                                         aNumCharsFit,
+                                         aLastWordDimensions, aFontID,
+                                         this);
 }
 
 NS_IMETHODIMP
@@ -1444,91 +1306,8 @@ nsRenderingContextGTK::DrawString(const char *aString, PRUint32 aLength,
                                   nscoord aX, nscoord aY,
                                   const nscoord* aSpacing)
 {
-  if (0 != aLength) {
-    g_return_val_if_fail(mTranMatrix != NULL, NS_ERROR_FAILURE);
-    g_return_val_if_fail(mSurface != NULL, NS_ERROR_FAILURE);
-    g_return_val_if_fail(aString != NULL, NS_ERROR_FAILURE);
-    g_return_val_if_fail(mCurrentFont != NULL, NS_ERROR_FAILURE);
-
-    nscoord x = aX;
-    nscoord y = aY;
-
-    UpdateGC();
-
-    nsXFont *xFont = mCurrentFont->GetXFont();
-    if (nsnull != aSpacing) {
-      // Render the string, one character at a time...
-      const char* end = aString + aLength;
-      while (aString < end) {
-        char ch = *aString++;
-        nscoord xx = x;
-        nscoord yy = y;
-        mTranMatrix->TransformCoord(&xx, &yy);
-        if (!mCurrentFont->GetXFontIs10646()) {
-          // 8 bit data with an 8 bit font
-          NS_ASSERTION(xFont->IsSingleByte(),"wrong string/font size");
-          xFont->DrawText8(mSurface->GetDrawable(), mGC, xx, yy, &ch, 1);
-        }
-        else {
-          // we have 8 bit data but a 16 bit font
-          NS_ASSERTION(!xFont->IsSingleByte(),"wrong string/font size");
-          Widen8To16AndDraw(mSurface->GetDrawable(), xFont, mGC,
-                                                xx, yy, &ch, 1);
-        }
-        x += *aSpacing++;
-      }
-    }
-    else {
-      mTranMatrix->TransformCoord(&x, &y);
-      if (!mCurrentFont->GetXFontIs10646()) { // keep 8 bit path fast
-        // 8 bit data with an 8 bit font
-        NS_ASSERTION(xFont->IsSingleByte(),"wrong string/font size");
-        xFont->DrawText8(mSurface->GetDrawable(), mGC, x, y, aString, aLength);
-        }
-      else {
-        // we have 8 bit data but a 16 bit font
-        NS_ASSERTION(!xFont->IsSingleByte(),"wrong string/font size");
-        Widen8To16AndDraw(mSurface->GetDrawable(), xFont, mGC,
-                                             x, y, aString, aLength);
-      }
-    }
-  }
-
-#if 0
-  //this is no longer to be done by this API, but another
-  //will take it's place that will need this code again. MMP
-  if (mFontMetrics)
-  {
-    const nsFont *font;
-    mFontMetrics->GetFont(font);
-    PRUint8 deco = font->decorations;
-
-    if (deco & NS_FONT_DECORATION_OVERLINE)
-      DrawLine(aX, aY, aX + aWidth, aY);
-
-    if (deco & NS_FONT_DECORATION_UNDERLINE)
-    {
-      nscoord ascent,descent;
-
-      mFontMetrics->GetMaxAscent(ascent);
-      mFontMetrics->GetMaxDescent(descent);
-
-      DrawLine(aX, aY + ascent + (descent >> 1),
-               aX + aWidth, aY + ascent + (descent >> 1));
-    }
-
-    if (deco & NS_FONT_DECORATION_LINE_THROUGH)
-    {
-      nscoord height;
-
-	  mFontMetrics->GetHeight(height);
-
-      DrawLine(aX, aY + (height >> 1), aX + aWidth, aY + (height >> 1));
-    }
-  }
-#endif
-
-  return NS_OK;
+  return mFontMetrics->DrawString(aString, aLength, aX, aY, aSpacing,
+                                  this, mSurface);
 }
 
 NS_IMETHODIMP
@@ -1537,102 +1316,8 @@ nsRenderingContextGTK::DrawString(const PRUnichar* aString, PRUint32 aLength,
                                   PRInt32 aFontID,
                                   const nscoord* aSpacing)
 {
-  if (aLength && mFontMetrics) {
-    g_return_val_if_fail(mTranMatrix != NULL, NS_ERROR_FAILURE);
-    g_return_val_if_fail(mSurface != NULL, NS_ERROR_FAILURE);
-    g_return_val_if_fail(aString != NULL, NS_ERROR_FAILURE);
-
-    nscoord x = aX;
-    nscoord y = aY;
-
-    mTranMatrix->TransformCoord(&x, &y);
-
-    nsFontMetricsGTK* metrics = (nsFontMetricsGTK*) mFontMetrics;
-    nsFontGTK* prevFont = nsnull;
-    PRUint32 start = 0;
-    PRUint32 i;
-    for (i = 0; i < aLength; i++) {
-      PRUnichar c = aString[i];
-      nsFontGTK* currFont = nsnull;
-      nsFontGTK** font = metrics->mLoadedFonts;
-      nsFontGTK** lastFont = &metrics->mLoadedFonts[metrics->mLoadedFontsCount];
-      while (font < lastFont) {
-        if (CCMAP_HAS_CHAR((*font)->mCCMap, c)) {
-          currFont = *font;
-          goto FoundFont; // for speed -- avoid "if" statement
-        }
-        font++;
-      }
-      currFont = metrics->FindFont(c);
-FoundFont:
-      // XXX avoid this test by duplicating code -- erik
-      if (prevFont) {
-        if (currFont != prevFont) {
-          if (aSpacing) {
-            const PRUnichar* str = &aString[start];
-            const PRUnichar* end = &aString[i];
-
-            // save off mCurrentFont and set it so that we cache the GC's font correctly
-            nsFontGTK *oldFont = mCurrentFont;
-            mCurrentFont = prevFont;
-            UpdateGC();
-
-            while (str < end) {
-              x = aX;
-              y = aY;
-              mTranMatrix->TransformCoord(&x, &y);
-              prevFont->DrawString(this, mSurface, x, y, str, 1);
-              aX += *aSpacing++;
-              str++;
-            }
-            mCurrentFont = oldFont;
-          }
-          else {
-            nsFontGTK *oldFont = mCurrentFont;
-            mCurrentFont = prevFont;
-            UpdateGC();
-            x += prevFont->DrawString(this, mSurface, x, y, &aString[start],
-                                      i - start);
-            mCurrentFont = oldFont;
-          }
-          prevFont = currFont;
-          start = i;
-        }
-      }
-      else {
-        prevFont = currFont;
-        start = i;
-      }
-    }
-
-    if (prevFont) {
-
-      nsFontGTK *oldFont = mCurrentFont;
-      mCurrentFont = prevFont;
-      UpdateGC();
-    
-      if (aSpacing) {
-        const PRUnichar* str = &aString[start];
-        const PRUnichar* end = &aString[i];
-        while (str < end) {
-          x = aX;
-          y = aY;
-          mTranMatrix->TransformCoord(&x, &y);
-          prevFont->DrawString(this, mSurface, x, y, str, 1);
-          aX += *aSpacing++;
-          str++;
-        }
-
-      }
-      else {
-        prevFont->DrawString(this, mSurface, x, y, &aString[start], i - start);
-      }
-
-      mCurrentFont = oldFont;
-    }
-  }
-
-  return NS_OK;
+  return mFontMetrics->DrawString(aString, aLength, aX, aY, aFontID,
+                                  aSpacing, this, mSurface);
 }
 
 NS_IMETHODIMP
@@ -1645,96 +1330,6 @@ nsRenderingContextGTK::DrawString(const nsString& aString,
                     aX, aY, aFontID, aSpacing);
 }
 
-NS_IMETHODIMP nsRenderingContextGTK::DrawImage(nsIImage *aImage,
-                                               nscoord aX, nscoord aY)
-{
-  nscoord width, height;
-
-  // we have to do this here because we are doing a transform below
-  width = NSToCoordRound(mP2T * aImage->GetWidth());
-  height = NSToCoordRound(mP2T * aImage->GetHeight());
-
-  return DrawImage(aImage, aX, aY, width, height);
-}
-
-NS_IMETHODIMP nsRenderingContextGTK::DrawImage(nsIImage *aImage, const nsRect& aRect)
-{
-  return DrawImage(aImage,
-                   aRect.x,
-                   aRect.y,
-                   aRect.width,
-                   aRect.height);
-}
-
-NS_IMETHODIMP nsRenderingContextGTK::DrawImage(nsIImage *aImage,
-                                               nscoord aX, nscoord aY,
-                                               nscoord aWidth, nscoord aHeight)
-{
-  nscoord x, y, w, h;
-
-  x = aX;
-  y = aY;
-  w = aWidth;
-  h = aHeight;
-
-  mTranMatrix->TransformCoord(&x, &y, &w, &h);
-
-#if 0
-  //  gdk_window_clear_area(mSurface->GetDrawable(), x, y, w, h);
-  PRInt32 xx, yy, ww, hh;
-  mClipRegion->GetBoundingBox(&xx,&yy,&ww,&hh);
-  printf("clip bounds: x = %i, y = %i, w = %i, h = %i\n", xx, yy, ww, hh);
-
-  nscolor color = mCurrentColor;
-  SetColor(NS_RGB(255,   0,   0));
-  FillRect(xx, yy, ww, hh);
-  SetColor(color);
-#endif
-
-  UpdateGC();
-
-  return aImage->Draw(*this, mSurface,
-                      x, y, w, h);
-}
-
-NS_IMETHODIMP nsRenderingContextGTK::DrawImage(nsIImage *aImage,
-                                               const nsRect& aSRect,
-                                               const nsRect& aDRect)
-{
-  nsRect	sr,dr;
-
-  sr = aSRect;
-  mTranMatrix->TransformCoord(&sr.x, &sr.y,
-                            &sr.width, &sr.height);
-  sr.x -= mTranMatrix->GetXTranslationCoord();
-  sr.y -= mTranMatrix->GetYTranslationCoord();
-
-  dr = aDRect;
-  mTranMatrix->TransformCoord(&dr.x, &dr.y,
-                           &dr.width, &dr.height);
-
-#if 0
-  PRInt32 x, y, w, h;
-  mClipRegion->GetBoundingBox(&x,&y,&w,&h);
-  printf("clip bounds: x = %i, y = %i, w = %i, h = %i\n", x, y, w, h);
-
-  //  gdk_window_clear_area(mSurface->GetDrawable(), sr.x, sr.y, sr.width, sr.height);
-
-  nscolor color = mCurrentColor;
-  SetColor(NS_RGB(255,   0,   0));
-  FillRect(x, y, w, h);
-  SetColor(color);
-#endif
-
-  UpdateGC();
-
-  return aImage->Draw(*this, mSurface,
-                      sr.x, sr.y,
-                      sr.width, sr.height,
-                      dr.x, dr.y,
-                      dr.width, dr.height);
-}
-
 NS_IMETHODIMP
 nsRenderingContextGTK::CopyOffScreenBits(nsDrawingSurface aSrcSurf,
                                          PRInt32 aSrcX, PRInt32 aSrcY,
@@ -1774,7 +1369,10 @@ nsRenderingContextGTK::CopyOffScreenBits(nsDrawingSurface aSrcSurf,
     destsurf = mSurface;
   }
   else
+  {
+    NS_ENSURE_TRUE(mOffscreenSurface != nsnull, NS_ERROR_FAILURE);
     destsurf = mOffscreenSurface;
+  }
 
   if (aCopyFlags & NS_COPYBITS_XFORM_SOURCE_VALUES)
     mTranMatrix->TransformCoord(&srcX, &srcY);
@@ -1818,85 +1416,13 @@ NS_IMETHODIMP nsRenderingContextGTK::RetrieveCurrentNativeGraphicData(PRUint32 *
 
 #ifdef MOZ_MATHML
 
-void
-Widen8To16AndGetTextExtents (nsXFont *xFont,  
-                        const gchar *text,
-                        gint         text_length,
-                        gint        *lbearing,
-                        gint        *rbearing,
-                        gint        *width,
-                        gint        *ascent,
-                        gint        *descent)
-{
-  NS_ASSERTION(!xFont->IsSingleByte(),"wrong string/font size");
-  XChar2b buf[WIDEN_8_TO_16_BUF_SIZE];
-  XChar2b *p = buf;
-  int uchar_size;
-
-  if (text_length > WIDEN_8_TO_16_BUF_SIZE) {
-    p = (XChar2b*)PR_Malloc(text_length*sizeof(XChar2b));
-    if (!p) { // handle malloc failure
-      *lbearing = 0;
-      *rbearing = 0;
-      *width    = 0;
-      *ascent   = 0;
-      *descent  = 0;
-      return;
-    }
-  }
-
-  uchar_size = Widen8To16AndMove(text, text_length, p);
-  xFont->TextExtents16(p, uchar_size/2,
-                    lbearing, 
-                    rbearing, 
-                    width, 
-                    ascent, 
-                    descent);
-
-  if (text_length > WIDEN_8_TO_16_BUF_SIZE) {
-    PR_Free((char*)p);
-  }
-}
-
 NS_IMETHODIMP
 nsRenderingContextGTK::GetBoundingMetrics(const char*        aString, 
                                           PRUint32           aLength,
                                           nsBoundingMetrics& aBoundingMetrics)
 {
-  aBoundingMetrics.Clear();
-  if (aString && 0 < aLength) {
-    g_return_val_if_fail(aString != NULL, NS_ERROR_FAILURE);
-    g_return_val_if_fail(mCurrentFont != NULL, NS_ERROR_FAILURE);
-    nsXFont *xFont = mCurrentFont->GetXFont();
-    if (!mCurrentFont->GetXFontIs10646()) {
-        // 8 bit data with an 8 bit font
-        NS_ASSERTION(xFont->IsSingleByte(),"wrong string/font size");
-        xFont->TextExtents8(aString, aLength,
-                                &aBoundingMetrics.leftBearing, 
-                                &aBoundingMetrics.rightBearing, 
-                                &aBoundingMetrics.width, 
-                                &aBoundingMetrics.ascent, 
-                                &aBoundingMetrics.descent);
-    }
-    else {
-        // we have 8 bit data but a 16 bit font
-        NS_ASSERTION(!xFont->IsSingleByte(),"wrong string/font size");
-        Widen8To16AndGetTextExtents (mCurrentFont->GetXFont(), aString, aLength,
-                          &aBoundingMetrics.leftBearing, 
-                          &aBoundingMetrics.rightBearing, 
-                          &aBoundingMetrics.width, 
-                          &aBoundingMetrics.ascent, 
-                          &aBoundingMetrics.descent);
-    }
-
-    aBoundingMetrics.leftBearing = NSToCoordRound(aBoundingMetrics.leftBearing * mP2T);
-    aBoundingMetrics.rightBearing = NSToCoordRound(aBoundingMetrics.rightBearing * mP2T);
-    aBoundingMetrics.width = NSToCoordRound(aBoundingMetrics.width * mP2T);
-    aBoundingMetrics.ascent = NSToCoordRound(aBoundingMetrics.ascent * mP2T);
-    aBoundingMetrics.descent = NSToCoordRound(aBoundingMetrics.descent * mP2T);
-  }
-
-  return NS_OK;
+  return mFontMetrics->GetBoundingMetrics(aString, aLength, aBoundingMetrics,
+                                          this);
 }
 
 NS_IMETHODIMP
@@ -1905,137 +1431,12 @@ nsRenderingContextGTK::GetBoundingMetrics(const PRUnichar*   aString,
                                           nsBoundingMetrics& aBoundingMetrics,
                                           PRInt32*           aFontID)
 {
-  aBoundingMetrics.Clear(); 
-  if (0 < aLength) {
-    g_return_val_if_fail(aString != NULL, NS_ERROR_FAILURE);
-
-    nsFontMetricsGTK* metrics = (nsFontMetricsGTK*) mFontMetrics;
-    nsFontGTK* prevFont = nsnull;
-
-    nsBoundingMetrics rawbm;
-    PRBool firstTime = PR_TRUE;
-    PRUint32 start = 0;
-    PRUint32 i;
-    for (i = 0; i < aLength; i++) {
-      PRUnichar c = aString[i];
-      nsFontGTK* currFont = nsnull;
-      nsFontGTK** font = metrics->mLoadedFonts;
-      nsFontGTK** end = &metrics->mLoadedFonts[metrics->mLoadedFontsCount];
-      while (font < end) {
-        if (CCMAP_HAS_CHAR((*font)->mCCMap, c)) {
-          currFont = *font;
-          goto FoundFont; // for speed -- avoid "if" statement
-        }
-        font++;
-      }
-      currFont = metrics->FindFont(c);
-    FoundFont:
-      // XXX avoid this test by duplicating code -- erik
-      if (prevFont) {
-        if (currFont != prevFont) {
-          prevFont->GetBoundingMetrics((const PRUnichar*) &aString[start],
-                                       i - start, rawbm);
-          if (firstTime) {
-            firstTime = PR_FALSE;
-            aBoundingMetrics = rawbm;
-          } 
-          else {
-            aBoundingMetrics += rawbm;
-          }
-          prevFont = currFont;
-          start = i;
-        }
-      }
-      else {
-        prevFont = currFont;
-        start = i;
-      }
-    }
-    
-    if (prevFont) {
-      prevFont->GetBoundingMetrics((const PRUnichar*) &aString[start],
-                                   i - start, rawbm);
-      if (firstTime) {
-        aBoundingMetrics = rawbm;
-      }
-      else {
-        aBoundingMetrics += rawbm;
-      }
-    }
-    // convert to app units
-    aBoundingMetrics.leftBearing = NSToCoordRound(aBoundingMetrics.leftBearing * mP2T);
-    aBoundingMetrics.rightBearing = NSToCoordRound(aBoundingMetrics.rightBearing * mP2T);
-    aBoundingMetrics.width = NSToCoordRound(aBoundingMetrics.width * mP2T);
-    aBoundingMetrics.ascent = NSToCoordRound(aBoundingMetrics.ascent * mP2T);
-    aBoundingMetrics.descent = NSToCoordRound(aBoundingMetrics.descent * mP2T);
-  }
-  if (nsnull != aFontID)
-    *aFontID = 0;
-
-  return NS_OK;
+  return mFontMetrics->GetBoundingMetrics(aString, aLength, aBoundingMetrics,
+                                          aFontID, this);
 }
+
 #endif /* MOZ_MATHML */
 
-
-/* static */ void
-nsRenderingContextGTK::my_gdk_draw_text (GdkDrawable *drawable,
-                                         GdkFont     *font,
-                                         GdkGC       *gc,
-                                         gint         x,
-                                         gint         y,
-                                         const gchar *text,
-                                         gint         text_length)
-{
-  GdkWindowPrivate *drawable_private;
-  GdkFontPrivate *font_private;
-  GdkGCPrivate *gc_private;
-
-  g_return_if_fail (drawable != NULL);
-  g_return_if_fail (font != NULL);
-  g_return_if_fail (gc != NULL);
-  g_return_if_fail (text != NULL);
-
-  drawable_private = (GdkWindowPrivate*) drawable;
-  if (drawable_private->destroyed)
-    return;
-  gc_private = (GdkGCPrivate*) gc;
-  font_private = (GdkFontPrivate*) font;
-
-  if (font->type == GDK_FONT_FONT)
-  {
-    XFontStruct *xfont = (XFontStruct *) font_private->xfont;
-
-    // gdk does this... we don't need it..
-    //    XSetFont(drawable_private->xdisplay, gc_private->xgc, xfont->fid);
-
-    // We clamp the sizes down to 32768 which is the maximum width of
-    // a window.  Even if a font was 1 pixel high and started at the
-    // left, the maximum size of a draw request could only be 32k.
-
-    if ((xfont->min_byte1 == 0) && (xfont->max_byte1 == 0))
-    {
-      XDrawString (drawable_private->xdisplay, drawable_private->xwindow,
-                   gc_private->xgc, x, y, text, MIN(text_length, 32768));
-    }
-    else
-    {
-      XDrawString16 (drawable_private->xdisplay, drawable_private->xwindow,
-                     gc_private->xgc, x, y, (XChar2b *) text, 
-                     MIN((text_length / 2), 32768));
-    }
-  }
-  else if (font->type == GDK_FONT_FONTSET)
-  {
-    XFontSet fontset = (XFontSet) font_private->xfont;
-    XmbDrawString (drawable_private->xdisplay, drawable_private->xwindow,
-                   fontset, gc_private->xgc, x, y, text, text_length);
-  }
-  else
-    g_error("undefined font type\n");
-}
-
-
-
 NS_IMETHODIMP nsRenderingContextGTK::DrawImage(imgIContainer *aImage, const nsRect * aSrcRect, const nsPoint * aDestPoint)
 {
   UpdateGC();
