@@ -78,10 +78,12 @@
 #include "nsIContentSink.h"
 #include "nsIHTMLContentSink.h"
 #include "nsHTMLTokenizer.h"
-#include "nsHTMLEntities.h"
-#include "nsIPref.h"
+#include "nsIPrefService.h"
+#include "nsIPrefBranch.h"
 #include "nsUnicharUtils.h"
 
+#include "nsIServiceManager.h"
+
 #include "COtherDTD.h"
 #include "nsElementTable.h"
 
@@ -111,8 +113,8 @@ static NS_DEFINE_IID(kClassIID,     NS_VIEWSOURCE_HTML_IID);
 // bug 22022 - these are used to toggle 'Wrap Long Lines' on the viewsource
 // window by selectively setting/unsetting the following class defined in
 // viewsource.css; the setting is remembered between invocations using a pref.
-static const char* kBodyId = "viewsource";
-static const char* kBodyClassWrap = "wrap";
+static const char kBodyId[] = "viewsource";
+static const char kBodyClassWrap[] = "wrap";
 
 /**
  *  This method gets called as part of our COM-like interfaces.
@@ -327,8 +329,6 @@ static const char* const kDumpFileAfterText[] = {
  *  @return  
  */
 CViewSourceHTML::CViewSourceHTML() : mFilename(), mTags(), mErrors() {
-  NS_INIT_ISUPPORTS();
-
   mStartTag = VIEW_SOURCE_START_TAG;
   mEndTag = VIEW_SOURCE_END_TAG;
   mCommentTag = VIEW_SOURCE_COMMENT;
@@ -344,14 +344,14 @@ CViewSourceHTML::CViewSourceHTML() : mFilename(), mTags(), mErrors() {
   mPopupTag = VIEW_SOURCE_POPUP;
   mSyntaxHighlight = PR_FALSE;
   mWrapLongLines = PR_FALSE;
-  nsCOMPtr<nsIPref> thePrefsService(do_GetService(NS_PREF_CONTRACTID));
-  if (thePrefsService) {
+  nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID));
+  if (prefBranch) {
     PRBool temp;
     nsresult rv;
-    rv = thePrefsService->GetBoolPref("view_source.syntax_highlight", &temp);
+    rv = prefBranch->GetBoolPref("view_source.syntax_highlight", &temp);
     mSyntaxHighlight = NS_SUCCEEDED(rv) ? temp : PR_TRUE;
 
-    rv = thePrefsService->GetBoolPref("view_source.wrap_long_lines", &temp);
+    rv = prefBranch->GetBoolPref("view_source.wrap_long_lines", &temp);
     mWrapLongLines = NS_SUCCEEDED(rv) ? temp : PR_FALSE;
   }
 
@@ -854,16 +854,6 @@ CViewSourceHTML::IntTagToStringTag(PRInt32 aIntTag) const
   return str_ptr;
 }
 
-NS_IMETHODIMP
-CViewSourceHTML::ConvertEntityToUnicode(const nsAString& aEntity,
-                                        PRInt32* aUnicode) const
-{
-  *aUnicode = nsHTMLEntities::EntityToUnicode(aEntity);
-
-  return NS_OK;
-}
-
-
 PRBool CViewSourceHTML::IsBlockElement(PRInt32 aTagID,PRInt32 aParentID) const {
   PRBool result=PR_FALSE;
   return result;
