@@ -47,7 +47,7 @@
  * JS number type and wrapper class.
  */
 #include "jsstddef.h"
-#ifdef XP_PC
+#if defined(XP_WIN) || defined(XP_OS2)
 #include <float.h>
 #endif
 #include <math.h>
@@ -438,12 +438,18 @@ static JSConstDoubleSpec number_constants[] = {
 
 static jsdouble NaN;
 
-#if !defined __MWERKS__ && defined XP_PC && (defined _M_IX86 || defined __GNUC__)
+
+#if (defined XP_WIN || defined XP_OS2) &&                                     \
+    !defined __MWERKS__ &&                                                    \
+    (defined _M_IX86 ||                                                       \
+    (defined __GNUC__ && !defined __MINGW32__ && !defined __EMX__))
 
 /*
+ * Set the exception mask to mask all exceptions and set the FPU precision
+ * to 53 bit mantissa.
  * On Alpha platform this is handled via Compiler option.
  */
-#define FIX_FPU() _control87(MCW_EM, MCW_EM)
+#define FIX_FPU() _control87(MCW_EM | PC_53, MCW_EM | MCW_PC)
 
 #else
 
