@@ -37,9 +37,10 @@
  * ***** END LICENSE BLOCK ***** */
 #include "nsContainerFrame.h"
 #include "nsIContent.h"
+#include "nsIDocument.h"
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsRect.h"
 #include "nsPoint.h"
 #include "nsGUIEvent.h"
@@ -47,7 +48,6 @@
 #include "nsIView.h"
 #include "nsIScrollableView.h"
 #include "nsVoidArray.h"
-#include "nsISizeOfHandler.h"
 #include "nsHTMLReflowCommand.h"
 #include "nsHTMLContainerFrame.h"
 #include "nsIFrameManager.h"
@@ -84,7 +84,7 @@ NS_IMETHODIMP
 nsContainerFrame::Init(nsIPresContext*  aPresContext,
                        nsIContent*      aContent,
                        nsIFrame*        aParent,
-                       nsIStyleContext* aContext,
+                       nsStyleContext*  aContext,
                        nsIFrame*        aPrevInFlow)
 {
   nsresult rv;
@@ -110,9 +110,7 @@ nsContainerFrame::SetInitialChildList(nsIPresContext* aPresContext,
   if (!mFrames.IsEmpty()) {
     // We already have child frames which means we've already been
     // initialized
-#ifdef DEBUG_dbaron // XXX Fix asserts and remove this ifdef.
     NS_NOTREACHED("unexpected second call to SetInitialChildList");
-#endif
     result = NS_ERROR_UNEXPECTED;
   } else if (aListName) {
     // All we know about is the unnamed principal child list
@@ -444,6 +442,10 @@ nsContainerFrame::PositionFrameView(nsIPresContext* aPresContext,
     // the frame tree of some enclosing document. We do nothing in that case,
     // but we have to check that containingView is nonnull or we will crash.
     if (nsnull != containingView && containingView != parentView) {
+      NS_ERROR("This hack should not be needed now!!! See bug 126263.");
+
+      // XXX roc this is no longer needed; remove it!!
+
       // it is possible for parent view not to have a frame attached to it
       // kind of an anonymous view. This happens with native scrollbars and
       // the clip view. To fix this we need to go up and parentView chain
@@ -499,10 +501,42 @@ nsContainerFrame::PositionFrameView(nsIPresContext* aPresContext,
   }
 }
 
+static PRBool
+NonZeroStyleCoord(const nsStyleCoord& aCoord) {
+  switch (aCoord.GetUnit()) {
+  case eStyleUnit_Percent:
+    return aCoord.GetPercentValue() > 0;
+  case eStyleUnit_Coord:
+    return aCoord.GetCoordValue() > 0;
+  case eStyleUnit_Null:
+    return PR_FALSE;
+  default:
+    return PR_TRUE;
+  }
+}
+
+static PRBool
+HasNonZeroBorderRadius(nsStyleContext* aStyleContext) {
+  const nsStyleBorder* border;
+  ::GetStyleData(aStyleContext, &border);
+
+  nsStyleCoord coord;
+  border->mBorderRadius.GetTop(coord);
+  if (NonZeroStyleCoord(coord)) return PR_TRUE;    
+  border->mBorderRadius.GetRight(coord);
+  if (NonZeroStyleCoord(coord)) return PR_TRUE;    
+  border->mBorderRadius.GetBottom(coord);
+  if (NonZeroStyleCoord(coord)) return PR_TRUE;    
+  border->mBorderRadius.GetLeft(coord);
+  if (NonZeroStyleCoord(coord)) return PR_TRUE;    
+
+  return PR_FALSE;
+}
+
 static void
 SyncFrameViewGeometryDependentProperties(nsIPresContext*  aPresContext,
                                          nsIFrame*        aFrame,
-                                         nsIStyleContext* aStyleContext,
+                                         nsStyleContext*  aStyleContext,
                                          nsIView*         aView,
                                          PRUint32         aFlags)
 {
@@ -525,16 +559,44 @@ SyncFrameViewGeometryDependentProperties(nsIPresContext*  aPresContext,
   PRBool  viewHasTransparentContent =
     !hasBG ||
     (bg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT) ||
-    !aFrame->CanPaintBackground();
-  if (isCanvas && viewHasTransparentContent) {
+    !aFrame->CanPaintBackground() ||
+    HasNonZeroBorderRadius(aStyleContext);
+
+  if (isCanvas) {
     nsIView* rootView;
     vm->GetRootView(rootView);
     nsIView* rootParent;
     rootView->GetParent(rootParent);
-    if (nsnull == rootParent) {
+    if (!rootParent) {
+      // We're the root of a view manager hierarchy. We will have to
+      // paint something. NOTE: this can be overridden below.
       viewHasTransparentContent = PR_FALSE;
     }
+
+    nsCOMPtr<nsIPresShell> shell;
+    aPresContext->GetShell(getter_AddRefs(shell));
+    nsCOMPtr<nsIDocument> doc;
+    shell->GetDocument(getter_AddRefs(doc));
+    if (doc) {
+      nsCOMPtr<nsIDocument> parentDoc;
+      doc->GetParentDocument(getter_AddRefs(parentDoc));
+      nsCOMPtr<nsIContent> rootElem;
+      doc->GetRootContent(getter_AddRefs(rootElem));
+      if (!parentDoc && rootElem && rootElem->IsContentOfType(nsIContent::eXUL)) {
+        // we're XUL at the root of the document hierarchy. Try to make our
+        // window translucent.
+        nsCOMPtr<nsIWidget> widget;
+        aView->GetWidget(*getter_AddRefs(widget));
+        // don't proceed unless this is the root view
+        // (sometimes the non-root-view is a canvas)
+        if (widget && aView == rootView) {
+          viewHasTransparentContent = hasBG && (bg->mBackgroundFlags & NS_STYLE_BG_COLOR_TRANSPARENT);
+          widget->SetWindowTranslucency(viewHasTransparentContent);
+        }
+      }
+    }
   }
+  // XXX we should also set widget transparency for XUL popups
 
   const nsStyleDisplay* display;
   ::GetStyleData(aStyleContext, &display);
@@ -677,23 +739,12 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
     // area, then size the view large enough to include those child
     // frames
     if ((kidState & NS_FRAME_OUTSIDE_CHILDREN) && aCombinedArea) {
-      vm->ResizeView(aView, *aCombinedArea);
+      vm->ResizeView(aView, *aCombinedArea, PR_TRUE);
     } else {
-      // If the width is unchanged and the height is not decreased
-      // then repaint only the newly exposed or contracted area,
-      // otherwise repaint the union of the old and new areas
-
-      // XXX: We currently invalidate the newly exposed areas only
-      // when the container frame's width is unchanged and the height
-      // is either unchanged or increased This is because some frames
-      // do not invalidate themselves properly. see bug 73825.  Once
-      // bug 73825 is fixed, we should always pass PR_TRUE instead of
-      // frameSize.width == width && frameSize.height >= height.
       nsSize frameSize;
       aFrame->GetSize(frameSize);
       nsRect newSize(0, 0, frameSize.width, frameSize.height);
-      vm->ResizeView(aView, newSize, 
-                     (frameSize.width == oldBounds.width && frameSize.height >= oldBounds.height));
+      vm->ResizeView(aView, newSize, PR_TRUE);
     }
 
     // Even if the size hasn't changed, we need to sync up the
@@ -702,8 +753,7 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
     // detect whether it has or not. Likewise, whether the view size
     // has changed or not, we may need to change the transparency
     // state even if there is no clip.
-    nsCOMPtr<nsIStyleContext> savedStyleContext;
-    aFrame->GetStyleContext(getter_AddRefs(savedStyleContext));
+    nsStyleContext* savedStyleContext = aFrame->GetStyleContext();
     SyncFrameViewGeometryDependentProperties(aPresContext, aFrame, savedStyleContext, aView, aFlags);
   }
 }
@@ -711,7 +761,7 @@ nsContainerFrame::SyncFrameViewAfterReflow(nsIPresContext* aPresContext,
 void
 nsContainerFrame::SyncFrameViewAfterSizeChange(nsIPresContext*  aPresContext,
                                                nsIFrame*        aFrame,
-                                               nsIStyleContext* aStyleContext,
+                                               nsStyleContext*  aStyleContext,
                                                nsIView*         aView,
                                                PRUint32         aFlags)
 {
@@ -719,10 +769,8 @@ nsContainerFrame::SyncFrameViewAfterSizeChange(nsIPresContext*  aPresContext,
     return;
   }
   
-  nsCOMPtr<nsIStyleContext> savedStyleContext;
   if (nsnull == aStyleContext) {
-    aFrame->GetStyleContext(getter_AddRefs(savedStyleContext));
-    aStyleContext = savedStyleContext;
+    aStyleContext = aFrame->GetStyleContext();
   }
 
   SyncFrameViewGeometryDependentProperties(aPresContext, aFrame, aStyleContext, aView, aFlags);
@@ -731,7 +779,7 @@ nsContainerFrame::SyncFrameViewAfterSizeChange(nsIPresContext*  aPresContext,
 void
 nsContainerFrame::SyncFrameViewProperties(nsIPresContext*  aPresContext,
                                           nsIFrame*        aFrame,
-                                          nsIStyleContext* aStyleContext,
+                                          nsStyleContext*  aStyleContext,
                                           nsIView*         aView,
                                           PRUint32         aFlags)
 {
@@ -742,10 +790,8 @@ nsContainerFrame::SyncFrameViewProperties(nsIPresContext*  aPresContext,
   nsCOMPtr<nsIViewManager> vm;
   aView->GetViewManager(*getter_AddRefs(vm));
 
-  nsCOMPtr<nsIStyleContext> savedStyleContext;
   if (nsnull == aStyleContext) {
-    aFrame->GetStyleContext(getter_AddRefs(savedStyleContext));
-    aStyleContext = savedStyleContext;
+    aStyleContext = aFrame->GetStyleContext();
   }
     
   const nsStyleVisibility* vis;
@@ -762,26 +808,13 @@ nsContainerFrame::SyncFrameViewProperties(nsIPresContext*  aPresContext,
     if (NS_STYLE_VISIBILITY_COLLAPSE == vis->mVisible) {
       viewIsVisible = PR_FALSE;
     }
-    else if (NS_STYLE_VISIBILITY_HIDDEN == vis->mVisible) {
-      // If it has a widget, hide the view because the widget can't deal with it
-      nsCOMPtr<nsIWidget> widget;
-      aView->GetWidget(*getter_AddRefs(widget));
-      if (widget) {
-        viewIsVisible = PR_FALSE;
-      }
-      else {
-        // If it's a scroll frame or a list control frame which is derived from the scrollframe, 
-        // then hide the view. This means that
-        // child elements can't override their parent's visibility, but
-        // it's not practical to leave it visible in all cases because
-        // the scrollbars will be showing
-        nsCOMPtr<nsIAtom> frameType;
-        aFrame->GetFrameType(getter_AddRefs(frameType));
-
-        if (frameType == nsLayoutAtoms::scrollFrame || frameType == nsLayoutAtoms::listControlFrame) {
-          viewIsVisible = PR_FALSE;
-        }
-      }
+    else if (NS_STYLE_VISIBILITY_HIDDEN == vis->mVisible &&
+             !aFrame->SupportsVisibilityHidden()) {
+      // If it's a scrollable frame that can't hide its scrollbars,
+      // hide the view. This means that child elements can't override
+      // their parent's visibility, but it's not practical to leave it
+      // visible in all cases because the scrollbars will be showing
+      viewIsVisible = PR_FALSE;
     } else {
       // if the view is for a popup, don't show the view if the popup is closed
       nsCOMPtr<nsIWidget> widget;
@@ -826,7 +859,7 @@ nsContainerFrame::SyncFrameViewProperties(nsIPresContext*  aPresContext,
 PRBool
 nsContainerFrame::FrameNeedsView(nsIPresContext* aPresContext,
                                  nsIFrame* aFrame,
-                                 nsIStyleContext* aStyleContext)
+                                 nsStyleContext* aStyleContext)
 {
   const nsStyleVisibility* vis;
   ::GetStyleData(aStyleContext, &vis);
@@ -854,8 +887,7 @@ nsContainerFrame::FrameNeedsView(nsIPresContext* aPresContext,
     return PR_TRUE;
   } 
 
-  nsCOMPtr<nsIAtom>  pseudoTag;
-  aStyleContext->GetPseudoType(*getter_AddRefs(pseudoTag));
+  nsCOMPtr<nsIAtom>  pseudoTag = aStyleContext->GetPseudoType();
   if (pseudoTag == nsCSSAnonBoxes::scrolledContent) {
     return PR_TRUE;
   }
@@ -902,11 +934,9 @@ nsContainerFrame::ReflowChild(nsIFrame*                aKidFrame,
   nsresult  result;
 
 #ifdef DEBUG
-  nsSize* saveMaxElementSize = aDesiredSize.maxElementSize;
 #ifdef REALLY_NOISY_MAX_ELEMENT_SIZE
-  if (nsnull != aDesiredSize.maxElementSize) {
-    aDesiredSize.maxElementSize->width = nscoord(0xdeadbeef);
-    aDesiredSize.maxElementSize->height = nscoord(0xdeadbeef);
+  if (aDesiredSize.mComputeMEW) {
+    aDesiredSize.mMaxElementWidth = nscoord(0xdeadbeef);
   }
 #endif
 #endif
@@ -928,20 +958,12 @@ nsContainerFrame::ReflowChild(nsIFrame*                aKidFrame,
                              aStatus);
 
 #ifdef DEBUG
-  if (saveMaxElementSize != aDesiredSize.maxElementSize) {
-    printf("nsContainerFrame: ");
-    nsFrame::ListTag(stdout, aKidFrame);
-    printf(" changed the maxElementSize *pointer* (baaaad boy!)\n");
-  }
 #ifdef REALLY_NOISY_MAX_ELEMENT_SIZE
-  if ((nsnull != aDesiredSize.maxElementSize) &&
-      ((nscoord(0xdeadbeef) == aDesiredSize.maxElementSize->width) ||
-       (nscoord(0xdeadbeef) == aDesiredSize.maxElementSize->height))) {
+  if (aDesiredSize.mComputeMEW &&
+      (nscoord(0xdeadbeef) == aDesiredSize.mMaxElementWidth)) {
     printf("nsContainerFrame: ");
     nsFrame::ListTag(stdout, aKidFrame);
-    printf(" didn't set max-element-size!\n");
-    aDesiredSize.maxElementSize->width = 0;
-    aDesiredSize.maxElementSize->height = 0;
+    printf(" didn't set max-element-width!\n");
   }
 #endif
 #endif
@@ -1228,7 +1250,7 @@ nsContainerFrame::PushChildren(nsIPresContext* aPresContext,
 }
 
 /**
- * Moves any frames on the overflwo lists (the prev-in-flow's overflow list and
+ * Moves any frames on the overflow lists (the prev-in-flow's overflow list and
  * the receiver's overflow list) to the child list.
  *
  * Updates this frame's child count and content mapping.
@@ -1344,14 +1366,4 @@ nsContainerFrame::List(nsIPresContext* aPresContext, FILE* out, PRInt32 aIndent)
 
   return NS_OK;
 }
-
-NS_IMETHODIMP
-nsContainerFrame::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const
-{
-  if (!aResult) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  *aResult = sizeof(*this);
-  return NS_OK;
-}
 #endif
