@@ -39,7 +39,7 @@
 #include "nsBlockFrame.h"
 #include "nsHTMLAtoms.h"
 #include "nsHTMLParts.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsIPresShell.h"
 #include "nsIPresContext.h"
 #include "nsIRenderingContext.h"
@@ -48,6 +48,7 @@
 #include "nsLayoutAtoms.h"
 #include "nsCSSAnonBoxes.h"
 #include "nsReflowPath.h"
+#include "nsAutoPtr.h"
 #ifdef ACCESSIBILITY
 #include "nsIServiceManager.h"
 #include "nsIAccessibilityService.h"
@@ -57,10 +58,8 @@
 #undef NOISY_PUSHING
 #endif
 
-nsIID nsInlineFrame::kInlineFrameCID = NS_INLINE_FRAME_CID;
-
-
 
+NS_DEFINE_IID(kInlineFrameCID, NS_INLINE_FRAME_CID);
 
 
 //////////////////////////////////////////////////////////////////////
@@ -250,13 +249,12 @@ nsInlineFrame::RemoveFrame(nsIPresContext* aPresContext,
     // Loop and destroy the frame and all of its continuations.
     PRBool generateReflowCommand = PR_FALSE;
 
-    // If the frame we are removing is a brFrame and we have a
-    // nextInFlow, we need a reflow so we can attempt to pull up
-    // any frames in the nextInFlow that can fit on the line
-    // the brFrame was on.
+    // If the frame we are removing is a brFrame, we need a reflow so
+    // the line the brFrame was on can attempt to pull up any frames
+    // that can fit from lines below it.
     nsCOMPtr<nsIAtom> frameType;
     aOldFrame->GetFrameType(getter_AddRefs(frameType));
-    if (frameType == nsLayoutAtoms::brFrame && mNextInFlow)
+    if (frameType == nsLayoutAtoms::brFrame)
       generateReflowCommand = PR_TRUE;
 
     nsIFrame* oldFrameParent;
@@ -324,6 +322,37 @@ nsInlineFrame::ReplaceFrame(nsIPresContext* aPresContext,
 }
 
 
+NS_IMETHODIMP
+nsInlineFrame::Paint(nsIPresContext*      aPresContext,
+                     nsIRenderingContext& aRenderingContext,
+                     const nsRect&        aDirtyRect,
+                     nsFramePaintLayer    aWhichLayer,
+                     PRUint32             aFlags)
+{
+  if (NS_FRAME_IS_UNFLOWABLE & mState) {
+    return NS_OK;
+  }
+
+  // Paint inline element backgrounds in the foreground layer (bug 36710).
+  if (aWhichLayer == NS_FRAME_PAINT_LAYER_FOREGROUND) {
+    PaintSelf(aPresContext, aRenderingContext, aDirtyRect);
+  }
+    
+  // The sole purpose of this is to trigger display of the selection
+  // window for Named Anchors, which don't have any children and
+  // normally don't have any size, but in Editor we use CSS to display
+  // an image to represent this "hidden" element.
+  if (!mFrames.FirstChild()) {
+    nsFrame::Paint(aPresContext, aRenderingContext, aDirtyRect,
+                   aWhichLayer, aFlags);
+  }
+
+  PaintDecorationsAndChildren(aPresContext, aRenderingContext,
+                              aDirtyRect, aWhichLayer, PR_FALSE,
+                              aFlags);
+  return NS_OK;
+}
+
 //////////////////////////////////////////////////////////////////////
 // Reflow methods
 
@@ -574,7 +603,8 @@ nsInlineFrame::ReflowFrames(nsIPresContext* aPresContext,
   // that are empty we force to empty so that things like collapsed
   // whitespace in an inline element don't affect the line-height.
   nsSize size;
-  lineLayout->EndSpan(this, size, aMetrics.maxElementSize);
+  lineLayout->EndSpan(this, size,
+                    aMetrics.mComputeMEW ? &aMetrics.mMaxElementWidth : nsnull);
   if ((0 == size.height) && (0 == size.width) &&
       ((nsnull != mPrevInFlow) || (nsnull != mNextInFlow))) {
     // This is a continuation of a previous inline. Therefore make
@@ -583,9 +613,8 @@ nsInlineFrame::ReflowFrames(nsIPresContext* aPresContext,
     aMetrics.height = 0;
     aMetrics.ascent = 0;
     aMetrics.descent = 0;
-    if (nsnull != aMetrics.maxElementSize) {
-      aMetrics.maxElementSize->width = 0;
-      aMetrics.maxElementSize->height = 0;
+    if (aMetrics.mComputeMEW) {
+      aMetrics.mMaxElementWidth = 0;
     }
   }
   else {
@@ -650,9 +679,8 @@ nsInlineFrame::ReflowFrames(nsIPresContext* aPresContext,
   ListTag(stdout);
   printf(": metrics=%d,%d ascent=%d descent=%d\n",
          aMetrics.width, aMetrics.height, aMetrics.ascent, aMetrics.descent);
-  if (nsnull != aMetrics.maxElementSize) {
-    printf(" maxElementSize %d,%d\n", 
-      aMetrics.maxElementSize->width, aMetrics.maxElementSize->height);
+  if (aMetrics.mComputeMEW) {
+    printf(" maxElementWidth %d\n", aMetrics.mMaxElementWidth);
   }
 #endif
 
@@ -863,7 +891,7 @@ nsInlineFrame::GetSkipSides() const
       skip |= 1 << NS_SIDE_LEFT;
     }
     else {
-      // If the prev-in-flow is empty, then go ahead and let our right
+      // If the prev-in-flow is empty, then go ahead and let our left
       // edge border render.
     }
   }
@@ -890,7 +918,8 @@ NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
   *aAccessible = nsnull;
   nsCOMPtr<nsIAtom> tagAtom;
   mContent->GetTag(*getter_AddRefs(tagAtom));
-  if (tagAtom == nsHTMLAtoms::img || tagAtom == nsHTMLAtoms::input || tagAtom == nsHTMLAtoms::hr) {
+  if (tagAtom == nsHTMLAtoms::img || tagAtom == nsHTMLAtoms::input || 
+     tagAtom == nsHTMLAtoms::label || tagAtom == nsHTMLAtoms::hr) {
     // Only get accessibility service if we're going to use it
     nsCOMPtr<nsIAccessibilityService> accService(do_GetService("@mozilla.org/accessibilityService;1"));
     if (!accService)
@@ -899,6 +928,8 @@ NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
       return accService->CreateHTML4ButtonAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
     else if (tagAtom == nsHTMLAtoms::img)  // Create accessible for broken <img>
       return accService->CreateHTMLImageAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
+    else if (tagAtom == nsHTMLAtoms::label)  // Creat accessible for <label>
+      return accService->CreateHTMLLabelAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
     // Create accessible for <hr>
     return accService->CreateHTMLHRAccessible(NS_STATIC_CAST(nsIFrame*, this), aAccessible);
   }
@@ -913,7 +944,7 @@ NS_IMETHODIMP nsInlineFrame::GetAccessible(nsIAccessible** aAccessible)
 
 static void
 ReParentChildListStyle(nsIPresContext* aPresContext,
-                       nsIStyleContext* aParentStyleContext,
+                       nsStyleContext* aParentStyleContext,
                        nsFrameList& aFrameList)
 {
   nsIFrame* kid = aFrameList.FirstChild();
@@ -1060,26 +1091,22 @@ nsFirstLineFrame::Reflow(nsIPresContext* aPresContext,
       // proper parent context.
       nsIFrame* parentFrame;
       first->GetParent(&parentFrame);
-      nsIStyleContext* parentContext;
-      parentFrame->GetStyleContext(&parentContext);
+      nsStyleContext* parentContext = parentFrame->GetStyleContext();
       if (parentContext) {
         // Create a new style context that is a child of the parent
         // style context thus removing the :first-line style. This way
         // we behave as if an anonymous (unstyled) span was the child
         // of the parent frame.
-        nsIStyleContext* newSC;
-        aPresContext->ResolvePseudoStyleContextFor(nsnull,
-                      nsCSSAnonBoxes::mozLineFrame, parentContext, &newSC);
+        nsRefPtr<nsStyleContext> newSC = aPresContext->ResolvePseudoStyleContextFor(nsnull,
+                                                                                    nsCSSAnonBoxes::mozLineFrame,
+                                                                                    parentContext);
         if (newSC) {
           // Switch to the new style context.
           SetStyleContext(aPresContext, newSC);
 
           // Re-resolve all children
           ReParentChildListStyle(aPresContext, mStyleContext, mFrames);
-
-          NS_RELEASE(newSC);
         }
-        NS_RELEASE(parentContext);
       }
     }
   }
@@ -1101,7 +1128,7 @@ NS_NewPositionedInlineFrame(nsIPresShell* aPresShell, nsIFrame** aNewFrame)
   if (nsnull == aNewFrame) {
     return NS_ERROR_NULL_POINTER;
   }
-  nsPositionedInlineFrame* it = new (aPresShell) nsPositionedInlineFrame;
+  nsPositionedInlineFrame* it = new (aPresShell) nsPositionedInlineFrame();
   if (nsnull == it) {
     return NS_ERROR_OUT_OF_MEMORY;
   }
@@ -1123,7 +1150,7 @@ nsPositionedInlineFrame::SetInitialChildList(nsIPresContext* aPresContext,
 {
   nsresult  rv;
 
-  if (nsLayoutAtoms::absoluteList == aListName) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     rv = mAbsoluteContainer.SetInitialChildList(this, aPresContext, aListName, aChildList);
   } else {
     rv = nsInlineFrame::SetInitialChildList(aPresContext, aListName, aChildList);
@@ -1140,7 +1167,7 @@ nsPositionedInlineFrame::AppendFrames(nsIPresContext* aPresContext,
 {
   nsresult  rv;
   
-  if (nsLayoutAtoms::absoluteList == aListName) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     rv = mAbsoluteContainer.AppendFrames(this, aPresContext, aPresShell, aListName,
                                          aFrameList);
   } else {
@@ -1160,7 +1187,7 @@ nsPositionedInlineFrame::InsertFrames(nsIPresContext* aPresContext,
 {
   nsresult  rv;
 
-  if (nsLayoutAtoms::absoluteList == aListName) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     rv = mAbsoluteContainer.InsertFrames(this, aPresContext, aPresShell, aListName,
                                          aPrevFrame, aFrameList);
   } else {
@@ -1179,7 +1206,7 @@ nsPositionedInlineFrame::RemoveFrame(nsIPresContext* aPresContext,
 {
   nsresult  rv;
 
-  if (nsLayoutAtoms::absoluteList == aListName) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     rv = mAbsoluteContainer.RemoveFrame(this, aPresContext, aPresShell, aListName, aOldFrame);
   } else {
     rv = nsInlineFrame::RemoveFrame(aPresContext, aPresShell, aListName, aOldFrame);
@@ -1195,7 +1222,7 @@ nsPositionedInlineFrame::ReplaceFrame(nsIPresContext* aPresContext,
                                       nsIFrame*       aOldFrame,
                                       nsIFrame*       aNewFrame)
 {
-  if (nsLayoutAtoms::absoluteList == aListName) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     return mAbsoluteContainer.ReplaceFrame(this, aPresContext, aPresShell,
                                            aListName, aOldFrame, aNewFrame);
   } else {
@@ -1211,7 +1238,7 @@ nsPositionedInlineFrame::GetAdditionalChildListName(PRInt32   aIndex,
   NS_PRECONDITION(nsnull != aListName, "null OUT parameter pointer");
   *aListName = nsnull;
   if (0 == aIndex) {
-    *aListName = nsLayoutAtoms::absoluteList;
+    *aListName = mAbsoluteContainer.GetChildListName();
     NS_ADDREF(*aListName);
   }
   return NS_OK;
@@ -1223,7 +1250,7 @@ nsPositionedInlineFrame::FirstChild(nsIPresContext* aPresContext,
                                     nsIFrame**      aFirstChild) const
 {
   NS_PRECONDITION(nsnull != aFirstChild, "null OUT parameter pointer");
-  if (aListName == nsLayoutAtoms::absoluteList) {
+  if (mAbsoluteContainer.GetChildListName() == aListName) {
     return mAbsoluteContainer.FirstChild(this, aListName, aFirstChild);
   }
 
@@ -1317,7 +1344,7 @@ nsPositionedInlineFrame::Reflow(nsIPresContext*          aPresContext,
 
     rv = mAbsoluteContainer.Reflow(this, aPresContext, aReflowState,
                                    containingBlockWidth, containingBlockHeight,
-                                   childBounds);
+                                   &childBounds);
     
     // XXX Although this seems like the correct thing to do the line layout
     // code seems to reset the NS_FRAME_OUTSIDE_CHILDREN and so it is ignored
@@ -1339,15 +1366,3 @@ nsPositionedInlineFrame::Reflow(nsIPresContext*          aPresContext,
 
   return rv;
 }
-
-#ifdef DEBUG
-NS_IMETHODIMP
-nsPositionedInlineFrame::SizeOf(nsISizeOfHandler* aHandler, PRUint32* aResult) const
-{
-  if (!aResult) {
-    return NS_ERROR_NULL_POINTER;
-  }
-  *aResult = sizeof(*this);
-  return NS_OK;
-}
-#endif
