@@ -47,7 +47,7 @@
 #include "nsIRenderingContext.h"
 #include "nsIPresContext.h"
 #include "nsIPresShell.h"
-#include "nsIStyleContext.h"
+#include "nsStyleContext.h"
 #include "nsLeafFrame.h"
 #include "nsCSSRendering.h"
 #include "nsISupports.h"
@@ -105,8 +105,7 @@ nsHTMLButtonControlFrame::nsHTMLButtonControlFrame()
 
   mCacheSize.width             = -1;
   mCacheSize.height            = -1;
-  mCachedMaxElementSize.width  = -1;
-  mCachedMaxElementSize.height = -1;
+  mCachedMaxElementWidth       = -1;
 }
 
 nsHTMLButtonControlFrame::~nsHTMLButtonControlFrame()
@@ -124,7 +123,7 @@ NS_IMETHODIMP
 nsHTMLButtonControlFrame::Init(nsIPresContext*  aPresContext,
               nsIContent*      aContent,
               nsIFrame*        aParent,
-              nsIStyleContext* aContext,
+              nsStyleContext*  aContext,
               nsIFrame*        aPrevInFlow)
 {
   nsresult  rv = nsHTMLContainerFrame::Init(aPresContext, aContent, aParent, aContext, aPrevInFlow);
@@ -146,13 +145,11 @@ nsHTMLButtonControlFrame::Init(nsIPresContext*  aPresContext,
   mFrames.SetFrames(areaFrame);
 
   // Resolve style and initialize the frame
-  nsIStyleContext* styleContext;
-  aPresContext->ResolvePseudoStyleContextFor(mContent,
+  nsRefPtr<nsStyleContext> styleContext;
+  styleContext = aPresContext->ResolvePseudoStyleContextFor(mContent,
                                             nsCSSAnonBoxes::buttonContent,
-                                            mStyleContext,
-                                            &styleContext);
+                                            mStyleContext);
   mFrames.FirstChild()->Init(aPresContext, mContent, this, styleContext, nsnull);
-  NS_RELEASE(styleContext);                                           
 
   return rv;
 }
@@ -206,10 +203,10 @@ NS_IMETHODIMP nsHTMLButtonControlFrame::GetAccessible(nsIAccessible** aAccessibl
 #endif
 
 
-NS_IMETHODIMP
-nsHTMLButtonControlFrame::GetType(PRInt32* aType) const
+NS_IMETHODIMP_(PRInt32)
+nsHTMLButtonControlFrame::GetType() const
 {
-  return nsFormControlHelper::GetType(mContent, aType);
+  return nsFormControlHelper::GetType(mContent);
 }
 
 NS_IMETHODIMP
@@ -334,14 +331,14 @@ nsHTMLButtonControlFrame::SetInitialChildList(nsIPresContext* aPresContext,
   //
   nsCOMPtr<nsIPresShell> shell;
   nsCOMPtr<nsIFrameManager> frameManager;
-  nsCOMPtr<nsIStyleContext> newParentContext;
+  nsStyleContext* newParentContext;
   aPresContext->GetShell(getter_AddRefs(shell));
   if (shell) {
     shell->GetFrameManager(getter_AddRefs(frameManager));
   }
   // get the new parent context from the first child: that is the frame that the
   // subsequent children will be made children of
-  mFrames.FirstChild()->GetStyleContext(getter_AddRefs(newParentContext));
+  newParentContext = mFrames.FirstChild()->GetStyleContext();
 
   // Set the parent for each of the child frames
   for (nsIFrame* frame = aChildList; nsnull != frame; frame->GetNextSibling(&frame)) {
@@ -430,7 +427,7 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
   }
 
 #if 0
-  nsresult skiprv = nsFormControlFrame::SkipResizeReflow(mCacheSize, mCachedMaxElementSize, aPresContext, 
+  nsresult skiprv = nsFormControlFrame::SkipResizeReflow(mCacheSize, mCachedMaxElementWidth, aPresContext, 
                                                          aDesiredSize, aReflowState, aStatus);
   if (NS_SUCCEEDED(skiprv)) {
     return skiprv;
@@ -453,15 +450,15 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
     GetView(&view);
     if (!view) {
       nsresult result = nsComponentManager::CreateInstance(kViewCID, nsnull, NS_GET_IID(nsIView), (void **)&view);
-	    nsCOMPtr<nsIPresShell> presShell;
+      nsCOMPtr<nsIPresShell> presShell;
       aPresContext->GetShell(getter_AddRefs(presShell));
-	    nsCOMPtr<nsIViewManager> viewMan;
+      nsCOMPtr<nsIViewManager> viewMan;
       presShell->GetViewManager(getter_AddRefs(viewMan));
 
       nsIFrame* parWithView;
-	    nsIView *parView;
+      nsIView *parView;
       GetParentWithView(&parWithView);
-	    parWithView->GetView(&parView);
+      parWithView->GetView(&parView);
       // the view's size is not know yet, but its size will be kept in synch with our frame.
       nsRect boundBox(0, 0, 500, 500); 
       result = view->Init(viewMan, boundBox, parView, nsnull);
@@ -477,11 +474,11 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
   }
 #endif
 
-  // reflow the child
+  // Reflow the child
   nsIFrame* firstKid = mFrames.FirstChild();
   nsSize availSize(aReflowState.mComputedWidth, NS_INTRINSICSIZE);
 
-  // indent the child inside us by the the focus border. We must do this separate from the
+  // Indent the child inside us by the the focus border. We must do this separate from the
   // regular border.
   nsMargin focusPadding = mRenderer.GetAddedButtonBorderAndPadding();
 
@@ -514,12 +511,82 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
     }
   }
 
-  nsHTMLReflowState reflowState(aPresContext, aReflowState, firstKid, availSize, reason);
-  //reflowState.computedWidth = availSize;
+  // Reflow the contents of the button a first time.
+  ReflowButtonContents(aPresContext, aDesiredSize, aReflowState, firstKid,
+                       availSize, reason, focusPadding, aStatus);
 
-  ReflowChild(firstKid, aPresContext, aDesiredSize, reflowState,
-              focusPadding.left + aReflowState.mComputedBorderPadding.left,
-              focusPadding.top + aReflowState.mComputedBorderPadding.top,
+  // If we just performed the first pass of a shrink-wrap reflow (which will have
+  // found the requested size of the children, but not placed them), perform
+  // the second pass of a shrink-wrap reflow (which places the children
+  // within this parent button which has now shrink-wrapped around them).
+  if (availSize.width == NS_SHRINKWRAPWIDTH) {
+    nsSize newAvailSize(aDesiredSize.width, NS_INTRINSICSIZE);
+
+    ReflowButtonContents(aPresContext, aDesiredSize, aReflowState, firstKid,
+                         newAvailSize, eReflowReason_Resize, focusPadding, aStatus);
+  }
+
+  // If computed use the computed values.
+  if (aReflowState.mComputedWidth != NS_INTRINSICSIZE) 
+    aDesiredSize.width = aReflowState.mComputedWidth;
+  else 
+    aDesiredSize.width  += focusPadding.left + focusPadding.right;
+
+  if (aReflowState.mComputedHeight != NS_INTRINSICSIZE) 
+    aDesiredSize.height = aReflowState.mComputedHeight;
+  else
+    aDesiredSize.height += focusPadding.top + focusPadding.bottom;
+
+  AddComputedBorderPaddingToDesiredSize(aDesiredSize, aReflowState);
+
+  //aDesiredSize.width  += aReflowState.mComputedBorderPadding.left + aReflowState.mComputedBorderPadding.right;
+  //aDesiredSize.height += aReflowState.mComputedBorderPadding.top + aReflowState.mComputedBorderPadding.bottom;
+
+  if (aDesiredSize.mComputeMEW) {
+    aDesiredSize.mMaxElementWidth = aDesiredSize.width;
+  }
+
+  // Make sure we obey min/max-width and min/max-height
+  if (aDesiredSize.width > aReflowState.mComputedMaxWidth) {
+    aDesiredSize.width = aReflowState.mComputedMaxWidth;
+  }
+  if (aDesiredSize.width < aReflowState.mComputedMinWidth) {
+    aDesiredSize.width = aReflowState.mComputedMinWidth;
+  } 
+
+  if (aDesiredSize.height > aReflowState.mComputedMaxHeight) {
+    aDesiredSize.height = aReflowState.mComputedMaxHeight;
+  }
+  if (aDesiredSize.height < aReflowState.mComputedMinHeight) {
+    aDesiredSize.height = aReflowState.mComputedMinHeight;
+  }
+
+  aDesiredSize.ascent  += aReflowState.mComputedBorderPadding.top + focusPadding.top;
+  aDesiredSize.descent = aDesiredSize.height - aDesiredSize.ascent;
+
+  aStatus = NS_FRAME_COMPLETE;
+
+  nsFormControlFrame::SetupCachedSizes(mCacheSize, mCachedAscent,
+                                       mCachedMaxElementWidth, aDesiredSize);
+  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
+  return NS_OK;
+}
+
+void
+nsHTMLButtonControlFrame::ReflowButtonContents(nsIPresContext* aPresContext,
+                                               nsHTMLReflowMetrics& aDesiredSize,
+                                               const nsHTMLReflowState& aReflowState,
+                                               nsIFrame* aFirstKid,
+                                               const nsSize& aAvailSize,
+                                               nsReflowReason aReason,
+                                               nsMargin aFocusPadding,
+                                               nsReflowStatus& aStatus)
+{
+  nsHTMLReflowState reflowState(aPresContext, aReflowState, aFirstKid, aAvailSize, aReason);
+
+  ReflowChild(aFirstKid, aPresContext, aDesiredSize, reflowState,
+              aFocusPadding.left + aReflowState.mComputedBorderPadding.left,
+              aFocusPadding.top + aReflowState.mComputedBorderPadding.top,
               0, aStatus);
   
   // calculate the min internal size so the contents gets centered correctly
@@ -544,14 +611,14 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
   // Place the child.  If we have a non-intrinsic width, we want to
   // reduce the left padding as needed to try and fit the text in the
   // button
-  nscoord xoffset = focusPadding.left + aReflowState.mComputedBorderPadding.left;
+  nscoord xoffset = aFocusPadding.left + aReflowState.mComputedBorderPadding.left;
   if (aReflowState.mComputedWidth != NS_INTRINSICSIZE) {
     // First, how much did we "overflow"?  This is the width of our
     // kid plus our special focus stuff (which did not get accounted
     // for in calculating aReflowState.mComputedWidth minus the width
     // we're forced to be.
     nscoord extrawidth =
-      aDesiredSize.width + focusPadding.left + focusPadding.right
+      aDesiredSize.width + aFocusPadding.left + aFocusPadding.right
       - aReflowState.mComputedWidth;
     if (extrawidth > 0) {
       // Split it evenly between right and left
@@ -563,62 +630,9 @@ nsHTMLButtonControlFrame::Reflow(nsIPresContext* aPresContext,
   }
   
   // Place the child
-  FinishReflowChild(firstKid, aPresContext, &reflowState, aDesiredSize,
+  FinishReflowChild(aFirstKid, aPresContext, &reflowState, aDesiredSize,
                     xoffset,
-                    yoff + focusPadding.top + aReflowState.mComputedBorderPadding.top, 0);
-
-  // if computed use the computed values.
-  if (aReflowState.mComputedWidth != NS_INTRINSICSIZE) 
-    aDesiredSize.width = aReflowState.mComputedWidth;
-  else 
-    aDesiredSize.width  += focusPadding.left + focusPadding.right;
-
-  if (aReflowState.mComputedHeight != NS_INTRINSICSIZE) 
-    aDesiredSize.height = aReflowState.mComputedHeight;
-  else
-    aDesiredSize.height += focusPadding.top + focusPadding.bottom;
-
-  AddComputedBorderPaddingToDesiredSize(aDesiredSize, aReflowState);
-
-  //aDesiredSize.width  += aReflowState.mComputedBorderPadding.left + aReflowState.mComputedBorderPadding.right;
-  //aDesiredSize.height += aReflowState.mComputedBorderPadding.top + aReflowState.mComputedBorderPadding.bottom;
-
-#if 0
-  //adjust our max element size, if necessary
-  if (aDesiredSize.maxElementSize) {
-    aDesiredSize.AddBorderPaddingToMaxElementSize(aReflowState.mComputedBorderPadding);
-  }
-#endif
-
-  if (nsnull != aDesiredSize.maxElementSize) {
-    aDesiredSize.maxElementSize->width  = aDesiredSize.width;
-    aDesiredSize.maxElementSize->height = aDesiredSize.height;
-  }
-
-  // Make sure we obey min/max-width and min/max-height
-  if (aDesiredSize.width > aReflowState.mComputedMaxWidth) {
-    aDesiredSize.width = aReflowState.mComputedMaxWidth;
-  }
-  if (aDesiredSize.width < aReflowState.mComputedMinWidth) {
-    aDesiredSize.width = aReflowState.mComputedMinWidth;
-  } 
-
-  if (aDesiredSize.height > aReflowState.mComputedMaxHeight) {
-    aDesiredSize.height = aReflowState.mComputedMaxHeight;
-  }
-  if (aDesiredSize.height < aReflowState.mComputedMinHeight) {
-    aDesiredSize.height = aReflowState.mComputedMinHeight;
-  }
-
-  aDesiredSize.ascent  += aReflowState.mComputedBorderPadding.top + focusPadding.top;
-  aDesiredSize.descent = aDesiredSize.height - aDesiredSize.ascent;
-
-  aStatus = NS_FRAME_COMPLETE;
-
-  nsFormControlFrame::SetupCachedSizes(mCacheSize, mCachedAscent,
-                                       mCachedMaxElementSize, aDesiredSize);
-  NS_FRAME_SET_TRUNCATION(aStatus, aReflowState, aDesiredSize);
-  return NS_OK;
+                    yoff + aFocusPadding.top + aReflowState.mComputedBorderPadding.top, 0);
 }
 
 NS_IMETHODIMP
@@ -700,18 +714,17 @@ NS_IMETHODIMP nsHTMLButtonControlFrame::GetProperty(nsIAtom* aName, nsAString& a
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsHTMLButtonControlFrame::GetAdditionalStyleContext(PRInt32 aIndex, 
-                                                    nsIStyleContext** aStyleContext) const
+nsStyleContext*
+nsHTMLButtonControlFrame::GetAdditionalStyleContext(PRInt32 aIndex) const
 {
-  return mRenderer.GetStyleContext(aIndex, aStyleContext);
+  return mRenderer.GetStyleContext(aIndex);
 }
 
-NS_IMETHODIMP
+void
 nsHTMLButtonControlFrame::SetAdditionalStyleContext(PRInt32 aIndex, 
-                                                    nsIStyleContext* aStyleContext)
+                                                    nsStyleContext* aStyleContext)
 {
-  return mRenderer.SetStyleContext(aIndex, aStyleContext);
+  mRenderer.SetStyleContext(aIndex, aStyleContext);
 }
 
 
