@@ -50,7 +50,7 @@ class nsIDocument;
 struct nsFrameItems;
 struct nsAbsoluteItems;
 struct nsTableCreator;
-class nsIStyleContext;
+class nsStyleContext;
 struct nsTableList;
 struct nsStyleContent;
 struct nsStyleDisplay;
@@ -92,6 +92,13 @@ public:
                                 nsIContent*     aDocElement,
                                 nsIFrame*&      aNewFrame);
 
+  NS_IMETHOD ConstructPageFrame(nsIPresShell*   aPresShell, 
+                                nsIPresContext* aPresContext,
+                                nsIFrame*       aParentFrame,
+                                nsIFrame*       aPrevPageFrame,
+                                nsIFrame*&      aPageFrame,
+                                nsIFrame*&      aPageContentFrame);
+
   NS_IMETHOD ReconstructDocElementHierarchy(nsIPresContext* aPresContext);
 
   NS_IMETHOD ContentAppended(nsIPresContext* aPresContext,
@@ -100,6 +107,7 @@ public:
 
   NS_IMETHOD ContentInserted(nsIPresContext*        aPresContext,
                              nsIContent*            aContainer,
+                             nsIFrame*              aContainerFrame,
                              nsIContent*            aChild,
                              PRInt32                aIndexInContainer,
                              nsILayoutHistoryState* aFrameState,
@@ -197,14 +205,14 @@ protected:
                                 nsFrameConstructorState& aState,
                                 nsIContent*              aContent,
                                 nsIFrame*                aParentFrame,
-                                nsIStyleContext*         aStyleContext,
+                                nsStyleContext*          aStyleContext,
                                 nsIFrame*                aPrevInFlow,
                                 nsIFrame*                aNewFrame);
 
-  nsresult ResolveStyleContext(nsIPresContext*   aPresContext,
-                               nsIFrame*         aParentFrame,
-                               nsIContent*       aContent,
-                               nsIStyleContext** aStyleContext);
+  already_AddRefed<nsStyleContext>
+  ResolveStyleContext(nsIPresContext*   aPresContext,
+                      nsIFrame*         aParentFrame,
+                      nsIContent*       aContent);
 
   nsresult ConstructFrame(nsIPresShell*            aPresShell,
                           nsIPresContext*          aPresContext,
@@ -218,7 +226,7 @@ protected:
                                     nsFrameConstructorState& aState,
                                     nsIContent*              aDocElement,
                                     nsIFrame*                aParentFrame,
-                                    nsIStyleContext*         aParentStyleContext,
+                                    nsStyleContext*          aParentStyleContext,
                                     nsIFrame*&               aNewFrame);
 
   nsresult ConstructDocElementTableFrame(nsIPresShell*          aPresShell, 
@@ -232,7 +240,7 @@ protected:
                                    nsIDocument*          aDocument,
                                    nsIFrame*             aParentFrame,
                                    nsIContent*           aContent,
-                                   nsIStyleContext*      aStyleContext,
+                                   nsStyleContext*       aStyleContext,
                                    const nsStyleContent* aStyleContent,
                                    PRUint32              aContentIndex,
                                    nsIFrame**            aFrame);
@@ -242,7 +250,7 @@ protected:
                                      nsFrameConstructorState& aState,
                                      nsIFrame*                aFrame,
                                      nsIContent*              aContent,
-                                     nsIStyleContext*         aStyleContext,
+                                     nsStyleContext*          aStyleContext,
                                      nsIAtom*                 aPseudoElement,
                                      nsIFrame**               aWrapperFrame,
                                      nsIFrame**               aResult);
@@ -259,8 +267,9 @@ protected:
                                nsIPresContext*          aPresContext,
                                nsFrameConstructorState& aState,
                                nsIContent*              aContent,
-                               nsIFrame*                aParent,
-                               nsIStyleContext*         aStyleContext,
+                               nsIFrame*                aGeometricParent,
+                               nsIFrame*                aContentParent,
+                               nsStyleContext*          aStyleContext,
                                nsTableCreator&          aTableCreator,
                                PRBool                   aIsPseudo,
                                nsFrameItems&            aChildItems,
@@ -273,7 +282,7 @@ protected:
                                       nsFrameConstructorState& aState,
                                       nsIContent*              aContent,
                                       nsIFrame*                aParent,
-                                      nsIStyleContext*         aStyleContext,
+                                      nsStyleContext*          aStyleContext,
                                       nsTableCreator&          aTableCreator,
                                       nsFrameItems&            aChildItems,
                                       nsIFrame*&               aNewFrame,
@@ -284,7 +293,7 @@ protected:
                                        nsFrameConstructorState& aState,
                                        nsIContent*              aContent,
                                        nsIFrame*                aParent,
-                                       nsIStyleContext*         aStyleContext,
+                                       nsStyleContext*          aStyleContext,
                                        nsTableCreator&          aTableCreator,
                                        PRBool                   aIsPseudo,
                                        nsFrameItems&            aChildItems,
@@ -296,7 +305,7 @@ protected:
                                        nsFrameConstructorState& aState,
                                        nsIContent*              aContent,
                                        nsIFrame*                aParent,
-                                       nsIStyleContext*         aStyleContext,
+                                       nsStyleContext*          aStyleContext,
                                        nsTableCreator&          aTableCreator,
                                        PRBool                   aIsPseudo,
                                        nsFrameItems&            aChildItems,
@@ -308,7 +317,7 @@ protected:
                                   nsFrameConstructorState& aState,
                                   nsIContent*              aContent,
                                   nsIFrame*                aParent,
-                                  nsIStyleContext*         aStyleContext,
+                                  nsStyleContext*          aStyleContext,
                                   nsTableCreator&          aTableCreator,
                                   PRBool                   aIsPseudo,
                                   nsFrameItems&            aChildItems,
@@ -320,7 +329,7 @@ protected:
                                   nsFrameConstructorState& aState,
                                   nsIContent*              aContent,
                                   nsIFrame*                aParent,
-                                  nsIStyleContext*         aStyleContext,
+                                  nsStyleContext*          aStyleContext,
                                   nsTableCreator&          aTableCreator,
                                   PRBool                   aIsPseudo,
                                   nsFrameItems&            aChildItems,
@@ -332,7 +341,7 @@ protected:
                                    nsFrameConstructorState& aState,
                                    nsIContent*              aContent,
                                    nsIFrame*                aParentFrame,
-                                   nsIStyleContext*         aStyleContext,
+                                   nsStyleContext*          aStyleContext,
                                    nsTableCreator&          aTableCreator,
                                    PRBool                   aIsPseudo,
                                    nsFrameItems&            aChildItems,
@@ -344,14 +353,14 @@ protected:
                                   nsIFrame*        aParentFrame,
                                   nsIAtom*         aTag,
                                   nsIContent*      aContent,
-                                  nsIStyleContext* aContext);
+                                  nsStyleContext*  aContext);
 
   nsresult ConstructTableForeignFrame(nsIPresShell*            aPresShell, 
                                       nsIPresContext*          aPresContext,
                                       nsFrameConstructorState& aState,
                                       nsIContent*              aContent,
                                       nsIFrame*                aParentFrameIn,
-                                      nsIStyleContext*         aStyleContext,
+                                      nsStyleContext*          aStyleContext,
                                       nsTableCreator&          aTableCreator,
                                       nsFrameItems&            aChildItems,
                                       nsIFrame*&               aNewFrame,
@@ -442,7 +451,7 @@ protected:
                              nsIContent*              aParentContent,
                              nsIFrame*                aParentFrame,
                              nsIAtom*                 aParentFrameType,
-                             nsIStyleContext*         aParentStyleContext,
+                             nsStyleContext*          aParentStyleContext,
                              nsTableCreator&          aTableCreator,
                              nsFrameItems&            aChildItems,
                              nsIFrame*&               aCaption);
@@ -456,15 +465,16 @@ protected:
                                      nsIFrameManager* aFrameManager,
                                      nsIContent*      aContent,
                                      nsIFrame*        aFrame,
-                                     nsIStyleContext* aStyleContext,
+                                     nsStyleContext*  aStyleContext,
                                      nsIFrame*        aParentFrame,
                                      nsIFrame**       aPlaceholderFrame);
 
   nsresult ConstructAlternateFrame(nsIPresShell*    aPresShell, 
                                    nsIPresContext*  aPresContext,
                                    nsIContent*      aContent,
-                                   nsIStyleContext* aStyleContext,
-                                   nsIFrame*        aParentFrame,
+                                   nsStyleContext*  aStyleContext,
+                                   nsIFrame*        aGeometricParent,
+                                   nsIFrame*        aContentParent,
                                    nsIFrame*&       aFrame);
 
   nsWidgetRendering GetFormElementRenderingMode(nsIPresContext* aPresContext,
@@ -474,13 +484,13 @@ protected:
                                       nsIPresContext*    aPresContext,
                                       nsIFrame*&         aNewFrame,
                                       nsIContent*        aContent,
-                                      nsIStyleContext*   aStyleContext);
+                                      nsStyleContext*    aStyleContext);
 
   nsresult ConstructCheckboxControlFrame(nsIPresShell*    aPresShell, 
                                          nsIPresContext*  aPresContext,
                                          nsIFrame*&       aNewFrame,
                                          nsIContent*      aContent,
-                                         nsIStyleContext* aStyleContext);
+                                         nsStyleContext*  aStyleContext);
 
   nsresult ConstructButtonControlFrame(nsIPresShell*    aPresShell,
                                        nsIPresContext*  aPresContext,
@@ -498,7 +508,7 @@ protected:
                                 nsIContent*              aContent,
                                 nsIFrame*                aParentFrame,
                                 nsIAtom*                 aTag,
-                                nsIStyleContext*         aStyleContext,
+                                nsStyleContext*          aStyleContext,
                                 nsIFrame*&               aNewFrame,
                                 PRBool&                  aProcessChildren,
                                 PRBool                   aIsAbsolutelyPositioned,
@@ -512,7 +522,7 @@ protected:
                                   nsIContent*              aContent,
                                   nsIFrame*                aParentFrame,
                                   nsIAtom*                 aTag,
-                                  nsIStyleContext*         aStyleContext,
+                                  nsStyleContext*          aStyleContext,
                                   nsIFrame*&               aNewFrame,
                                   PRBool&                  aProcessChildren,
                                   PRBool                   aIsAbsolutelyPositioned,
@@ -524,7 +534,7 @@ protected:
                               nsFrameConstructorState& aState,
                               nsIContent*              aContent,
                               nsIFrame*                aParentFrame,
-                              nsIStyleContext*         aStyleContext,
+                              nsStyleContext*          aStyleContext,
                               nsFrameItems&            aFrameItems);
 
   nsresult ConstructPageBreakFrame(nsIPresShell*            aPresShell, 
@@ -532,7 +542,7 @@ protected:
                                    nsFrameConstructorState& aState,
                                    nsIContent*              aContent,
                                    nsIFrame*                aParentFrame,
-                                   nsIStyleContext*         aStyleContext,
+                                   nsStyleContext*          aStyleContext,
                                    nsFrameItems&            aFrameItems);
 
   // Construct a page break frame if page-break-before:always is set in aStyleContext
@@ -543,7 +553,7 @@ protected:
                          nsFrameConstructorState& aState,
                          nsIContent*              aContent,
                          nsIFrame*                aParentFrame,
-                         nsIStyleContext*         aStyleContext,
+                         nsStyleContext*          aStyleContext,
                          nsFrameItems&            aFrameItems);
 
   nsresult ConstructHTMLFrame(nsIPresShell*            aPresShell, 
@@ -553,7 +563,7 @@ protected:
                               nsIFrame*                aParentFrame,
                               nsIAtom*                 aTag,
                               PRInt32                  aNameSpaceID,
-                              nsIStyleContext*         aStyleContext,
+                              nsStyleContext*          aStyleContext,
                               nsFrameItems&            aFrameItems);
 
   nsresult ConstructFrameInternal( nsIPresShell*            aPresShell, 
@@ -563,26 +573,28 @@ protected:
                                    nsIFrame*                aParentFrame,
                                    nsIAtom*                 aTag,
                                    PRInt32                  aNameSpaceID,
-                                   nsIStyleContext*         aStyleContext,
+                                   nsStyleContext*          aStyleContext,
                                    nsFrameItems&            aFrameItems,
                                    PRBool                   aXBLBaseTag);
 
-  nsresult CreateAnonymousFrames(nsIPresShell*                        aPresShell, 
-                                 nsIPresContext*                      aPresContext,
-                                             nsIAtom*                 aTag,
-                                             nsFrameConstructorState& aState,
-                                             nsIContent*              aParent,
-                                             nsIFrame*                aNewFrame,
-                                             nsFrameItems&            aChildItems,
-                                             PRBool                   aIsRoot = PR_FALSE);
+  nsresult CreateAnonymousFrames(nsIPresShell*            aPresShell, 
+                                 nsIPresContext*          aPresContext,
+                                 nsIAtom*                 aTag,
+                                 nsFrameConstructorState& aState,
+                                 nsIContent*              aParent,
+                                 nsIFrame*                aNewFrame,
+                                 PRBool                   aAppendToExisting,
+                                 nsFrameItems&            aChildItems,
+                                 PRBool                   aIsRoot = PR_FALSE);
 
-  nsresult CreateAnonymousFrames(nsIPresShell*                        aPresShell, 
-                                             nsIPresContext*          aPresContext,
-                                             nsFrameConstructorState& aState,
-                                             nsIContent*              aParent,
-                                             nsIDocument*             aDocument,
-                                             nsIFrame*                aNewFrame,
-                                             nsFrameItems&            aChildItems);
+  nsresult CreateAnonymousFrames(nsIPresShell*            aPresShell, 
+                                 nsIPresContext*          aPresContext,
+                                 nsFrameConstructorState& aState,
+                                 nsIContent*              aParent,
+                                 nsIDocument*             aDocument,
+                                 nsIFrame*                aNewFrame,
+                                 PRBool                   aAppendToExisting,
+                                 nsFrameItems&            aChildItems);
 
 //MathML Mod - RBS
 #ifdef MOZ_MATHML
@@ -593,11 +605,10 @@ protected:
                                 nsIFrame*                aParentFrame,
                                 nsIAtom*                 aTag,
                                 PRInt32                  aNameSpaceID,
-                                nsIStyleContext*         aStyleContext,
+                                nsStyleContext*          aStyleContext,
                                 nsFrameItems&            aFrameItems);
 #endif
 
-#ifdef INCLUDE_XUL
   nsresult ConstructXULFrame(nsIPresShell*            aPresShell, 
                              nsIPresContext*          aPresContext,
                              nsFrameConstructorState& aState,
@@ -605,12 +616,11 @@ protected:
                              nsIFrame*                aParentFrame,
                              nsIAtom*                 aTag,
                              PRInt32                  aNameSpaceID,
-                             nsIStyleContext*         aStyleContext,
+                             nsStyleContext*          aStyleContext,
                              nsFrameItems&            aFrameItems,
                              PRBool                   aXBLBaseTag,
                              PRBool&                  aHaltProcessing);
 
-#endif
 
 // SVG - rods
 #ifdef MOZ_SVG
@@ -621,7 +631,7 @@ protected:
                                 nsIFrame*                aParentFrame,
                                 nsIAtom*                 aTag,
                                 PRInt32                  aNameSpaceID,
-                                nsIStyleContext*         aStyleContext,
+                                nsStyleContext*          aStyleContext,
                                 nsFrameItems&            aFrameItems);
 #endif
 
@@ -633,7 +643,7 @@ protected:
                                        PRInt32                  aNameSpaceID,
                                        nsIAtom*                 aTag,
                                        nsIFrame*                aParentFrame,
-                                       nsIStyleContext*         aStyleContext,
+                                       nsStyleContext*          aStyleContext,
                                        nsFrameItems&            aFrameItems);
 
   nsresult ProcessChildren(nsIPresShell*            aPresShell, 
@@ -650,7 +660,7 @@ protected:
                             nsIPresContext*  aPresContext,
                             nsIContent*      aContent,
                             nsIFrame*&       aFrame,
-                            nsIStyleContext* aStyleContext);
+                            nsStyleContext*  aStyleContext);
 
   nsresult AddDummyFrameToSelect(nsIPresContext*          aPresContext,
                                  nsIPresShell*            aPresShell,
@@ -690,28 +700,27 @@ protected:
                    nsIPresContext*          aPresContext,
                    nsFrameConstructorState& aState,
                    nsIContent*              aContent,
-                   nsIStyleContext*         aContentStyle,
+                   nsStyleContext*          aContentStyle,
                    nsIFrame*                aScrolledFrame,
                    nsIFrame*                aParentFrame,
+                   nsIFrame*                aContentParentFrame,
                    nsIFrame*&               aNewFrame,
-                   nsIStyleContext*&        aScrolledChildStyle,
+                   nsStyleContext*&         aScrolledChildStyle,
                    nsIFrame*                aScrollPort = nsnull);
 
-  // Builds the initial ScrollFrame: 
-  //  if Gfx scrollbasrs in ares, it creates a GfxScrollFrame, otherwise it creates a ScrollFrame
-  //
-  nsresult
+  // Builds the initial ScrollFrame
+  already_AddRefed<nsStyleContext>
   BeginBuildingScrollFrame(nsIPresShell*            aPresShell, 
                            nsIPresContext*          aPresContext,
                            nsFrameConstructorState& aState,
                            nsIContent*              aContent,
-                           nsIStyleContext*         aContentStyle,
+                           nsStyleContext*          aContentStyle,
                            nsIFrame*                aParentFrame,
+                           nsIFrame*                aContentParentFrame,
                            nsIAtom*                 aScrolledPseudo,
                            nsIDocument*             aDocument,
                            PRBool                   aIsRoot,
                            nsIFrame*&               aNewFrame, 
-                           nsCOMPtr<nsIStyleContext>& aScrolledChildStyle,
                            nsIFrame*&               aScrollableFrame,
                            nsIFrame*                aScrollPort = nsnull);
 
@@ -724,22 +733,23 @@ protected:
                             nsIContent*              aContent,
                             nsIFrame*                aScrollFrame,
                             nsIFrame*                aScrolledFrame,
-                            nsIStyleContext*         scrolledPseudoStyle);
+                            nsStyleContext*          scrolledPseudoStyle);
 
   // Creates a new GfxScrollFrame, Initializes it, and creates a scroll port for it
   //
   nsresult
-  BuildGfxScrollFrame(nsIPresShell*            aPresShell, 
-                      nsIPresContext*          aPresContext,
-                      nsFrameConstructorState& aState,
-                      nsIContent*              aContent,
-                      nsIDocument*             aDocument,
-                      nsIFrame*                aParentFrame,
-                      nsIStyleContext*         aStyleContext,
-                      PRBool                   aIsRoot,
-                      nsIFrame*&               aNewFrame,
-                      nsFrameItems&            aAnonymousFrames,
-                      nsIFrame*                aScrollPort = nsnull);
+  InitGfxScrollFrame(nsIPresShell*            aPresShell, 
+                     nsIPresContext*          aPresContext,
+                     nsFrameConstructorState& aState,
+                     nsIContent*              aContent,
+                     nsIDocument*             aDocument,
+                     nsIFrame*                aParentFrame,
+                     nsIFrame*                aContentParentFrame,
+                     nsStyleContext*          aStyleContext,
+                     PRBool                   aIsRoot,
+                     nsIFrame*&               aNewFrame,
+                     nsFrameItems&            aAnonymousFrames,
+                     nsIFrame*                aScrollPort = nsnull);
 
 
   nsresult
@@ -750,21 +760,16 @@ protected:
                         nsIFrame*                scrolledFrame,
                         nsIContent*              aContent,
                         nsIFrame*                aParentFrame,
-                        nsIStyleContext*         aStyleContext,
+                        nsStyleContext*          aStyleContext,
                         PRBool                   aIsAbsolutelyPositioned,
                         PRBool                   aIsFixedPositioned,
                         PRBool                   aCreateBlock);
 
-  // cache the "nglayout.widget.gfxscrollbars" pref
-  PRBool HasGfxScrollbars();
   // cache the "nglayout.debug.enable_xbl_forms" pref
   PRBool UseXBLForms();
 
   nsresult RecreateFramesForContent(nsIPresContext*  aPresContext,
-                                    nsIContent*      aContent, 
-                                    PRBool           aInlineStyle = PR_FALSE,
-                                    nsIStyleRule*    aRule = nsnull,
-                                    nsIStyleContext* aContext = nsnull);
+                                    nsIContent*      aContent);
 
   nsresult RecreateFramesOnAttributeChange(nsIPresContext* aPresContext,
                                            nsIContent*     aContent,
@@ -775,7 +780,7 @@ protected:
                                            nsIFrame*        aFrame,
                                            nsIFrame*        aParentFrame,
                                            nsIContent*      aContent,
-                                           nsIStyleContext* aStyleContext,
+                                           nsStyleContext*  aStyleContext,
                                            nsIFrame**       aContinuingFrame);
 
   nsresult CreateContinuingTableFrame(nsIPresShell*    aPresShell, 
@@ -783,32 +788,34 @@ protected:
                                       nsIFrame*        aFrame,
                                       nsIFrame*        aParentFrame,
                                       nsIContent*      aContent,
-                                      nsIStyleContext* aStyleContext,
+                                      nsStyleContext*  aStyleContext,
                                       nsIFrame**       aContinuingFrame);
 
   //----------------------------------------
 
   // Methods support creating block frames and their children
 
-  static nsIStyleContext* GetFirstLetterStyle(nsIPresContext*  aPresContext,
-                                              nsIContent*      aContent,
-                                              nsIStyleContext* aStyleContext);
+  static already_AddRefed<nsStyleContext>
+  GetFirstLetterStyle(nsIPresContext*  aPresContext,
+                      nsIContent*      aContent,
+                      nsStyleContext*  aStyleContext);
 
-  static nsIStyleContext* GetFirstLineStyle(nsIPresContext*  aPresContext,
-                                            nsIContent*      aContent,
-                                            nsIStyleContext* aStyleContext);
+  static already_AddRefed<nsStyleContext>
+  GetFirstLineStyle(nsIPresContext*  aPresContext,
+                    nsIContent*      aContent,
+                    nsStyleContext*  aStyleContext);
 
   static PRBool HaveFirstLetterStyle(nsIPresContext*  aPresContext,
                                      nsIContent*      aContent,
-                                     nsIStyleContext* aStyleContext);
+                                     nsStyleContext*  aStyleContext);
 
   static PRBool HaveFirstLineStyle(nsIPresContext*  aPresContext,
                                    nsIContent*      aContent,
-                                   nsIStyleContext* aStyleContext);
+                                   nsStyleContext*  aStyleContext);
 
   static void HaveSpecialBlockStyle(nsIPresContext*  aPresContext,
                                     nsIContent*      aContent,
-                                    nsIStyleContext* aStyleContext,
+                                    nsStyleContext*  aStyleContext,
                                     PRBool*          aHaveFirstLetterStyle,
                                     PRBool*          aHaveFirstLineStyle);
 
@@ -822,8 +829,9 @@ protected:
                           const nsStyleDisplay*    aDisplay,
                           nsIContent*              aContent,
                           nsIFrame*                aParentFrame,
-                          nsIStyleContext*         aStyleContext,
-                          nsIFrame*                aNewFrame);
+                          nsStyleContext*          aStyleContext,
+                          nsIFrame*                aNewFrame,
+                          PRBool                   aRelPos);
 
   nsresult ProcessBlockChildren(nsIPresShell*            aPresShell, 
                                 nsIPresContext*          aPresContext,
@@ -840,7 +848,7 @@ protected:
                            const nsStyleDisplay*    aDisplay,
                            nsIContent*              aContent,
                            nsIFrame*                aParentFrame,
-                           nsIStyleContext*         aStyleContext,
+                           nsStyleContext*          aStyleContext,
                            PRBool                   aIsPositioned,
                            nsIFrame*                aNewFrame,
                            nsIFrame**               aNewBlockFrame,
@@ -917,7 +925,7 @@ protected:
                                  nsIFrame*                aTextFrame,
                                  nsIContent*              aBlockContent,
                                  nsIFrame*                aParentFrame,
-                                 nsIStyleContext*         aStyleContext,
+                                 nsStyleContext*          aStyleContext,
                                  nsFrameItems&            aResult);
 
   nsresult CreateLetterFrame(nsIPresShell*            aPresShell, 
@@ -1047,7 +1055,6 @@ protected:
   // Cached Prefs
   PRPackedBool        mGotGfxPrefs;
   PRPackedBool        mGotXBLFormPrefs;
-  PRPackedBool        mHasGfxScrollbars;
   PRPackedBool        mUseXBLForms;
 
   nsCOMPtr<nsILayoutHistoryState> mTempFrameTreeState;
