@@ -40,7 +40,6 @@
 #include "nsBoxLayoutState.h"
 #include "nsBox.h"
 #include "nsBoxFrame.h"
-#include "nsIStyleContext.h"
 #include "nsIPresContext.h"
 #include "nsCOMPtr.h"
 #include "nsIContent.h"
@@ -216,28 +215,37 @@ void Coelesced()
 
 #endif
 
-PRUint32 nsBox::gRefCnt = 0;
+PRBool nsBox::gGotTheme = PR_FALSE;
 nsITheme* nsBox::gTheme = nsnull;
 
+MOZ_DECL_CTOR_COUNTER(nsBox)
+
 nsBox::nsBox(nsIPresShell* aShell):mMouseThrough(unset),
                                    mNextChild(nsnull),
                                    mParentBox(nsnull)
                                    
 {
+  MOZ_COUNT_CTOR(nsBox);
   //mX = 0;
   //mY = 0;
-  gRefCnt++;
-  if (gRefCnt == 1)
-    nsServiceManager::GetService("@mozilla.org/chrome/chrome-native-theme;1",
-                                 NS_GET_IID(nsITheme),
-                                 (nsISupports**)&gTheme);
+  if (!gGotTheme) {
+    gGotTheme = PR_TRUE;
+    CallGetService("@mozilla.org/chrome/chrome-native-theme;1", &gTheme);
+  }
 }
 
 nsBox::~nsBox()
 {
-  gRefCnt--;
-  if (gRefCnt == 0 && gTheme)
-    nsServiceManager::ReleaseService("@mozilla.org/chrome/chrome-native-theme;1", gTheme);
+  // NOTE:  This currently doesn't get called for |nsBoxToBlockAdaptor|
+  // objects, so don't rely on putting anything here.
+  MOZ_COUNT_DTOR(nsBox);
+}
+
+/* static */ void
+nsBox::Shutdown()
+{
+  gGotTheme = PR_FALSE;
+  NS_IF_RELEASE(gTheme);
 }
 
 NS_IMETHODIMP 
@@ -255,7 +263,7 @@ nsBox::IsDirty(PRBool& aDirty)
   GetFrame(&frame);
   frame->GetFrameState(&state);
 
-  aDirty = (state & NS_FRAME_IS_DIRTY);  
+  aDirty = (state & NS_FRAME_IS_DIRTY) != 0;
   return NS_OK;
 }
 
@@ -267,7 +275,7 @@ nsBox::HasDirtyChildren(PRBool& aDirty)
   GetFrame(&frame);
   frame->GetFrameState(&state);
 
-  aDirty = (state & NS_FRAME_HAS_DIRTY_CHILDREN);  
+  aDirty = (state & NS_FRAME_HAS_DIRTY_CHILDREN) != 0;
   return NS_OK;
 }
 
@@ -823,7 +831,7 @@ nsBox::GetWasCollapsed(nsBoxLayoutState& aState)
   nsFrameState state;
   frame->GetFrameState(&state);
  
-  return (state & NS_STATE_IS_COLLAPSED);
+  return (state & NS_STATE_IS_COLLAPSED) != 0;
 }
 
 void
@@ -1074,7 +1082,7 @@ nsBox::GetOrientation(PRBool& aIsHorizontal)
    GetFrame(&frame);
    nsFrameState state;
    frame->GetFrameState(&state);
-   aIsHorizontal = (state & NS_STATE_IS_HORIZONTAL);
+   aIsHorizontal = (state & NS_STATE_IS_HORIZONTAL) != 0;
    return NS_OK;
 }
 
@@ -1085,7 +1093,7 @@ nsBox::GetDirection(PRBool& aIsNormal)
    GetFrame(&frame);
    nsFrameState state;
    frame->GetFrameState(&state);
-   aIsNormal = (state & NS_STATE_IS_DIRECTION_NORMAL);
+   aIsNormal = (state & NS_STATE_IS_DIRECTION_NORMAL) != 0;
    return NS_OK;
 }
 
@@ -1306,12 +1314,14 @@ nsIBox::AddCSSMinSize(nsBoxLayoutState& aState, nsIBox* aBox, nsSize& aSize)
                                       display->mAppearance, &size, &canOverride);
           float p2t;
           aState.GetPresContext()->GetScaledPixelsToTwips(&p2t);
-          aSize.width = NSIntPixelsToTwips(size.width, p2t);
-          aSize.height = NSIntPixelsToTwips(size.height, p2t);
-          if (aSize.width)
+          if (size.width) {
+            aSize.width = NSIntPixelsToTwips(size.width, p2t);
             widthSet = PR_TRUE;
-          if (aSize.height)
+          }
+          if (size.height) {
+            aSize.height = NSIntPixelsToTwips(size.height, p2t);
             heightSet = PR_TRUE;
+          }
         }
       }
     }
