@@ -137,7 +137,7 @@ static PLDHashOperator PR_CALLBACK VisitEntry(PLDHashTable* table, PLDHashEntryH
 }
 
 // member variables
-static PLDHashTableOps gTokenTableOps = {
+static const PLDHashTableOps gTokenTableOps = {
     PL_DHashAllocTable,
     PL_DHashFreeTable,
     GetKey,
@@ -397,7 +397,6 @@ TokenStreamListener::TokenStreamListener(TokenAnalyzer* analyzer)
     :   mAnalyzer(analyzer),
         mBuffer(NULL), mBufferSize(kBufferSize), mLeftOverCount(0)
 {
-    NS_INIT_ISUPPORTS();
 }
 
 TokenStreamListener::~TokenStreamListener()
@@ -511,8 +510,6 @@ nsBayesianFilter::nsBayesianFilter()
     :   mGoodCount(0), mBadCount(0),
         mBatchLevel(0), mTrainingDataDirty(PR_FALSE)
 {
-    NS_INIT_ISUPPORTS();
-    
     PRBool ok = (mGoodTokens && mBadTokens);
     NS_ASSERTION(ok, "error allocating tokenizers");
     if (ok)
@@ -534,7 +531,7 @@ public:
       mCurMessageToClassify = 0;
       mNumMessagesToClassify = aNumMessagesToClassify;
       mMessageURIs = (char **) nsMemory::Alloc(sizeof(char *) * aNumMessagesToClassify);
-      for (PRInt32 i = 0; i < aNumMessagesToClassify; i++)
+      for (PRUint32 i = 0; i < aNumMessagesToClassify; i++)
         mMessageURIs[i] = PL_strdup(aMessageURIs[i]);
 
     }
@@ -604,7 +601,26 @@ void nsBayesianFilter::classifyMessage(Tokenizer& tokenizer, const char* message
 {
     Token* tokens = tokenizer.copyTokens();
     if (!tokens) return;
-    
+  
+    // the algorithm in "A Plan For Spam" assumes that you have a large good
+    // corpus and a large junk corpus.
+    // that won't be the case with users who first use the junk mail feature
+    // so, we do certain things to encourage them to train.
+    //
+    // if there are no good tokens, assume the message is junk
+    // this will "encourage" the user to train
+    // and if there are no bad tokens, assume the message is not junk
+    // this will also "encourage" the user to train
+    // see bug #194238
+    if (listener && !mGoodCount && !mGoodTokens.countTokens()) {
+      listener->OnMessageClassified(messageURI, nsMsgJunkStatus(nsIJunkMailPlugin::JUNK));
+      return;
+    }
+    if (listener && !mBadCount && !mBadTokens.countTokens()) {
+      listener->OnMessageClassified(messageURI, nsMsgJunkStatus(nsIJunkMailPlugin::GOOD));
+      return;
+    }
+
     /* run the kernel of the Graham filter algorithm here. */
     PRUint32 i, count = tokenizer.countTokens();
     double ngood = mGoodCount, nbad = mBadCount;
