@@ -395,7 +395,7 @@ char *nsIMAPGenericParser::CreateNilString()
 {
 	if (!PL_strncasecmp(fNextToken, "NIL", 3))
 	{
-		if (nsCRT::strlen(fNextToken) != 3)
+		if (strlen(fNextToken) != 3)
 			fNextToken += 3;
 		//fNextToken = GetNextToken();
 		return NULL;
@@ -520,9 +520,14 @@ char *nsIMAPGenericParser::CreateLiteral()
 	int32 numberOfCharsInMessage = atoi(fNextToken + 1);
 	int32 charsReadSoFar = 0, currentLineLength = 0;
 	int32 bytesToCopy = 0;
-	
-	char *returnString = (char *) PR_Malloc(numberOfCharsInMessage + 1);
-	
+
+	uint32 numBytes = numberOfCharsInMessage + 1;
+  NS_ASSERTION(numBytes, "overflow!");
+  if (!numBytes)
+    return nsnull;
+
+	char *returnString = (char *) PR_Malloc(numBytes);
+
 	if (returnString)
 	{
 		*(returnString + numberOfCharsInMessage) = 0; // Null terminate it first
