@@ -36,6 +36,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#ifdef MOZ_LOGGING
+// sorry, this has to be before the pre-compiled header
+#define FORCE_PR_LOG /* Allow logging in the release build */
+#endif
 
 #include "nsICharsetConverterManager.h"
 #include "nsICharsetAlias.h"
@@ -62,133 +66,25 @@
 #include "plstr.h"
 #include "prmem.h"
 #include "ImportDebug.h"
+#include "nsImportService.h"
 
+PRLogModuleInfo *IMPORTLOGMODULE = nsnull;
 
 static NS_DEFINE_CID(kComponentManagerCID, 	NS_COMPONENTMANAGER_CID);
 static NS_DEFINE_CID(kCharsetConverterManagerCID, NS_ICHARSETCONVERTERMANAGER_CID);
-
-
 static nsIImportService *	gImportService = nsnull;
-
 static const char *	kWhitespace = "\b\t\r\n ";
 
 
-class nsImportModuleList;
-
-class nsImportService : public nsIImportService
-{
-public:
-
-	nsImportService();
-	virtual ~nsImportService();
-	
-	NS_DECL_ISUPPORTS
-
-	NS_DECL_NSIIMPORTSERVICE
-
-private:
-	nsresult	LoadModuleInfo( const char *pClsId, const char *pSupports);
-	nsresult	DoDiscover( void);
-
-private:
-	nsImportModuleList *	m_pModules;
-	PRBool					m_didDiscovery;	
-	nsString				m_sysCharset;
-	nsIUnicodeDecoder *		m_pDecoder;
-	nsIUnicodeEncoder *		m_pEncoder;
-};
-
-// extern nsresult NS_NewImportService(nsIImportService** aImportService);
-
-class ImportModuleDesc {
-public:
-	ImportModuleDesc() { m_pModule = nsnull;}
-	~ImportModuleDesc() { ReleaseModule();	}
-	
-	void	SetCID( const nsCID& cid) { m_cid = cid;}
-	void	SetName( const PRUnichar *pName) { m_name = pName;}
-	void	SetDescription( const PRUnichar *pDesc) { m_description = pDesc;}
-	void	SetSupports( const char *pSupports) { m_supports = pSupports;}
-	
-	nsCID			GetCID( void) { return( m_cid);}
-	const PRUnichar *GetName( void) { return( m_name.get());}
-	const PRUnichar *GetDescription( void) { return( m_description.get());}
-	const char *	GetSupports( void) { return( m_supports.get());}
-	
-	nsIImportModule *	GetModule( PRBool keepLoaded = PR_FALSE); // Adds ref
-	void				ReleaseModule( void);
-	
-	PRBool			SupportsThings( const char *pThings);
-
-private:
-	nsCID				m_cid;
-	nsString			m_name;
-	nsString			m_description;
-	nsCString			m_supports;
-	nsIImportModule *	m_pModule;
-};
-
-class nsImportModuleList {
-public:
-	nsImportModuleList() { m_pList = nsnull; m_alloc = 0; m_count = 0;}
-	~nsImportModuleList() { ClearList(); }
-
-	void	AddModule( const nsCID& cid, const char *pSupports, const PRUnichar *pName, const PRUnichar *pDesc);
-
-	void	ClearList( void);
-	
-	PRInt32	GetCount( void) { return( m_count);}
-	
-	ImportModuleDesc *	GetModuleDesc( PRInt32 idx) 
-		{ if ((idx < 0) || (idx >= m_count)) return( nsnull); else return( m_pList[idx]);}
-	
-private:
-	
-private:
-	ImportModuleDesc **	m_pList;
-	PRInt32				m_alloc;
-	PRInt32				m_count;
-};
-
-////////////////////////////////////////////////////////////////////////
-NS_IMETHODIMP NS_NewImportService( nsISupports* aOuter, REFNSIID aIID, void **aResult)
-{
-    NS_PRECONDITION(aResult != nsnull, "null ptr");
-    if (! aResult)
-        return( NS_ERROR_NULL_POINTER);
-    if (aOuter) {                                                    
-        *aResult = nsnull;
-        return( NS_ERROR_NO_AGGREGATION);
-    }
-
-	if (!gImportService) {	
-    	gImportService = new nsImportService();
-    	if (! gImportService)
-        	return( NS_ERROR_OUT_OF_MEMORY);
-	}       
-
-    NS_ADDREF( gImportService);
-
-	nsresult rv = gImportService->QueryInterface(aIID, aResult);
-    if (NS_FAILED(rv)) {
-        *aResult = nsnull;
-    }
-    NS_RELEASE( gImportService);
-	
-	nsImportStringBundle::GetStringBundle();
-
-	return( rv);
-}
-
-
 ////////////////////////////////////////////////////////////////////////
 
 
 nsImportService::nsImportService() : m_pModules( nsnull)
 {
-    NS_INIT_ISUPPORTS();
-
-	IMPORT_LOG0( "* nsImport Service Created\n");
+	// Init logging module.
+  if (!IMPORTLOGMODULE)
+    IMPORTLOGMODULE = PR_NewLogModule("IMPORT");
+  IMPORT_LOG0( "* nsImport Service Created\n");
 
 	m_didDiscovery = PR_FALSE;
 	m_pDecoder = nsnull;
@@ -198,6 +94,8 @@ nsImportService::nsImportService() : m_pModules( nsnull)
 	// thread issues later.
 	nsString	str;
 	SystemStringToUnicode( "Dummy", str);
+    
+    nsImportStringBundle::GetStringBundle(); 
 }
 
 
@@ -212,7 +110,6 @@ nsImportService::~nsImportService()
         delete m_pModules;
 
 	IMPORT_LOG0( "* nsImport Service Deleted\n");
-
 }
 
 
