@@ -248,6 +248,12 @@ NS_IMETHODIMP nsMsgMailboxParser::OnReadChanged(nsIDBChangeListener *instigator)
     return NS_OK;
 }
 
+/* void OnJunkScoreChanged (in nsIDBChangeListener instigator); */
+NS_IMETHODIMP nsMsgMailboxParser::OnJunkScoreChanged(nsIDBChangeListener *instigator)
+{
+    return NS_OK;
+}
+
 nsMsgMailboxParser::nsMsgMailboxParser() : nsMsgLineBuffer(nsnull, PR_FALSE)
 {
   Init();
@@ -266,8 +272,6 @@ nsMsgMailboxParser::~nsMsgMailboxParser()
 
 void nsMsgMailboxParser::Init()
 {
-  /* the following macro is used to initialize the ref counting data */
-	NS_INIT_ISUPPORTS();
 	m_obuffer = nsnull;
 	m_obuffer_size = 0;
 	m_graph_progress_total = 0;
@@ -501,7 +505,6 @@ NS_IMPL_ISUPPORTS1(nsParseMailMessageState, nsIMsgParseMailMsgState)
 
 nsParseMailMessageState::nsParseMailMessageState()
 {
-	NS_INIT_ISUPPORTS();
 	m_position = 0;
 	m_IgnoreXMozillaStatus = PR_FALSE;
 	m_state = nsIMsgParseMailMsgState::ParseBodyState;
@@ -1531,7 +1534,7 @@ void nsParseNewMailState::DoneParsingFolder(nsresult status)
 
 PRInt32 nsParseNewMailState::PublishMsgHeader(nsIMsgWindow *msgWindow)
 {
-  PRBool		moved = PR_FALSE;
+  PRBool moved = PR_FALSE;
   
   FinishHeader();
   
@@ -1553,7 +1556,7 @@ PRInt32 nsParseNewMailState::PublishMsgHeader(nsIMsgWindow *msgWindow)
         if (!(oldFlags & MSG_FLAG_READ)) // don't mark read messages as new.
           m_newMsgHdr->OrFlags(MSG_FLAG_NEW, &newFlags);
         
-        m_mailDB->AddNewHdrToDB (m_newMsgHdr, PR_TRUE);
+        m_mailDB->AddNewHdrToDB(m_newMsgHdr, PR_TRUE);
       }
     }		// if it was moved by imap filter, m_parseMsgState->m_newMsgHdr == nsnull
     m_newMsgHdr = nsnull;
@@ -1676,7 +1679,7 @@ NS_IMETHODIMP nsParseNewMailState::ApplyFilterHit(nsIMsgFilter *filter, nsIMsgWi
 			    if ((const char*)actionTargetFolderUri &&
               nsCRT::strcasecmp(m_inboxUri, actionTargetFolderUri))
           {
-            nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, (const char *) actionTargetFolderUri, filter, msgWindow);
+            nsresult err = MoveIncorporatedMessage(msgHdr, m_mailDB, actionTargetFolderUri, filter, msgWindow);
             if (NS_SUCCEEDED(err))
             {
               if (loggingEnabled)
@@ -1736,7 +1739,7 @@ int nsParseNewMailState::MarkFilteredMessageRead(nsIMsgDBHdr *msgHdr)
 
 nsresult nsParseNewMailState::MoveIncorporatedMessage(nsIMsgDBHdr *mailHdr, 
 											   nsIMsgDatabase *sourceDB, 
-											   const char *destFolderUri,
+											   const nsACString& destFolderUri,
                                                       nsIMsgFilter *filter,
                                                       nsIMsgWindow *msgWindow)
 {
