@@ -177,9 +177,11 @@ MimeMultCMS_get_content_info(MimeObject *obj,
 extern PRBool MimeEncryptedCMS_encrypted_p (MimeObject *obj);
 extern PRBool MimeCMSHeadersAndCertsMatch(MimeObject *obj,
 											 nsICMSMessage *,
+											 PRBool *signing_cert_without_email_address,
 											 char **);
 extern char *MimeCMS_MakeSAURL(MimeObject *obj);
 extern char *IMAP_CreateReloadAllPartsUrl(const char *url);
+extern int MIMEGetRelativeCryptoNestLevel(MimeObject *obj);
 
 static void *
 MimeMultCMS_init (MimeObject *obj)
@@ -441,20 +443,7 @@ MimeMultCMS_generate (void *crypto_closure)
   PRInt32 signature_status = nsICMSMessageErrors::GENERAL_ERROR;
   nsCOMPtr<nsIX509Cert> signerCert;
 
-  // if we are the child of the topmost message, aNestLeve == 1
-  int aNestLevel = 0;
-
-  if (data->self) {
-    MimeObject *walker = data->self;
-    while (walker) {
-      // Crypto mime objects are transparent wrt nesting.
-      if (!mime_typep(walker, (MimeObjectClass *) &mimeEncryptedClass)
-          && !mime_typep(walker, (MimeObjectClass *) &mimeMultipartSignedClass)) {
-        ++aNestLevel;
-      }
-      walker = walker->parent;
-    }
-  }
+  int aRelativeNestLevel = MIMEGetRelativeCryptoNestLevel(data->self);
 
   unverified_p = data->self->options->missing_parts; 
 
@@ -482,11 +471,19 @@ MimeMultCMS_generate (void *crypto_closure)
         data->verify_error = -1;
       }
     } else {
+		  PRBool signing_cert_without_email_address;
+
 		  good_p = MimeCMSHeadersAndCertsMatch(data->self,
 												 data->content_info,
+												 &signing_cert_without_email_address,
 												 &data->sender_addr);
       if (!good_p) {
-        signature_status = nsICMSMessageErrors::VERIFY_HEADER_MISMATCH;
+        if (signing_cert_without_email_address) {
+          signature_status = nsICMSMessageErrors::VERIFY_CERT_WITHOUT_ADDRESS;
+        }
+        else {
+          signature_status = nsICMSMessageErrors::VERIFY_HEADER_MISMATCH;
+        }
         if (!data->verify_error) {
           data->verify_error = -1;
           // XXX Fix this		data->verify_error = SEC_ERROR_CERT_ADDR_MISMATCH; XXX //
@@ -524,11 +521,13 @@ MimeMultCMS_generate (void *crypto_closure)
 
   PRInt32 maxNestLevel = 0;
   if (data->smimeHeaderSink) {
-    data->smimeHeaderSink->MaxWantedNesting(&maxNestLevel);
+    if (aRelativeNestLevel >= 0) {
+      data->smimeHeaderSink->MaxWantedNesting(&maxNestLevel);
 
-    if (aNestLevel <= maxNestLevel)
-    {
-      data->smimeHeaderSink->SignedStatus(aNestLevel, signature_status, signerCert);
+      if (aRelativeNestLevel <= maxNestLevel)
+      {
+        data->smimeHeaderSink->SignedStatus(aRelativeNestLevel, signature_status, signerCert);
+      }
     }
   }
 
