@@ -54,7 +54,8 @@
 #include "nsNewsDatabase.h"
 #include "nsMsgDBCID.h"
 #include "nsMsgBaseCID.h"
-#include "nsIPref.h"
+#include "nsIPrefBranch.h"
+#include "nsIPrefService.h"
 #include "nsCRT.h"  // for nsCRT::strtok
 #include "nsNntpService.h"
 #include "nsIChannel.h"
@@ -91,14 +92,12 @@
 #define PREF_MAIL_ROOT_NNTP 	"mail.root.nntp"
 
 static NS_DEFINE_CID(kCNewsDB, NS_NEWSDB_CID);
-static NS_DEFINE_CID(kCPrefServiceCID, NS_PREF_CID); 
 static NS_DEFINE_CID(kMessengerMigratorCID, NS_MESSENGERMIGRATOR_CID);
 static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);
 static NS_DEFINE_CID(kCacheServiceCID, NS_CACHESERVICE_CID);
                     
 nsNntpService::nsNntpService()
 {
-  NS_INIT_ISUPPORTS();
   mPrintingOperation = PR_FALSE;
 	mOpenAttachmentOperation = PR_FALSE;
 }
@@ -622,10 +621,10 @@ nsNntpService::GetFolderFromUri(const char *uri, nsIMsgFolder **folder)
     if ((strlen(uri) > kNntpRootURILen) && nsCRT::strncmp(uri, kNntpRootURI, kNntpRootURILen) == 0) {
       nsCAutoString uriStr(kNewsRootURI);
       uriStr.Append(uri+kNntpRootURILen);
-      rv = rdf->GetResource(uriStr.get(), getter_AddRefs(res));
+      rv = rdf->GetResource(uriStr, getter_AddRefs(res));
     }
     else {
-      rv = rdf->GetResource(uri, getter_AddRefs(res));
+      rv = rdf->GetResource(nsDependentCString(uri), getter_AddRefs(res));
     }
     NS_ENSURE_SUCCESS(rv,rv);
 
@@ -750,7 +749,7 @@ nsNntpService::SetUpNntpUrlForPosting(nsINntpUrl *nntpUrl, const char *newsgroup
 
   nsCAutoString host;
 
-  // newsgroupsNames can be a comma seperated list of these:
+  // newsgroupsNames can be a comma separated list of these:
   // news://host/group
   // news://group
   // host/group
@@ -863,7 +862,7 @@ nsNntpService::GenerateNewsHeaderValsForPosting(const char *newsgroupsList, char
   NS_ENSURE_ARG_POINTER(newshostHeaderVal);
   NS_ENSURE_ARG_POINTER(*newsgroupsList);
 
-  // newsgroupsList can be a comma seperated list of these:
+  // newsgroupsList can be a comma separated list of these:
   // news://host/group
   // news://group
   // host/group
@@ -1213,13 +1212,28 @@ nsNntpService::GetProtocolForUri(nsIURI *aUri, nsIMsgWindow *aMsgWindow, nsINNTP
 
   nsCAutoString spec;
   rv = aUri->GetSpec(spec);
+  NS_ENSURE_SUCCESS(rv,rv);
+
+#if 0 // this not ready yet. 
+  nsNewsAction action = nsINntpUrl::ActionUnknown;
+  nsCOMPtr <nsINntpUrl> nntpUrl = do_QueryInterface(aUri);
+  if (nntpUrl) {
+    rv = nntpUrl->GetNewsAction(&action);
+    NS_ENSURE_SUCCESS(rv,rv);
+  }
 
   // if this is a news-message:/ uri, decompose it and set hasMsgOffline on the uri
   // Or, if it's of this form, we need to do the same.
   // "news://news.mozilla.org:119/3D612B96.1050301%40netscape.com?part=1.2&type=image/gif&filename=hp_icon_logo.gif"
 
-  if (!PL_strncmp(spec.get(), kNewsMessageRootURI, kNewsMessageRootURILen)
-    || !PL_strncmp(spec.get(), kNewsRootURI, kNewsRootURILen)) {
+  // XXX todo, or do we want to check if it is a news-message:// uri or
+  // a news:// uri (but action is not a fetch related action?)
+  if (!PL_strncmp(spec.get(), kNewsMessageRootURI, kNewsMessageRootURILen) ||
+      (action == nsINntpUrl::ActionFetchPart || action == nsINntpUrl::ActionFetchArticle)) {
+#else
+  // if this is a news-message:/ uri, decompose it and set hasMsgOffline on the uri
+  if (!PL_strncmp(spec.get(), kNewsMessageRootURI, kNewsMessageRootURILen)) {
+#endif
     nsCOMPtr <nsIMsgFolder> folder;
     nsMsgKey key = nsMsgKey_None;
     rv = DecomposeNewsMessageURI(spec.get(), getter_AddRefs(folder), &key);
@@ -1419,11 +1433,10 @@ NS_IMETHODIMP
 nsNntpService::SetDefaultLocalPath(nsIFileSpec *aPath)
 {
     nsresult rv;
-    nsCOMPtr<nsIPref> prefs(do_GetService(kCPrefServiceCID, &rv));
+    nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
     if (NS_FAILED(rv)) return rv;
 
-    rv = prefs->SetFilePref(PREF_MAIL_ROOT_NNTP, aPath, PR_FALSE /* set default */);
-    return rv;
+    return prefBranch->SetComplexValue(PREF_MAIL_ROOT_NNTP, NS_GET_IID(nsIFileSpec), aPath);
 }
 
 NS_IMETHODIMP
@@ -1433,13 +1446,14 @@ nsNntpService::GetDefaultLocalPath(nsIFileSpec ** aResult)
     *aResult = nsnull;
     
     nsresult rv;
-    nsCOMPtr<nsIPref> prefs(do_GetService(kCPrefServiceCID, &rv));
+    nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
     if (NS_FAILED(rv)) return rv;
     
     PRBool havePref = PR_FALSE;
     nsCOMPtr<nsIFile> localFile;
     nsCOMPtr<nsILocalFile> prefLocal;
-    rv = prefs->GetFileXPref(PREF_MAIL_ROOT_NNTP, getter_AddRefs(prefLocal));
+    rv = prefBranch->GetComplexValue(PREF_MAIL_ROOT_NNTP, NS_GET_IID(nsILocalFile),
+                                     getter_AddRefs(prefLocal));
     if (NS_SUCCEEDED(rv)) {
         localFile = prefLocal;
         havePref = PR_TRUE;
@@ -1449,7 +1463,7 @@ nsNntpService::GetDefaultLocalPath(nsIFileSpec ** aResult)
         if (NS_FAILED(rv)) return rv;
         havePref = PR_FALSE;
     }
-        
+
     PRBool exists;
     rv = localFile->Exists(&exists);
     if (NS_FAILED(rv)) return rv;
@@ -1696,11 +1710,11 @@ NS_IMETHODIMP nsNntpService::GetChromeUrlForTask(char **aChromeUrlForTask)
 { 
     if (!aChromeUrlForTask) return NS_ERROR_FAILURE; 
 	nsresult rv;
-	nsCOMPtr<nsIPref> prefService(do_GetService(kCPrefServiceCID, &rv));
+	nsCOMPtr<nsIPrefBranch> prefBranch(do_GetService(NS_PREFSERVICE_CONTRACTID, &rv));
 	if (NS_SUCCEEDED(rv))
 	{
 		PRInt32 layout;
-		rv = prefService->GetIntPref("mail.pane_config", &layout);		
+		rv = prefBranch->GetIntPref("mail.pane_config", &layout);		
 		if(NS_SUCCEEDED(rv))
 		{
 			if(layout == 0)
