@@ -37,6 +37,7 @@
 
 #include "nscore.h" 
 #include "nsProfile.h"
+#include "nsProfileLock.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch.h"
 
@@ -49,7 +50,6 @@
 #include "nsIComponentManager.h"
 #include "nsIEnumerator.h"
 #include "nsXPIDLString.h"
-#include "nsIFileSpec.h"
 #include "nsEscape.h"
 #include "nsIURL.h"
 
@@ -91,17 +91,15 @@
 #include "nsIDialogParamBlock.h"
 #include "nsIDOMWindowInternal.h"
 #include "nsIWindowWatcher.h"
+#include "jsapi.h"
+#include "nsIJSContextStack.h"
 
-#if defined (XP_UNIX)
-#elif defined (XP_MAC)
+
+#if defined(XP_MAC) || defined(XP_MACOSX)
 #define OLD_REGISTRY_FILE_NAME "Netscape Registry"
-#elif defined (XP_BEOS)
-#else /* assume XP_PC */
-#ifndef XP_OS2
-#include <direct.h>
-#endif
+#elif defined(XP_WIN) || defined(XP_OS2)
 #define OLD_REGISTRY_FILE_NAME "nsreg.dat"
-#endif /* XP_UNIX */
+#endif
 
 
 // A default profile name, in case automigration 4x profile fails
@@ -152,7 +150,7 @@ static PRInt32          gInstanceCount = 0;
 // Profile database to remember which profile has been
 // created with UILocale and contentLocale on profileManager
 static nsHashtable *gLocaleProfiles = nsnull;
-nsCOMPtr<nsProfileDirServiceProvider> gDirServiceProvider;
+static nsProfileDirServiceProvider *gDirServiceProvider = nsnull;
 
 // IID and CIDs of all the services needed
 static NS_DEFINE_CID(kIProfileIID, NS_IPROFILE_IID);
@@ -238,7 +236,6 @@ nsresult RecursiveCopy(nsIFile* srcDir, nsIFile* destDir)
  */
 nsProfile::nsProfile()
 {
-    NS_INIT_ISUPPORTS();
     mStartingUp = PR_FALSE;
     mAutomigrate = PR_FALSE;
     mOutofDiskSpace = PR_FALSE;
@@ -247,6 +244,8 @@ nsProfile::nsProfile()
 
     mIsUILocaleSpecified = PR_FALSE;
     mIsContentLocaleSpecified = PR_FALSE;
+    
+    mShutdownProfileToreDownNetwork = PR_FALSE;
 }
 
 nsProfile::~nsProfile() 
@@ -259,7 +258,7 @@ nsProfile::~nsProfile()
         
       delete gProfileDataAccess;
       delete gLocaleProfiles;
-      gDirServiceProvider = nsnull;
+      NS_IF_RELEASE(gDirServiceProvider);
     }
 }
 
@@ -275,11 +274,10 @@ nsProfile::Init()
         gLocaleProfiles = new nsHashtable();
         if (!gLocaleProfiles)
             return NS_ERROR_OUT_OF_MEMORY;
-          
-        NS_NewProfileDirServiceProvider(PR_FALSE, getter_AddRefs(gDirServiceProvider));
-        if (!gDirServiceProvider)
-            return NS_ERROR_FAILURE;
-        rv = gDirServiceProvider->Register();
+
+        rv = NS_NewProfileDirServiceProvider(PR_FALSE, &gDirServiceProvider);
+        if (NS_SUCCEEDED(rv))
+            rv = gDirServiceProvider->Register();
     }
     return rv;
 }
@@ -450,7 +448,6 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
     nsCOMPtr<nsIPrefBranch> prefBranch;
     nsCOMPtr<nsIURI> profileURL;
     PRInt32 numProfiles=0;
-    nsXPIDLString currentProfileStr;
   
     nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID, &rv);
     if (NS_FAILED(rv)) return rv;
@@ -461,6 +458,15 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
 
     if (profileURLStr.Length() == 0)
     {
+        // If this flag is TRUE, it makes the multiple profile case
+        // just like the single profile case - the profile will be
+        // set to that returned by GetCurrentProfile(). It will prevent
+        // the profile selection dialog from being shown when we have
+        // multiple profiles.
+        
+        PRBool startWithLastUsedProfile = PR_FALSE;
+        GetStartWithLastUsedProfile(&startWithLastUsedProfile);
+        
         // This means that there was no command-line argument to force
         // profile UI to come up. But we need the UI anyway if there
         // are no profiles yet, or if there is more than one.
@@ -470,7 +476,7 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
             if (NS_FAILED(rv)) return rv;
             // Will get set in call to SetCurrentProfile() below
         }
-        else if (numProfiles == 1)
+        else if (numProfiles == 1 || startWithLastUsedProfile)
         {
             // If we get here and the 1 profile is the current profile,
             // which can happen with QuickLaunch, there's no need to do
@@ -507,13 +513,16 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
 
         nsCOMPtr<nsIWindowWatcher> windowWatcher(do_GetService(NS_WINDOWWATCHER_CONTRACTID, &rv));
         if (NS_FAILED(rv)) return rv;
- 
-        // We need to send a param to OpenWindow if the window is to be considered
-        // a dialog. It needs to be for script security reasons. This param block
-        // will be made use of soon. See bug 66833.
+
         nsCOMPtr<nsIDialogParamBlock> ioParamBlock(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID, &rv));
         if (NS_FAILED(rv)) return rv;
-       
+
+        // String0  -> mode
+        // Int0    <-  result code (1 == OK, 0 == Cancel)
+
+        ioParamBlock->SetNumberStrings(1);
+        ioParamBlock->SetString(0, NS_LITERAL_STRING("startup").get());
+
         nsCOMPtr<nsIDOMWindow> newWindow;
         rv = windowWatcher->OpenWindow(nsnull,
                                        profileURLStr.get(),
@@ -522,15 +531,14 @@ nsProfile::LoadDefaultProfileDir(nsCString & profileURLStr, PRBool canInteract)
                                        ioParamBlock,
                                        getter_AddRefs(newWindow));
         if (NS_FAILED(rv)) return rv;
+        PRInt32 dialogConfirmed;
+        ioParamBlock->GetInt(0, &dialogConfirmed);
+        if (dialogConfirmed == 0) return NS_ERROR_ABORT;
     }
 
-    // if we get here, and we don't have a current profile, 
-    // return a failure so we will exit
-    // this can happen, if the user hits Exit in the profile manager dialog
+    nsXPIDLString currentProfileStr;    
     rv = GetCurrentProfile(getter_Copies(currentProfileStr));
-    if (NS_FAILED(rv) || (*(const PRUnichar*)currentProfileStr == 0)) {
-        return NS_ERROR_FAILURE;
-    }
+    if (NS_FAILED(rv)) return rv;
 
     // if at this point we have a current profile but it is not set, set it
     if (!mCurrentProfileAvailable) {
@@ -879,7 +887,7 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
 		NS_ASSERTION(NS_SUCCEEDED(rv),"failed to determine if we should force migration");
 	}
 
-#ifndef MOZ_PHOENIX // The phoenix doesn't use old profiles.
+#ifndef MOZ_XUL_APP // The phoenix/thunderbird doesn't use old profiles.
 
     // Start Migaration activity
     rv = cmdLineArgs->GetCmdLineValue(INSTALLER_CMD_LINE_ARG, getter_Copies(cmdResult));
@@ -921,7 +929,7 @@ nsProfile::ProcessArgs(nsICmdLineService *cmdLineArgs,
         }
     }
 
-#endif // MOZ_PHOENIX
+#endif // MOZ_XUL_APP
 
 #ifdef DEBUG_profile_verbose
     printf("Profile Manager : Command Line Options : End\n");
@@ -1098,6 +1106,16 @@ NS_IMETHODIMP nsProfile::GetFirstProfile(PRUnichar **profileName)
     return NS_OK;
 }
 
+NS_IMETHODIMP nsProfile::GetStartWithLastUsedProfile(PRBool *aStartWithLastUsedProfile)
+{
+    NS_ENSURE_ARG_POINTER(aStartWithLastUsedProfile);
+    return gProfileDataAccess->GetStartWithLastUsedProfile(aStartWithLastUsedProfile);
+}
+
+NS_IMETHODIMP nsProfile::SetStartWithLastUsedProfile(PRBool aStartWithLastUsedProfile)
+{
+    return gProfileDataAccess->SetStartWithLastUsedProfile(aStartWithLastUsedProfile);
+}
 
 // Returns the name of the current profile i.e., the last used profile
 NS_IMETHODIMP
@@ -1171,9 +1189,31 @@ nsProfile::SetCurrentProfile(const PRUnichar * aCurrentProfile)
 
         // Phase 2a: Send the network teardown notification
         observerService->NotifyObservers(subject, "profile-change-net-teardown", context.get());
+        mShutdownProfileToreDownNetwork = PR_TRUE;
 
         // Phase 2b: Send the "teardown" notification
         observerService->NotifyObservers(subject, "profile-change-teardown", context.get());
+        if (mProfileChangeVetoed)
+        {
+            // Notify we will not proceed with changing the profile
+            observerService->NotifyObservers(subject, "profile-change-teardown-veto", context.get());
+
+            // Bring network back online and return
+            observerService->NotifyObservers(subject, "profile-change-net-restore", context.get());
+            return NS_OK;
+        }
+        
+        // Phase 2c: Now that things are torn down, force JS GC so that things which depend on
+        // resources which are about to go away in "profile-before-change" are destroyed first.
+        nsCOMPtr<nsIThreadJSContextStack> stack =
+          do_GetService("@mozilla.org/js/xpc/ContextStack;1", &rv);
+        if (NS_SUCCEEDED(rv))
+        {
+          JSContext *cx = nsnull;
+          stack->GetSafeJSContext(&cx);
+          if (cx)
+            ::JS_GC(cx);
+        }
         
         // Phase 3: Notify observers of a profile change
         observerService->NotifyObservers(subject, "profile-before-change", context.get());        
@@ -1182,22 +1222,17 @@ nsProfile::SetCurrentProfile(const PRUnichar * aCurrentProfile)
     }
 
     // Do the profile switch
+    localLock.Unlock(); // gDirServiceProvider will get and hold its own lock
     gDirServiceProvider->SetProfileDir(profileDir);  
     mCurrentProfileName.Assign(aCurrentProfile);    
     gProfileDataAccess->SetCurrentProfile(aCurrentProfile);
     gProfileDataAccess->mProfileDataChanged = PR_TRUE;
     gProfileDataAccess->UpdateRegistry(nsnull);
-    mCurrentProfileLock = localLock;
             
     if (NS_FAILED(rv)) return rv;
     mCurrentProfileAvailable = PR_TRUE;
     
-    if (isSwitch)
-    {
-        // Bring network back online
-        observerService->NotifyObservers(subject, "profile-change-net-restore", context.get());
-    }
-    else
+    if (!isSwitch)
     {
         // Ensure that the prefs service exists so it can respond to
         // the notifications we're about to send around. It needs to.
@@ -1205,6 +1240,13 @@ nsProfile::SetCurrentProfile(const PRUnichar * aCurrentProfile)
         NS_ASSERTION(NS_SUCCEEDED(rv), "Could not get prefs service");
     }
 
+    if (mShutdownProfileToreDownNetwork)
+    {
+        // Bring network back online
+        observerService->NotifyObservers(subject, "profile-change-net-restore", context.get());
+        mShutdownProfileToreDownNetwork = PR_FALSE;
+    }
+
     // Phase 4: Notify observers that the profile has changed - Here they respond to new profile
     observerService->NotifyObservers(subject, "profile-do-change", context.get());
 
@@ -1214,6 +1256,9 @@ nsProfile::SetCurrentProfile(const PRUnichar * aCurrentProfile)
     // Now that a profile is established, set the profile defaults dir for the locale of this profile
     rv = DefineLocaleDefaultsDir();
     NS_ASSERTION(NS_SUCCEEDED(rv), "nsProfile::DefineLocaleDefaultsDir failed");
+
+    // Phase 6: One last notification after the new profile is established
+    observerService->NotifyObservers(subject, "profile-initial-state", context.get());
       
     return NS_OK;
 }
@@ -1259,8 +1304,24 @@ NS_IMETHODIMP nsProfile::ShutDownCurrentProfile(PRUint32 shutDownType)
       if (mProfileChangeVetoed)
         return NS_OK;
       
-      // Phase 2: Send the "teardown" notification
+      // Phase 2a: Send the network teardown notification
+      observerService->NotifyObservers(subject, "profile-change-net-teardown", context.get());
+      mShutdownProfileToreDownNetwork = PR_TRUE;
+
+      // Phase 2b: Send the "teardown" notification
       observerService->NotifyObservers(subject, "profile-change-teardown", context.get());
+
+      // Phase 2c: Now that things are torn down, force JS GC so that things which depend on
+      // resources which are about to go away in "profile-before-change" are destroyed first.
+      nsCOMPtr<nsIThreadJSContextStack> stack =
+        do_GetService("@mozilla.org/js/xpc/ContextStack;1", &rv);
+      if (NS_SUCCEEDED(rv))
+      {
+        JSContext *cx = nsnull;
+        stack->GetSafeJSContext(&cx);
+        if (cx)
+          ::JS_GC(cx);
+      }
       
       // Phase 3: Notify observers of a profile change
       observerService->NotifyObservers(subject, "profile-before-change", context.get());        
@@ -1270,7 +1331,6 @@ NS_IMETHODIMP nsProfile::ShutDownCurrentProfile(PRUint32 shutDownType)
     UpdateCurrentProfileModTime(PR_TRUE);
     mCurrentProfileAvailable = PR_FALSE;
     mCurrentProfileName.Truncate(0);
-    mCurrentProfileLock.Unlock();
     
     return NS_OK;
 }
@@ -1881,40 +1941,13 @@ nsresult nsProfile::Update4xProfileInfo()
 {
     nsresult rv = NS_OK;
 
-#if defined(XP_PC) || defined(XP_MAC)
-
-    char * oldRegFile = GetOldRegLocation();
+#ifndef XP_BEOS
+    nsCOMPtr<nsIFile> oldRegFile;
+    rv = GetOldRegLocation(getter_AddRefs(oldRegFile));
+    if (NS_SUCCEEDED(rv))
     rv = gProfileDataAccess->Get4xProfileInfo(oldRegFile, PR_TRUE);
-    nsMemory::Free(oldRegFile);
-#elif defined (XP_BEOS)
-#else
-    /* XP_UNIX */
-    rv = gProfileDataAccess->Get4xProfileInfo(nsnull, PR_TRUE);
-#endif /* XP_PC || XP_MAC */
-    return rv;
-}
-
-// launch the application with a profile of user's choice
-// Prefs and FileLocation services are used here.
-// FileLocation service to make ir forget about the global profile dir it had.
-// Prefs service to kick off the startup to start the app with new profile's prefs.
-NS_IMETHODIMP nsProfile::StartApprunner(const PRUnichar* profileName)
-{
-    NS_ENSURE_ARG_POINTER(profileName);   
-
-    nsresult rv = NS_OK;
-
-#if defined(DEBUG_profile)
-    {
-      printf("ProfileManager : StartApprunner\n");
-
-      printf("profileName passed in: %s\n", NS_LossyConvertUCS2toASCII(profileName).get());
-    }
 #endif
 
-    rv = SetCurrentProfile(profileName);
-    NS_ASSERTION(NS_SUCCEEDED(rv), "failed to set profile");
-
     return rv;
 }
 
@@ -1930,34 +1963,35 @@ nsresult nsProfile::LoadNewProfilePrefs()
     return NS_OK;
 }
 
-char * nsProfile::GetOldRegLocation()
+nsresult nsProfile::GetOldRegLocation(nsIFile **aOldRegFile)
 {
-#if defined(XP_WIN) || defined(XP_OS2) || defined(XP_MAC)
+    NS_ENSURE_ARG_POINTER(aOldRegFile);
+    *aOldRegFile = nsnull;
+    nsresult rv = NS_OK;
+    
+    // For XP_UNIX there was no registry for 4.x profiles.
+    // Return nsnull for the file, and NS_OK - we succeeded in doing nothing.
+    
+#if defined(XP_WIN) || defined(XP_OS2) || defined(XP_MAC) || defined(XP_MACOSX)
     nsCOMPtr<nsIFile> oldRegFile;
-    nsCAutoString oldRegFilePath;
-    nsresult rv;
 
-#if defined(XP_OS2)
-    NS_GetSpecialDirectory(NS_OS2_DIR, getter_AddRefs(oldRegFile));
-#elif defined(XP_WIN)
-    NS_GetSpecialDirectory(NS_WIN_WINDOWS_DIR, getter_AddRefs(oldRegFile));
-#elif defined(XP_MAC)
-    NS_GetSpecialDirectory(NS_MAC_CLASSICPREFS_DIR, getter_AddRefs(oldRegFile));
+#if defined(XP_WIN)
+    rv = NS_GetSpecialDirectory(NS_WIN_WINDOWS_DIR, getter_AddRefs(oldRegFile));
+#elif defined(XP_OS2)
+    rv = NS_GetSpecialDirectory(NS_OS2_DIR, getter_AddRefs(oldRegFile));
+#elif defined(XP_MAC) || defined(XP_MACOSX)
+    rv = NS_GetSpecialDirectory(NS_MAC_PREFS_DIR, getter_AddRefs(oldRegFile));
 #endif
 
-    if (!oldRegFile)
-        return nsnull;
+    if (NS_FAILED(rv))
+        return rv;
     rv = oldRegFile->AppendNative(nsDependentCString(OLD_REGISTRY_FILE_NAME));
     if (NS_FAILED(rv))
-        return nsnull;
-    rv = oldRegFile->GetNativePath(oldRegFilePath);
-    if (NS_FAILED(rv))
-        return nsnull;
-    
-    return ToNewCString(oldRegFilePath);
-#else
-    return nsnull;
+        return rv;
+    NS_ADDREF(*aOldRegFile = oldRegFile);
 #endif
+
+    return rv;
 }
 
 nsresult nsProfile::UpdateCurrentProfileModTime(PRBool updateRegistry)
@@ -1982,24 +2016,15 @@ NS_IMETHODIMP nsProfile::MigrateProfileInfo()
 {
     nsresult rv = NS_OK;
 
-#if defined(XP_PC) || defined(XP_MAC)
-
-#if defined(DEBUG_profile_verbose)
-    printf("Entered MigrateProfileInfo.\n");
-#endif
-
-    char * oldRegFile = GetOldRegLocation();
+#ifndef XP_BEOS
+    nsCOMPtr<nsIFile> oldRegFile;
+    rv = GetOldRegLocation(getter_AddRefs(oldRegFile));
+    if (NS_SUCCEEDED(rv)) {
     rv = gProfileDataAccess->Get4xProfileInfo(oldRegFile, PR_FALSE);
-    nsMemory::Free(oldRegFile);
-
-#elif defined (XP_BEOS)
-#else
-    /* XP_UNIX */
-    rv = gProfileDataAccess->Get4xProfileInfo(nsnull, PR_FALSE);
-#endif /* XP_PC || XP_MAC */
-
     gProfileDataAccess->mProfileDataChanged = PR_TRUE;
     gProfileDataAccess->UpdateRegistry(nsnull);
+    }
+#endif
 
 	return rv;
 }
@@ -2360,8 +2385,6 @@ nsProfile::CreateDefaultProfile(void)
 {
     nsresult rv = NS_OK;
 
-    nsFileSpec profileDirSpec;
-    
     // Get the default user profiles folder
     nsCOMPtr<nsIFile> profileRootDir;
     rv = NS_GetSpecialDirectory(NS_APP_USER_PROFILES_ROOT_DIR, getter_AddRefs(profileRootDir));
@@ -2400,6 +2423,9 @@ nsProfile::SetRegStrings(const PRUnichar* profileName,
 
    rv = gProfileDataAccess->GetValue(profileName, &aProfile);
    if (NS_FAILED(rv)) return rv;
+
+   if (aProfile == nsnull)
+     return NS_ERROR_FAILURE;
    
    aProfile->NCHavePregInfo = regString;
 
@@ -2433,3 +2459,47 @@ NS_IMETHODIMP nsProfile::VetoChange()
     mProfileChangeVetoed = PR_TRUE;
     return NS_OK;
 }
+
+
+NS_IMETHODIMP
+nsProfile::GetRegStrings(const PRUnichar *aProfileName, 
+                        PRUnichar **aRegString, 
+                        PRUnichar **aRegName, 
+                        PRUnichar **aRegEmail, 
+                        PRUnichar **aRegOption)
+{
+    NS_ENSURE_ARG_POINTER(aProfileName);
+    NS_ENSURE_ARG_POINTER(aRegString);
+    NS_ENSURE_ARG_POINTER(aRegName);
+    NS_ENSURE_ARG_POINTER(aRegEmail);
+    NS_ENSURE_ARG_POINTER(aRegOption);
+
+    ProfileStruct*    profileVal;
+
+    nsresult rv = gProfileDataAccess->GetValue(aProfileName, &profileVal);
+    if (NS_FAILED(rv)) 
+      return rv;
+
+    if (profileVal == nsnull)
+      return NS_ERROR_FAILURE;
+
+    *aRegString = ToNewUnicode(profileVal->NCHavePregInfo);
+    if (!*aRegString)
+      return NS_ERROR_OUT_OF_MEMORY;    
+
+    *aRegName = ToNewUnicode(profileVal->NCProfileName);
+    if (!*aRegName)
+      return NS_ERROR_OUT_OF_MEMORY;    
+
+    *aRegEmail = ToNewUnicode(profileVal->NCEmailAddress);
+    if (!*aRegEmail)
+      return NS_ERROR_OUT_OF_MEMORY;    
+
+    *aRegOption = ToNewUnicode(profileVal->NCDeniedService);
+    if (!*aRegOption)
+      return NS_ERROR_OUT_OF_MEMORY;    
+
+     delete profileVal;
+
+     return NS_OK;
+}
