@@ -94,14 +94,14 @@ Distance_Impl( const nsReadingIterator<CharT>& aStart,
 
 NS_COM
 size_t
-Distance( const nsAString::const_iterator& aStart, const nsAString::const_iterator& aEnd )
+Distance( const nsReadingIterator<PRUnichar>&aStart, const nsReadingIterator<PRUnichar>&aEnd )
   {
     return Distance_Impl(aStart, aEnd);
   }
 
 NS_COM
 size_t
-Distance( const nsACString::const_iterator& aStart, const nsACString::const_iterator& aEnd )
+Distance( const nsReadingIterator<char>&aStart, const nsReadingIterator<char>&aEnd )
   {
     return Distance_Impl(aStart, aEnd);
   }
@@ -211,13 +211,13 @@ ToNewUTF8String( const nsAString& aSource )
     NS_ConvertUCS2toUTF8 temp(aSource);
 
     char* result;
-    if (temp.mOwnsBuffer) {
+    if (temp.GetOwnsBuffer()) {
       // We allocated. Trick the string into not freeing its buffer to
       // avoid an extra allocation.
       result = temp.mStr;
 
       temp.mStr=0;
-      temp.mOwnsBuffer = PR_FALSE;
+      temp.SetOwnsBuffer(PR_FALSE);
     }
     else {
       // We didn't allocate a buffer, so we need to copy it out of the
@@ -337,8 +337,133 @@ IsASCII( const nsAString& aString )
     return PR_TRUE;
   }
 
+NS_COM
+PRBool
+IsASCII( const nsACString& aString )
+  {
+    static const char NOT_ASCII = char(~0x7F);
 
 
+    // Don't want to use |copy_string| for this task, since we can stop at the first non-ASCII character
+
+    nsACString::const_iterator done_reading;
+    aString.EndReading(done_reading);
+
+      // for each chunk of |aString|...
+    PRUint32 fragmentLength = 0;
+    nsACString::const_iterator iter;
+    for ( aString.BeginReading(iter); iter != done_reading; iter.advance( PRInt32(fragmentLength) ) )
+      {
+        fragmentLength = PRUint32(iter.size_forward());
+        const char* c = iter.get();
+        const char* fragmentEnd = c + fragmentLength;
+
+          // for each character in this chunk...
+        while ( c < fragmentEnd )
+          if ( *c++ & NOT_ASCII )
+            return PR_FALSE;
+      }
+
+    return PR_TRUE;
+  }
+
+NS_COM
+PRBool
+IsUTF8( const nsACString& aString )
+  {
+    nsReadingIterator<char> done_reading;
+    aString.EndReading(done_reading);
+
+    PRInt32 state = 0;
+    PRBool overlong = PR_FALSE;
+    PRBool surrogate = PR_FALSE;
+    PRBool nonchar = PR_FALSE;
+    PRUint16 olupper = 0; // overlong byte upper bound.
+    PRUint16 slower = 0;  // surrogate byte lower bound.
+
+      // for each chunk of |aString|...
+    PRUint32 fragmentLength = 0;
+    nsReadingIterator<char> iter;
+
+    for ( aString.BeginReading(iter); iter != done_reading; iter.advance( PRInt32(fragmentLength) ) )
+      {
+        fragmentLength = PRUint32(iter.size_forward());
+        const char* ptr = iter.get();
+        const char* fragmentEnd = ptr + fragmentLength;
+
+          // for each character in this chunk...
+        while ( ptr < fragmentEnd )
+          {
+            PRUint8 c;
+            
+            if (0 == state)
+              {
+                c = *ptr++;
+
+                if ( UTF8traits::isASCII(c) ) 
+                  continue;
+
+                if ( c <= 0xC1 ) // [80-BF] where not expected, [C0-C1] for overlong.
+                  return PR_FALSE;
+                else if ( UTF8traits::is2byte(c) ) 
+                    state = 1;
+                else if ( UTF8traits::is3byte(c) ) 
+                  {
+                    state = 2;
+                    if ( c == 0xE0 ) // to exclude E0[80-9F][80-BF] 
+                      {
+                        overlong = PR_TRUE;
+                        olupper = 0x9F;
+                      }
+                    else if ( c == 0xED ) // ED[A0-BF][80-BF] : surrogate codepoint
+                      {
+                        surrogate = PR_TRUE;
+                        slower = 0xA0;
+                      }
+                    else if ( c == 0xEF ) // EF BF [BE-BF] : non-character
+                      nonchar = PR_TRUE;
+                  }
+                else if ( c <= 0xF4 ) // XXX replace /w UTF8traits::is4byte when it's updated to exclude [F5-F7].(bug 199090)
+                  {
+                    state = 3;
+                    nonchar = PR_TRUE;
+                    if ( c == 0xF0 ) // to exclude F0[80-8F][80-BF]{2}
+                      {
+                        overlong = PR_TRUE;
+                        olupper = 0x8F;
+                      }
+                    else if ( c == 0xF4 ) // to exclude F4[90-BF][80-BF] 
+                      {
+                        // actually not surrogates but codepoints beyond 0x10FFFF
+                        surrogate = PR_TRUE;
+                        slower = 0x90;
+                      }
+                  }
+                else
+                  return PR_FALSE; // Not UTF8 string
+              }
+              
+              while (ptr < fragmentEnd && state)
+                {
+                  c = *ptr++;
+                  --state;
+
+                  // non-character : EF BF [BE-BF] or F[0-7] [89AB]F BF [BE-BF]
+                  if ( nonchar &&  ( !state &&  c < 0xBE ||
+                       state == 1 && c != 0xBF  ||
+                       state == 2 && 0x0F != (0x0F & c) ))
+                     nonchar = PR_FALSE;
+
+                  if ( !UTF8traits::isInSeq(c) || overlong && c <= olupper || 
+                       surrogate && slower <= c || nonchar && !state )
+                    return PR_FALSE; // Not UTF8 string
+                  overlong = surrogate = PR_FALSE;
+                }
+            }
+        }
+    return !state; // state != 0 at the end indicates an invalid UTF-8 seq. 
+  }
+
   /**
    * A character sink for in-place case conversion.
    */
@@ -385,7 +510,7 @@ void
 ToUpperCase( nsCString& aCString )
   {
     ConvertToUpperCase converter;
-    converter.write(aCString.mStr, aCString.mLength);
+    converter.write(aCString.mStr, aCString.Length());
   }
 
   /**
@@ -411,6 +536,8 @@ class CopyToUpperCase
             char ch = *aSource;
             if ((ch >= 'a') && (ch <= 'z'))
               *cp = ch - ('a' - 'A');
+            else
+              *cp = ch;
             ++aSource;
             ++cp;
           }
@@ -479,7 +606,7 @@ void
 ToLowerCase( nsCString& aCString )
   {
     ConvertToLowerCase converter;
-    converter.write(aCString.mStr, aCString.mLength);
+    converter.write(aCString.mStr, aCString.Length());
   }
 
   /**
@@ -505,6 +632,8 @@ class CopyToLowerCase
             char ch = *aSource;
             if ((ch >= 'A') && (ch <= 'Z'))
               *cp = ch + ('a' - 'A');
+            else
+              *cp = ch;
             ++aSource;
             ++cp;
           }
@@ -623,7 +752,7 @@ CaseInsensitiveFindInReadable( const nsACString& aPattern, nsACString::const_ite
    */
 NS_COM
 PRBool
-RFindInReadable( const nsAString& aPattern, nsAString::const_iterator& aSearchStart, nsAString::const_iterator& aSearchEnd )
+RFindInReadable( const nsAString& aPattern, nsAString::const_iterator& aSearchStart, nsAString::const_iterator& aSearchEnd, const nsStringComparator& aComparator)
   {
     PRBool found_it = PR_FALSE;
 
@@ -632,7 +761,7 @@ RFindInReadable( const nsAString& aPattern, nsAString::const_iterator& aSearchSt
 
     while ( searchStart != searchEnd )
       {
-        if ( FindInReadable(aPattern, searchStart, searchEnd) )
+        if ( FindInReadable(aPattern, searchStart, searchEnd, aComparator) )
           {
             found_it = PR_TRUE;
 
@@ -656,7 +785,7 @@ RFindInReadable( const nsAString& aPattern, nsAString::const_iterator& aSearchSt
 
 NS_COM
 PRBool
-RFindInReadable( const nsACString& aPattern, nsACString::const_iterator& aSearchStart, nsACString::const_iterator& aSearchEnd )
+RFindInReadable( const nsACString& aPattern, nsACString::const_iterator& aSearchStart, nsACString::const_iterator& aSearchEnd, const nsCStringComparator& aComparator)
   {
     PRBool found_it = PR_FALSE;
 
@@ -665,7 +794,7 @@ RFindInReadable( const nsACString& aPattern, nsACString::const_iterator& aSearch
 
     while ( searchStart != searchEnd )
       {
-        if ( FindInReadable(aPattern, searchStart, searchEnd) )
+        if ( FindInReadable(aPattern, searchStart, searchEnd, aComparator) )
           {
             found_it = PR_TRUE;
 
