@@ -79,6 +79,7 @@ nsBaseWidget::nsBaseWidget()
 , mEventListener(nsnull)
 , mMenuListener(nsnull)
 , mCursor(eCursor_standard)
+, mWindowType(eWindowType_child)
 , mBorderStyle(eBorderStyle_none)
 , mIsShiftDown(PR_FALSE)
 , mIsControlDown(PR_FALSE)
@@ -100,8 +101,6 @@ nsBaseWidget::nsBaseWidget()
 #endif
 
     NS_NewISupportsArray(getter_AddRefs(mChildren));
-    
-    NS_INIT_ISUPPORTS();
 }
 
 
@@ -483,6 +482,26 @@ NS_IMETHODIMP nsBaseWidget::SetWindowType(nsWindowType aWindowType)
   return NS_OK;
 }
 
+//-------------------------------------------------------------------------
+//
+// Window transparency methods
+//
+//-------------------------------------------------------------------------
+
+NS_IMETHODIMP nsBaseWidget::SetWindowTranslucency(PRBool aTranslucent) {
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP nsBaseWidget::GetWindowTranslucency(PRBool& aTranslucent) {
+  aTranslucent = PR_FALSE;
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsBaseWidget::UpdateTranslucentWindowAlpha(const nsRect& aRect, PRUint8* aAlphas) {
+  NS_ASSERTION(PR_FALSE, "Window is not translucent");
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 //-------------------------------------------------------------------------
 //
 // Hide window borders/decorations for this widget
@@ -1279,7 +1298,6 @@ nsBaseWidget::debug_DumpInvalidate(FILE *                aFileOut,
 nsBaseWidget::Enumerator::Enumerator(nsBaseWidget & inParent)
   : mCurrentPosition(0), mParent(inParent)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 
