@@ -49,21 +49,16 @@
 #include "nsILocalFileMac.h"
 #endif
 
-// Logging of debug output
-#ifdef MOZ_LOGGING
-#define FORCE_PR_LOG /* Allow logging in the release build */
-#endif
 #include "prlog.h"
 extern PRLogModuleInfo *nsComponentManagerLog;
 
 nsNativeComponentLoader::nsNativeComponentLoader() :
-    mCompMgr(nsnull), mDllStore(nsnull)
+    mCompMgr(nsnull), mDllStore(nsnull), mLoadedDependentLibs(nsnull)
 {
-    NS_INIT_ISUPPORTS();
 }
 
 static PRBool PR_CALLBACK
-nsDll_Destroy(nsHashKey *aKey, void *aData, void* closure)
+DLLStore_Destroy(nsHashKey *aKey, void *aData, void* closure)
 {
     nsDll* entry = NS_STATIC_CAST(nsDll*, aData);
     delete entry;
@@ -73,11 +68,13 @@ nsDll_Destroy(nsHashKey *aKey, void *aData, void* closure)
 nsNativeComponentLoader::~nsNativeComponentLoader()
 {
     mCompMgr = nsnull;
-
     delete mDllStore;
+    delete mLoadedDependentLibs;
 }
     
-NS_IMPL_THREADSAFE_ISUPPORTS1(nsNativeComponentLoader, nsIComponentLoader);
+NS_IMPL_THREADSAFE_ISUPPORTS2(nsNativeComponentLoader, 
+                              nsIComponentLoader,
+                              nsINativeComponentLoader);
 
 NS_IMETHODIMP
 nsNativeComponentLoader::GetFactory(const nsIID & aCID,
@@ -102,10 +99,14 @@ nsNativeComponentLoader::GetFactory(const nsIID & aCID,
         return NS_ERROR_OUT_OF_MEMORY;
 
     if (!dll->IsLoaded()) {
+#ifdef PR_LOGGING
+        nsXPIDLCString displayPath;
+        dll->GetDisplayPath(displayPath);
+
         PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG,
                ("nsNativeComponentLoader: loading \"%s\"",
-                dll->GetDisplayPath()));
-
+                displayPath.get()));
+#endif
         if (!dll->Load()) {
 
             PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
@@ -129,13 +130,7 @@ nsNativeComponentLoader::GetFactory(const nsIID & aCID,
         return rv;      // XXX translate error code?
     
     rv = GetFactoryFromModule(dll, aCID, _retval);
-#ifdef OBSOLETE_MODULE_LOADING
-    if (NS_FAILED(rv)) {
-        if (rv == NS_ERROR_FACTORY_NOT_LOADED) {
-            rv = GetFactoryFromNSGetFactory(dll, aCID, serviceMgr, _retval);
-        }
-    }
-#endif
+
     PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
            ("nsNativeComponentLoader: Factory creation %s for %s",
             (NS_SUCCEEDED(rv) ? "succeeded" : "FAILED"),
@@ -161,11 +156,20 @@ nsNativeComponentLoader::Init(nsIComponentManager *aCompMgr, nsISupports *aReg)
     if (!mCompMgr)
         return NS_ERROR_INVALID_ARG;
 
-    mDllStore = new nsObjectHashtable(nsnull, nsnull, // never copy
-                                      nsDll_Destroy, nsnull,
-                                      256, /* Thead Safe */ PR_TRUE);
+    mDllStore = new nsObjectHashtable(nsnull, 
+                                      nsnull,
+                                      DLLStore_Destroy, 
+                                      nsnull,
+                                      256, 
+                                      PR_TRUE);
     if (!mDllStore)
         return NS_ERROR_OUT_OF_MEMORY;
+
+    mLoadedDependentLibs = new nsHashtable(16, PR_TRUE);
+
+    if (!mLoadedDependentLibs)
+        return NS_ERROR_OUT_OF_MEMORY;
+
     return NS_OK;
 }
 
@@ -277,25 +281,6 @@ nsFreeLibrary(nsDll *dll, nsIServiceManager *serviceMgr, PRInt32 when)
     {
         rv = mobj->CanUnload(nsComponentManagerImpl::gComponentManager, &canUnload);
     }
-#ifdef OBSOLETE_MODULE_LOADING
-    else
-    {
-        // Try the old method of module unloading
-        nsCanUnloadProc proc = (nsCanUnloadProc)dll->FindSymbol("NSCanUnload");
-        if (proc)
-        {
-            canUnload = proc(serviceMgr);
-            rv = NS_OK;    // No error status returned by call.
-        }
-        else
-        {
-            PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
-                   ("nsNativeComponentLoader: Unload cant get nsIModule or NSCanUnload for %s",
-                    dll->GetDisplayPath()));
-            return rv;
-        }
-    }
-#endif /* OBSOLETE_MODULE_LOADING */
 
     mobj = nsnull; // Release our reference to the module object
     // When shutting down, whether we can unload the dll or not,
@@ -308,9 +293,14 @@ nsFreeLibrary(nsDll *dll, nsIServiceManager *serviceMgr, PRInt32 when)
     // Check error status on CanUnload() call
     if (NS_FAILED(rv))
     {
+#ifdef PR_LOGGING
+        nsXPIDLCString displayPath;
+        dll->GetDisplayPath(displayPath);
+
         PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
                ("nsNativeComponentLoader: nsIModule::CanUnload() returned error for %s.",
-                dll->GetDisplayPath()));
+                displayPath.get()));
+#endif
         return rv;
     }
 
@@ -318,9 +308,15 @@ nsFreeLibrary(nsDll *dll, nsIServiceManager *serviceMgr, PRInt32 when)
     {
         if (dllMarkedForUnload)
         {
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
+
             PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG, 
-                   ("nsNativeComponentLoader: + Unloading \"%s\".", dll->GetDisplayPath()));
-#if 0
+                   ("nsNativeComponentLoader: + Unloading \"%s\".", displayPath.get()));
+#endif
+
+#ifdef DEBUG_dougt
             // XXX dlls aren't counting their outstanding instances correctly
             // XXX hence, dont unload until this gets enforced.
             rv = dll->Unload();
@@ -328,14 +324,24 @@ nsFreeLibrary(nsDll *dll, nsIServiceManager *serviceMgr, PRInt32 when)
         }
         else
         {
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
+
             PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG,
-                   ("nsNativeComponentLoader: Ready for unload \"%s\".", dll->GetDisplayPath()));
+                   ("nsNativeComponentLoader: Ready for unload \"%s\".", displayPath.get()));
+#endif
         }
     }
     else
     {
+#ifdef PR_LOGGING
+        nsXPIDLCString displayPath;
+        dll->GetDisplayPath(displayPath);
+
         PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS, 
-               ("nsNativeComponentLoader: NOT ready for unload %s", dll->GetDisplayPath()));
+               ("nsNativeComponentLoader: NOT ready for unload %s", displayPath.get()));
+#endif
         rv = NS_ERROR_FAILURE;
     }
     return rv;
@@ -390,8 +396,13 @@ nsNativeComponentLoader::SelfRegisterDll(nsDll *dll,
         return NS_ERROR_FAILURE;
     }
 
+#ifdef PR_LOGGING
+    nsXPIDLCString displayPath;
+    dll->GetDisplayPath(displayPath);
+    
     PR_LOG(nsComponentManagerLog, PR_LOG_DEBUG,
-           ("nsNativeComponentLoader: Loaded \"%s\".", dll->GetDisplayPath()));
+           ("nsNativeComponentLoader: Loaded \"%s\".", displayPath.get()));
+#endif
 
     // Tell the module to self register
     nsCOMPtr<nsIFile> fs;
@@ -409,33 +420,27 @@ nsNativeComponentLoader::SelfRegisterDll(nsDll *dll,
          *************************************************************/
         nsresult res2 = dll->GetDllSpec(getter_AddRefs(fs));    // don't change 'res2' -- see warning, above
         if (NS_SUCCEEDED(res2)) {
+            // in the case of re-registering a component, we want to remove 
+            // any optional data that this file may have had.
+            AddDependentLibrary(fs, nsnull);
+
             res = mobj->RegisterSelf(mCompMgr, fs, registryLocation,
                                      nativeComponentType);
         }
         else
         {
             res = res2;         // don't take this out -- see warning, above
+
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
             PR_LOG(nsComponentManagerLog, PR_LOG_ERROR, 
                    ("nsNativeComponentLoader: dll->GetDllSpec() on %s FAILED.",
-                    dll->GetDisplayPath()));
+                    displayPath.get()));
+#endif
         }
         mobj = NULL;    // Force a release of the Module object before unload()
     }
-#ifdef OBSOLETE_MODULE_LOADING
-    else
-    {
-        res = NS_ERROR_NO_INTERFACE;
-        nsRegisterProc regproc = (nsRegisterProc)dll->FindSymbol("NSRegisterSelf");
-        if (regproc)
-        {
-            // Call the NSRegisterSelfProc to enable dll registration
-            res = regproc(serviceMgr, registryLocation);
-            PR_LOG(nsComponentManagerLog, PR_LOG_ERROR, 
-                   ("nsNativeComponentLoader: %s using OBSOLETE NSRegisterSelf()",
-                    dll->GetDisplayPath()));
-        }
-    }
-#endif /* OBSOLETE_MODULE_LOADING */
 
     // Update the timestamp and size of the dll in registry
     // Don't enter deferred modules in the registry, because it might only be
@@ -523,24 +528,16 @@ nsNativeComponentLoader::DumpLoadError(nsDll *dll,
 #endif // MOZ_DEMANGLE_SYMBOLS
     
     // Do NSPR log
+#ifdef PR_LOGGING
+    nsXPIDLCString displayPath;
+    dll->GetDisplayPath(displayPath);
+
     PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
            ("nsNativeComponentLoader: %s(%s) Load FAILED with error:%s", 
             aCallerName,
-            dll->GetDisplayPath(), 
+            displayPath.get(), 
             errorMsg.get()));
-    
-
-    // Dump to screen if needed
-#ifdef PRINT_CRITICAL_ERROR_TO_SCREEN
-    fprintf(stderr, 
-           "**************************************************\n"
-           "nsNativeComponentLoader: %s(%s) Load FAILED with error: %s\n"
-           "**************************************************\n",
-           aCallerName,
-           dll->GetDisplayPath(), 
-           errorMsg.get());
 #endif
-
     return NS_OK;
 }
 
@@ -563,8 +560,13 @@ nsNativeComponentLoader::SelfUnregisterDll(nsDll *dll)
     res = dll->GetModule(mCompMgr, getter_AddRefs(mobj));
     if (NS_SUCCEEDED(res))
     {
+#ifdef PR_LOGGING
+        nsXPIDLCString displayPath;
+        dll->GetDisplayPath(displayPath);
+
         PR_LOG(nsComponentManagerLog, PR_LOG_ERROR, 
-               ("nsNativeComponentLoader: %s using nsIModule to unregister self.", dll->GetDisplayPath()));
+               ("nsNativeComponentLoader: %s using nsIModule to unregister self.", displayPath.get()));
+#endif
         nsCOMPtr<nsIFile> fs;
         res = dll->GetDllSpec(getter_AddRefs(fs));
         if (NS_FAILED(res)) return res;
@@ -580,19 +582,6 @@ nsNativeComponentLoader::SelfUnregisterDll(nsDll *dll)
         if (NS_FAILED(res)) return res;
         mobj->UnregisterSelf(mCompMgr, fs, registryName);
     }
-#ifdef OBSOLETE_MODULE_LOADING
-    else
-    {
-        res = NS_ERROR_NO_INTERFACE;
-        nsUnregisterProc unregproc =
-            (nsUnregisterProc) dll->FindSymbol("NSUnregisterSelf");
-        if (unregproc)
-        {
-            // Call the NSUnregisterSelfProc to enable dll de-registration
-            res = unregproc(serviceMgr, dll->GetPersistentDescriptorString());
-        }
-    }
-#endif /* OBSOLETE_MODULE_LOADING */
     return res;
 }
 
@@ -636,9 +625,14 @@ nsNativeComponentLoader::AutoUnregisterComponent(PRInt32 when,
 
     rv = SelfUnregisterDll(dll);
 
+#ifdef PR_LOGGING
+    nsXPIDLCString displayPath;
+    dll->GetDisplayPath(displayPath);
+
     PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
            ("nsNativeComponentLoader: AutoUnregistration for %s %s.",
-            (NS_FAILED(rv) ? "FAILED" : "succeeded"), dll->GetDisplayPath()));
+            (NS_FAILED(rv) ? "FAILED" : "succeeded"), displayPath.get()));
+#endif
 
     if (NS_FAILED(rv))
         return rv;
@@ -762,22 +756,18 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
 
     if (dll != NULL)
     {
-        // Make sure the dll is OK
-        if (dll->GetStatus() != NS_OK)
-        {
-            PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS, 
-                   ("nsNativeComponentLoader: + nsDll not NS_OK \"%s\". Skipping...",
-                    dll->GetDisplayPath()));
-            return NS_ERROR_FAILURE;
-        }
-            
         // We already have seen this dll. Check if this dll changed
         if (!dll->HasChanged())
         {
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
+            
             // Dll hasn't changed. Skip.
             PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS, 
                    ("nsNativeComponentLoader: + nsDll not changed \"%s\". Skipping...",
-                    dll->GetDisplayPath()));
+                    displayPath.get()));
+#endif
             *registered = PR_TRUE;
             return NS_OK;
         }
@@ -829,19 +819,28 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
                 // THIS IS THE WORST SITUATION TO BE IN.
                 // Dll doesn't want to be unloaded. Cannot re-register
                 // this dll.
+#ifdef PR_LOGGING
+                nsXPIDLCString displayPath;
+                dll->GetDisplayPath(displayPath);
+                
                 PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS,
                        ("nsNativeComponentLoader: *** Dll already loaded. "
                         "Cannot unload either. Hence cannot re-register "
-                        "\"%s\". Skipping...", dll->GetDisplayPath()));
+                        "\"%s\". Skipping...", displayPath.get()));
+#endif
                 return rv;
             }
             else {
                                 // dll doesn't have a CanUnload proc. Guess it is
                                 // ok to unload it.
                 dll->Unload();
+#ifdef PR_LOGGING
+                nsXPIDLCString displayPath;
+                dll->GetDisplayPath(displayPath);
                 PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS, 
                        ("nsNativeComponentLoader: + Unloading \"%s\". (no CanUnloadProc).",
-                        dll->GetDisplayPath()));
+                        displayPath.get()));
+#endif
             }
                 
         } // dll isloaded
@@ -852,9 +851,13 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
             // We went through all the above to make sure the dll
             // is unloaded. And here we are with the dll still
             // loaded. Whoever taught dp programming...
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
             PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS,
                    ("nsNativeComponentLoader: Dll still loaded. Cannot re-register "
-                    "\"%s\". Skipping...", dll->GetDisplayPath()));
+                    "\"%s\". Skipping...", displayPath.get()));
+#endif
             return NS_ERROR_FAILURE;
         }
     } // dll != NULL
@@ -864,7 +867,7 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
         // It is ok to do this even if the creation of nsDll
         // didnt succeed. That way we wont do this again
         // when we encounter the same dll.
-        dll = new nsDll(persistentDescriptor);
+        dll = new nsDll(component, this);
         if (dll == NULL)
             return NS_ERROR_OUT_OF_MEMORY;
         mDllStore->Put(&key, (void *) dll);
@@ -883,17 +886,27 @@ nsNativeComponentLoader::AutoRegisterComponent(PRInt32 when,
             *registered = PR_TRUE;
             return NS_OK;
         } else {
+#ifdef PR_LOGGING
+            nsXPIDLCString displayPath;
+            dll->GetDisplayPath(displayPath);
+
             PR_LOG(nsComponentManagerLog, PR_LOG_ERROR,
                    ("nsNativeComponentLoader: Autoregistration FAILED for "
-                    "\"%s\". Skipping...", dll->GetDisplayPath()));
+                    "\"%s\". Skipping...", displayPath.get()));
+#endif
             return NS_ERROR_FACTORY_NOT_REGISTERED;
         }
     }
     else
     {
+#ifdef PR_LOGGING
+        nsXPIDLCString displayPath;
+        dll->GetDisplayPath(displayPath);
+
         PR_LOG(nsComponentManagerLog, PR_LOG_ALWAYS,
                ("nsNativeComponentLoader: Autoregistration Passed for "
-                "\"%s\".", dll->GetDisplayPath()));
+                "\"%s\".", displayPath.get()));
+#endif
         // Marking dll along with modified time and size in the
         // registry happens at PlatformRegister(). No need to do it
         // here again.
@@ -917,7 +930,7 @@ nsNativeComponentLoader::RegisterDeferredComponents(PRInt32 aWhen,
     for (int i = mDeferredComponents.Count() - 1; i >= 0; i--) {
         nsDll *dll = NS_STATIC_CAST(nsDll *, mDeferredComponents[i]);
         nsresult rv = SelfRegisterDll(dll,
-                                      dll->GetRegistryLocation(),
+                                      nsnull,
                                       PR_TRUE);
         if (rv != NS_ERROR_FACTORY_REGISTER_AGAIN) {
             if (NS_SUCCEEDED(rv))
@@ -940,8 +953,10 @@ nsNativeComponentLoader::RegisterDeferredComponents(PRInt32 aWhen,
 nsresult
 nsNativeComponentLoader::OnRegister(const nsIID &aCID, const char *aType,
                                     const char *aClassName,
-                                    const char *aContractID, const char *aLocation,
-                                    PRBool aReplace, PRBool aPersist)
+                                    const char *aContractID, 
+                                    const char *aLocation,
+                                    PRBool aReplace, 
+                                    PRBool aPersist)
 {
     return NS_OK;
 }
@@ -1000,19 +1015,14 @@ nsNativeComponentLoader::CreateDll(nsIFile *aSpec,
 
     if (!aSpec)
     {
-        if (!nsCRT::strncmp(aLocation, XPCOM_LIB_PREFIX, 4)) {
-            dll = new nsDll(aLocation+4, 1 /* dumb magic flag */);
-            if (!dll) return NS_ERROR_OUT_OF_MEMORY;
-        } else {
-            // what I want to do here is QI for a Component Registration Manager.  Since this 
-            // has not been invented yet, QI to the obsolete manager.  Kids, don't do this at home.
-            nsCOMPtr<nsIComponentManagerObsolete> obsoleteManager = do_QueryInterface(mCompMgr, &rv);
-            if (obsoleteManager)
-                rv = obsoleteManager->SpecForRegistryLocation(aLocation,
-                                                              getter_AddRefs(spec));
-            if (NS_FAILED(rv))
-                return rv;
-        }
+        // what I want to do here is QI for a Component Registration Manager.  Since this 
+        // has not been invented yet, QI to the obsolete manager.  Kids, don't do this at home.
+        nsCOMPtr<nsIComponentManagerObsolete> obsoleteManager = do_QueryInterface(mCompMgr, &rv);
+        if (obsoleteManager)
+            rv = obsoleteManager->SpecForRegistryLocation(aLocation,
+                                                          getter_AddRefs(spec));
+        if (NS_FAILED(rv))
+            return rv;
     }
     else
     {
@@ -1021,7 +1031,7 @@ nsNativeComponentLoader::CreateDll(nsIFile *aSpec,
 
     if (!dll)
     {
-        dll = new nsDll(spec, aLocation);
+        dll = new nsDll(spec, this);
         if (!dll)
             return NS_ERROR_OUT_OF_MEMORY;
     }
@@ -1047,11 +1057,35 @@ nsNativeComponentLoader::GetFactoryFromModule(nsDll *aDll, const nsCID &aCID,
                                   (void **)aFactory);
 }
 
-nsresult
-nsNativeComponentLoader::GetFactoryFromNSGetFactory(nsDll *aDll,
-                                                    const nsCID &aCID,
-                                                    nsIServiceManager *aServMgr,
-                                                    nsIFactory **aFactory)
+
+NS_IMETHODIMP
+nsNativeComponentLoader::AddDependentLibrary(nsIFile* aFile, const char* libName)
 {
-    return NS_ERROR_FACTORY_NOT_LOADED;
+    nsCOMPtr<nsIComponentLoaderManager> manager = do_QueryInterface(mCompMgr);
+    if (!manager) 
+    {
+        NS_WARNING("Something is terribly wrong");
+        return NS_ERROR_FAILURE;
+    }
+
+    // the native component loader uses the optional data
+    // to store a space delimited list of dependent library 
+    // names
+
+    if (!libName) 
+    {
+        manager->SetOptionalData(aFile, nsnull, nsnull);
+        return NS_OK;
+    }
+
+    nsXPIDLCString data;
+    manager->GetOptionalData(aFile, nsnull, getter_Copies(data));
+
+    if (!data.IsEmpty())
+        data.Append(NS_LITERAL_CSTRING(" "));
+
+    data.Append(nsDependentCString(libName));
+    
+    manager->SetOptionalData(aFile, nsnull, data);
+    return NS_OK;
 }
