@@ -57,9 +57,11 @@ public:
   NS_DECL_ISUPPORTS
 
   NS_IMETHOD Read(PRUnichar* aBuf,
-                  PRUint32 aOffset,
                   PRUint32 aCount,
                   PRUint32 *aReadCount);
+  NS_IMETHOD ReadSegments(nsWriteUnicharSegmentFun aWriter,
+                          void* aClosure,
+                          PRUint32 aCount, PRUint32* aReadCount);
   NS_IMETHOD Close();
 
   nsString* mString;
@@ -69,7 +71,6 @@ public:
 
 StringUnicharInputStream::StringUnicharInputStream(nsString* aString)
 {
-  NS_INIT_ISUPPORTS();
   mString = aString;
   mPos = 0;
   mLen = aString->Length();
@@ -82,10 +83,10 @@ StringUnicharInputStream::~StringUnicharInputStream()
   }
 }
 
-nsresult StringUnicharInputStream::Read(PRUnichar* aBuf,
-                                        PRUint32 aOffset,
-                                        PRUint32 aCount,
-                                        PRUint32 *aReadCount)
+NS_IMETHODIMP
+StringUnicharInputStream::Read(PRUnichar* aBuf,
+                               PRUint32 aCount,
+                               PRUint32 *aReadCount)
 {
   if (mPos >= mLen) {
     *aReadCount = 0;
@@ -103,6 +104,36 @@ nsresult StringUnicharInputStream::Read(PRUnichar* aBuf,
   return NS_OK;
 }
 
+NS_IMETHODIMP
+StringUnicharInputStream::ReadSegments(nsWriteUnicharSegmentFun aWriter,
+                                       void* aClosure,
+                                       PRUint32 aCount, PRUint32 *aReadCount)
+{
+  PRUint32 bytesWritten;
+  PRUint32 totalBytesWritten = 0;
+
+  nsresult rv;
+  aCount = PR_MIN(mString->Length() - mPos, aCount);
+  
+  while (aCount) {
+    rv = aWriter(this, aClosure, mString->get() + mPos,
+                 totalBytesWritten, aCount, &bytesWritten);
+    
+    if (NS_FAILED(rv)) {
+      // don't propagate errors to the caller
+      break;
+    }
+    
+    aCount -= bytesWritten;
+    totalBytesWritten += bytesWritten;
+    mPos += bytesWritten;
+  }
+  
+  *aReadCount = totalBytesWritten;
+  
+  return NS_OK;
+}
+
 nsresult StringUnicharInputStream::Close()
 {
   mPos = mLen;
@@ -144,9 +175,12 @@ public:
 
   NS_DECL_ISUPPORTS
   NS_IMETHOD Read(PRUnichar* aBuf,
-                  PRUint32 aOffset,
                   PRUint32 aCount,
                   PRUint32 *aReadCount);
+  NS_IMETHOD ReadSegments(nsWriteUnicharSegmentFun aWriter,
+                          void* aClosure,
+                          PRUint32 aCount,
+                          PRUint32 *aReadCount);
   NS_IMETHOD Close();
 
 protected:
@@ -168,7 +202,6 @@ UTF8InputStream::UTF8InputStream() :
   mUnicharDataOffset(0),
   mUnicharDataLength(0)
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsresult 
@@ -205,7 +238,6 @@ nsresult UTF8InputStream::Close()
 }
 
 nsresult UTF8InputStream::Read(PRUnichar* aBuf,
-                               PRUint32 aOffset,
                                PRUint32 aCount,
                                PRUint32 *aReadCount)
 {
@@ -223,13 +255,56 @@ nsresult UTF8InputStream::Read(PRUnichar* aBuf,
   if (rv > aCount) {
     rv = aCount;
   }
-  memcpy(aBuf + aOffset, mUnicharData->GetBuffer() + mUnicharDataOffset,
+  memcpy(aBuf, mUnicharData->GetBuffer() + mUnicharDataOffset,
          rv * sizeof(PRUnichar));
   mUnicharDataOffset += rv;
   *aReadCount = rv;
   return NS_OK;
 }
 
+NS_IMETHODIMP
+UTF8InputStream::ReadSegments(nsWriteUnicharSegmentFun aWriter,
+                              void* aClosure,
+                              PRUint32 aCount, PRUint32 *aReadCount)
+{
+  NS_ASSERTION(mUnicharDataLength >= mUnicharDataOffset, "unsigned madness");
+  PRUint32 bytesToWrite = mUnicharDataLength - mUnicharDataOffset;
+  nsresult rv = NS_OK;
+  if (0 == bytesToWrite) {
+    // Fill the unichar buffer
+    bytesToWrite = Fill(&rv);
+    if (bytesToWrite <= 0) {
+      *aReadCount = 0;
+      return rv;
+    }
+  }
+  
+  if (bytesToWrite > aCount)
+    bytesToWrite = aCount;
+  
+  PRUint32 bytesWritten;
+  PRUint32 totalBytesWritten = 0;
+
+  while (bytesToWrite) {
+    rv = aWriter(this, aClosure,
+                 mUnicharData->GetBuffer() + mUnicharDataOffset,
+                 totalBytesWritten, bytesToWrite, &bytesWritten);
+
+    if (NS_FAILED(rv)) {
+      // don't propagate errors to the caller
+      break;
+    }
+    
+    bytesToWrite -= bytesWritten;
+    totalBytesWritten += bytesWritten;
+    mUnicharDataOffset += bytesWritten;
+  }
+
+  *aReadCount = totalBytesWritten;
+  
+  return NS_OK;
+}
+
 PRInt32 UTF8InputStream::Fill(nsresult * aErrorCode)
 {
   if (nsnull == mInput) {
